@@ -42,6 +42,10 @@
 typedef pfm_dfl_smpl_hdr_t	btb_hdr_t;
 typedef pfm_dfl_smpl_entry_t	btb_entry_t;
 typedef pfm_dfl_smpl_arg_t	smpl_arg_t;
+#define BTB_FMT_UUID	       	PFM_DFL_SMPL_UUID
+
+static pfm_uuid_t buf_fmt_id = BTB_FMT_UUID;
+
 
 #define NUM_PMCS PFMLIB_MAX_PMCS
 #define NUM_PMDS PFMLIB_MAX_PMDS
@@ -271,7 +275,7 @@ main(void)
 	pfarg_load_t load_args;
 	pfmlib_options_t pfmlib_options;
 	struct sigaction act;
-	unsigned int i;
+	unsigned int i, j;
 
 	/*
 	 * Initialize pfm library (required before we can use it)
@@ -303,7 +307,7 @@ main(void)
 	 */
 	memset(&pfmlib_options, 0, sizeof(pfmlib_options));
 	pfmlib_options.pfm_debug = 0; /* set to 1 for debug */
-	pfmlib_options.pfm_verbose = 1; /* set to 1 for debug */
+	pfmlib_options.pfm_verbose = 0; /* set to 1 for debug */
 	pfm_set_options(&pfmlib_options);
 
 	memset(pd, 0, sizeof(pd));
@@ -332,7 +336,7 @@ main(void)
 	ita2_inp.pfp_ita2_btb.btb_brt = 0x0;
 	ita2_inp.pfp_ita2_btb.btb_plm = PFM_PLM3;
 
-	if (pfm_find_full_event("BRANCH_EVENT", &inp.pfp_events[0]) != PFMLIB_SUCCESS) {
+	if (pfm_find_event_byname("BRANCH_EVENT", &inp.pfp_events[0].event) != PFMLIB_SUCCESS) {
 		fatal_error("cannot find event BRANCH_EVENT\n");
 	}
 
@@ -352,6 +356,13 @@ main(void)
 	if ((ret=pfm_dispatch_events(&inp, &ita2_inp, &outp, NULL)) != PFMLIB_SUCCESS) {
 		fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
 	}
+	 /*
+	  * We initialize the format specific information.
+	  * The format is identified by its UUID which must be copied
+	  * into the ctx_buf_fmt_id field.
+	  */
+	memcpy(ctx.ctx_smpl_buf_id, buf_fmt_id, sizeof(pfm_uuid_t));
+
 	/*
 	 * the size of the buffer is indicated in bytes (not entries).
 	 *
@@ -364,21 +375,26 @@ main(void)
 	/*
 	 * now create the context for self monitoring/per-task
 	 */
-	id = pfm_create_context(&ctx, "default", &buf_arg, sizeof(buf_arg));
-	if (id == -1) {
+	if (pfm_create_context(&ctx, &buf_arg, sizeof(buf_arg)) == -1 ) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
 
+	/*
+	 * extract our file descriptor
+	 */
+	id = ctx.ctx_fd;
+
 	/*
 	 * retrieve the virtual address at which the sampling
 	 * buffer has been mapped
 	 */
-	smpl_vaddr = mmap(NULL, (size_t)buf_arg.buf_size, PROT_READ, MAP_PRIVATE, id, 0);
-	if (smpl_vaddr == MAP_FAILED)
+	smpl_vaddr = mmap(NULL, (size_t)ctx.ctx_smpl_buf_size, PROT_READ, MAP_PRIVATE, id, 0);
+	if (smpl_vaddr == MAP_FAILED) {
 		fatal_error("cannot mmap sampling buffer errno %d\n", errno);
+	}
 
 	printf("Sampling buffer mapped at %p\n", smpl_vaddr);
 
@@ -399,11 +415,11 @@ main(void)
 
 	/*
 	 * figure out pmd mapping from output pmc
-	 * PMD16 is part of the set of used PMD returned by libpfm.
-	 * It will be reset automatically
 	 */
-	for (i=0; i < outp.pfp_pmd_count; i++)
-		pd[i].reg_num   = outp.pfp_pmds[i].reg_num;
+	for (i=0, j=0; i < inp.pfp_event_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
+		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
+	}
 
 	/*
 	 * indicate we want notification when buffer is full
@@ -435,13 +451,24 @@ main(void)
 	 */
 	pfm_bv_set(pd[0].reg_reset_pmds, 16);
 
-		/*
+	/*
+	 * reset pmd16 (BTB index), short and long reset value are set to zero as well
+	 *
+	 * We use slot 1 of our pd[] array for this.
+	 */
+	pd[1].reg_num         = 16;
+	pd[1].reg_value       = 0;
+
+	/*
 	 * Now program the registers
 	 */
 	if (pfm_write_pmcs(id, pc, outp.pfp_pmc_count) == -1)
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
 
-	if (pfm_write_pmds(id, pd, outp.pfp_pmd_count) == -1)
+	/*
+	 * we use 2 = 1 for the branch_event + 1 for the reset of PMD16.
+	 */
+	if (pfm_write_pmds(id, pd, 2) == -1)
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
 
 	/*
@@ -485,7 +512,6 @@ main(void)
 	/*
 	 * let's stop this now
 	 */
-	munmap(smpl_vaddr, (size_t)buf_arg.buf_size);
 	close(id);
 
 	return 0;
