@@ -138,7 +138,7 @@ main(int argc, char **argv)
 		unsigned long gp;
 	} *fd;
 	int ret, type = 0;
-	unsigned int i;
+	unsigned int i, j;
 	int id;
 	char name[MAX_EVT_NAME_LEN];
 
@@ -263,13 +263,17 @@ main(int argc, char **argv)
 	/*
 	 * now create the context for self monitoring/per-task
 	 */
-	id = pfm_create_context(ctx, NULL, NULL, 0);
-	if (id == -1) {
+	if (pfm_create_context(ctx, NULL, 0) == -1) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("cannot create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract our file descriptor
+	 */
+	id = ctx[0].ctx_fd;
+
 	/*
 	 * Now prepare the argument to initialize the PMDs and PMCS.
 	 * We must pfp_pmc_count to determine the number of PMC to intialize.
@@ -284,8 +288,12 @@ main(int argc, char **argv)
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
-	for (i=0; i < outp.pfp_pmd_count; i++) {
-		pd[i].reg_num   = outp.pfp_pmds[i].reg_num;
+	/*
+	 * figure out pmd mapping from output pmc
+	 */
+	for (i=0, j=0; i < inp.pfp_event_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
+		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
 	}
 	
 	/*
@@ -304,8 +312,9 @@ main(int argc, char **argv)
 	 * IMPORTANT: programming the debug register MUST always be done before the PMCs
 	 * otherwise the kernel will fail on PFM_WRITE_PMCS. This is for security reasons.
 	 */
-	if (pfm_write_pmcs(id, ibrs, ita_outp.pfp_ita_irange.rr_nbr_used) == -1)
+	if (pfm_write_pmcs(id, ibrs, ita_outp.pfp_ita_irange.rr_nbr_used) == -1) {
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
+	}
 
 	/*
 	 * Now program the registers
@@ -314,11 +323,13 @@ main(int argc, char **argv)
 	 * the kernel because, as we said earlier, pc may contain more elements than
 	 * the number of events we specified, i.e., contains more than coutning monitors.
 	 */
-	if (pfm_write_pmcs(id, pc, outp.pfp_pmc_count) == -1)
+	if (pfm_write_pmcs(id, pc, outp.pfp_pmc_count) == -1) {
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
+	}
 
-	if (pfm_write_pmds(id, pd, outp.pfp_pmd_count) == -1)
+	if (pfm_write_pmds(id, pd, inp.pfp_event_count) == -1) {
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
+	}
 
 	/*
 	 * now we load (i.e., attach) the context to ourself
@@ -358,8 +369,8 @@ main(int argc, char **argv)
 	 * we need to use the pc[] array to figure out where event i was allocated.
 	 */
 	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_full_event_name(&inp.pfp_events[i], name, MAX_EVT_NAME_LEN);
-		printf("PMD%-3u %20lu %s (expected %lu)\n",
+		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
+		printf("PMD%u %20lu %s (expected %lu)\n",
 			pd[i].reg_num,
 			pd[i].reg_value,
 			name,
