@@ -456,14 +456,17 @@ measure_one_task(char **argv)
 	/*
 	 * create the context
 	 */
-
-	ctxid = pfm_create_context(ctx, NULL, NULL, 0);
-	if (ctxid == -1 ) {
+	if (pfm_create_context(ctx, NULL, 0) == -1 ) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract context id
+	 */
+	ctxid = ctx[0].ctx_fd;
+
 	/*
 	 * set close-on-exec to ensure we will be getting the PFM_END_MSG, i.e.,
 	 * fd not visible to child.
@@ -624,13 +627,16 @@ measure_one_cpu(char **argv)
 	/*
 	 * create the context
 	 */
-	ctxid = pfm_create_context(ctx, NULL, NULL, 0);
-	if (ctxid == -1) {
+	if (pfm_create_context(ctx, NULL, 0) == -1 ) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract context id
+	 */
+	ctxid = ctx[0].ctx_fd;
 
 	/*
 	 * set close-on-exec to ensure we will be getting the PFM_END_MSG, i.e.,
@@ -734,10 +740,9 @@ mainloop(char **argv)
 	pfmlib_input_param_t inp;
 	pfmlib_output_param_t outp;
 	pfmlib_regmask_t impl_counters, used_pmcs;
-	pfmlib_event_t cycle_event;
-	unsigned int i, j;
+	unsigned int i, j, k;
 	char *p, *str;
-	unsigned int max_counters, allowed_counters;
+	unsigned int max_counters, allowed_counters, cycles_event_idx;
 	int ret;
 
 	pfm_get_num_counters(&max_counters);
@@ -777,7 +782,7 @@ mainloop(char **argv)
 		/*
 		 * look for our trigger event
 		 */
-		if (pfm_get_cycle_event(&cycle_event) != PFMLIB_SUCCESS) {
+		if (pfm_get_cycle_event(&cycles_event_idx) != PFMLIB_SUCCESS) {
 			fatal_error("Cannot find cycle event\n");
 		}
 	}
@@ -819,7 +824,7 @@ mainloop(char **argv)
 			p = strchr(str, ',');
 			if (p) *p = '\0';
 
-			if (pfm_find_full_event(str, &inp.pfp_events[j]) != PFMLIB_SUCCESS) {
+			if (pfm_find_event(str, &inp.pfp_events[j].event) != PFMLIB_SUCCESS) {
 				fatal_error("Cannot find %s event for set %d event %d\n", str, i, j);
 			}
 			if (p) {
@@ -837,15 +842,15 @@ mainloop(char **argv)
 		 * add the cycle event as the last event when we switch on overflow
 		 */
 		if (options.opt_ovfl_switch) {
-			inp.pfp_events[j]   = cycle_event;
-			inp.pfp_event_count = j+1;
-			e->n_events	    = j+1;
+			inp.pfp_events[j].event = cycles_event_idx;
+			inp.pfp_event_count     = j+1;
+			e->n_events		= j+1;
 		} else {
-			e->n_events         = j;
-			inp.pfp_event_count = j;
+			e->n_events             = j;
+			inp.pfp_event_count     = j;
 		}
 
-		inp.pfp_dfl_plm = options.opt_plm;
+		inp.pfp_dfl_plm         = options.opt_plm;
 
 		if (options.opt_is_system) 
 			inp.pfp_flags = PFMLIB_PFP_SYSTEMWIDE;
@@ -869,12 +874,14 @@ mainloop(char **argv)
 		for (j=0; j < outp.pfp_pmc_count; j++, num_pmcs++) {
 			all_pmcs[num_pmcs].reg_num   = outp.pfp_pmcs[j].reg_num;
 			all_pmcs[num_pmcs].reg_value = outp.pfp_pmcs[j].reg_value;
-		}
-		for (j=0; j < outp.pfp_pmd_count; j++, num_pmds++)
-			all_pmds[num_pmds].reg_num = outp.pfp_pmds[j].reg_num;
 
-		e->npmcs  = num_pmcs - e->pmcs_base;
-		e->npmds  = num_pmds - e->pmds_base;
+		}
+		e->npmcs  = j;
+		for (j=0, k=0; j < inp.pfp_event_count; j++, num_pmds++) {
+			all_pmds[num_pmds].reg_num   = outp.pfp_pmcs[k].reg_pmd_num;;
+			for(; k < outp.pfp_pmc_count; k++)  if (outp.pfp_pmcs[k].reg_evt_idx != j) break;
+		}
+		e->npmds  = j;
 
 		if (options.opt_ovfl_switch) {
 			/*
@@ -929,17 +936,17 @@ static void
 generate_default_sets(void)
 {
 	event_set_t *es, *tail = NULL;
-	pfmlib_event_t events[2];
+	unsigned int idx[2];
 	size_t len;
 	char *name;
 	unsigned int i;
 	int ret;
 	
-	ret = pfm_get_cycle_event(&events[0]);
+	ret = pfm_get_cycle_event(idx);
 	if (ret != PFMLIB_SUCCESS)
 		fatal_error("cannot find cycle event\n");
 
-	ret = pfm_get_inst_retired_event(&events[1]);
+	ret = pfm_get_inst_retired_event(idx+1);
 	if (ret != PFMLIB_SUCCESS)
 		fatal_error("cannot find instruction retired event\n");
 
@@ -950,7 +957,7 @@ generate_default_sets(void)
 		if (name == NULL) {
 			fatal_error("cannot allocate space for event name\n");
 		}
-		pfm_get_full_event_name(&events[i], name, len+1);
+		pfm_get_event_name(idx[i], name, len+1);
 
 		es = (event_set_t *)malloc(sizeof(event_set_t));
 		if (es == NULL)
