@@ -50,7 +50,11 @@
 
 #define SMPL_FREQ_IN_HZ	300
 
-#define NUM_PMCS 256
+#define NUM_PMCS PMU_MAX_PMCS
+#define NUM_PMDS PMU_MAX_PMDS
+
+#define MAX_NUM_COUNTERS	NUM_PMDS
+#define MAX_PMU_NAME_LEN	32
 
 typedef struct {
 	struct {
@@ -254,7 +258,7 @@ dec2sep(char *str2, char *str, char sep)
 }
 
 static void
-print_results(int ctxid, uint64_t *eff_timeout)
+print_results(int ctxid, uint32_t *eff_timeout)
 {
 	unsigned int i, j, cnt, ovfl_event;
 	uint64_t value, tot_runs = 0, set_runs;
@@ -283,8 +287,9 @@ print_results(int ctxid, uint64_t *eff_timeout)
 	 * is usally at page size (16KB)
 	 */
 	ret = pfm_read_pmds(ctxid, all_pmds, num_pmds);
-	if (ret == -1)
+	if (ret == -1) {
 		fatal_error("cannot read pmds: %s\n", strerror(errno));
+	}
 
 	/*
 	 * extract all set information
@@ -294,8 +299,9 @@ print_results(int ctxid, uint64_t *eff_timeout)
 	 * is usually at page size (16KB)
 	 */
 	ret = pfm_getinfo_evtsets(ctxid, all_setinfos, num_sets);
-	if (ret == -1)
+	if (ret == -1) {
 		fatal_error("cannot get set info: %s\n", strerror(errno));
+	}
 
 	/*
 	 * compute average number of runs
@@ -318,15 +324,15 @@ print_results(int ctxid, uint64_t *eff_timeout)
 	 *
 	 */
 	if (options.opt_no_header == 0) {
-		printf("# %u Hz period = %u nsecs\n# %"PRIu64" cycles @ %lu MHz\n", 
+		printf("# %u Hz period = %u usecs\n# %"PRIu64" cycles @ %lu MHz\n", 
 			options.smpl_freq, 
-			1000000000 / options.smpl_freq, 
+			1000000 / options.smpl_freq, 
 			options.smpl_period,
 			options.cpu_mhz);
 
 		if (options.opt_ovfl_switch == 0)
 			printf("# using time-based multiplexing\n"
-				"# %"PRIu64" nsecs effective switch timeout\n", 
+				"# %u usecs effective switch timeout\n", 
 				*eff_timeout);
 		else
 			printf("# using overflow-based multiplexing\n");
@@ -406,13 +412,13 @@ measure_one_task(char **argv)
 	pfarg_pmc_t *my_pmcs;
 	pfarg_pmd_t *my_pmds;
 	pfarg_load_t load_arg;
-	uint64_t eff_timeout;
+	uint32_t eff_timeout;
 	pfm_msg_t msg;
 	pid_t pid;
 	int status, ret;
 
-	my_pmcs = malloc(sizeof(pfarg_pmc_t)*num_pmcs);
-	my_pmds = malloc(sizeof(pfarg_pmd_t)*num_pmds);
+	my_pmcs = malloc(sizeof(pfarg_pmc_t)*total_events);
+	my_pmds = malloc(sizeof(pfarg_pmd_t)*total_events);
 	my_sets = malloc(sizeof(pfarg_setdesc_t)*num_sets);
 
 	if (my_pmcs == NULL || my_pmds == NULL || my_sets == NULL)
@@ -430,13 +436,17 @@ measure_one_task(char **argv)
 	/*
 	 * create the context
 	 */
-	ctxid = pfm_create_context(ctx, NULL, NULL, 0);
-	if (ctxid == -1 ) {
+	if (pfm_create_context(ctx, NULL, 0) == -1 ) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract context id
+	 */
+	ctxid = ctx[0].ctx_fd;
+
 	/*
 	 * set close-on-exec to ensure we will be getting the PFM_END_MSG, i.e.,
 	 * fd not visible to child.
@@ -451,22 +461,23 @@ measure_one_task(char **argv)
 	 * reason. However to avoid special casing set0 for creation, a PFM_CREATE_EVTSETS
 	 * for set0 does not complain and behaves as a PFM_CHANGE_EVTSETS
 	 */
-	vbprintf("requested timeout %"PRIu64" nsecs\n", my_sets[0].set_timeout);
+	vbprintf("requested timeout %u usecs\n", my_sets[0].set_timeout);
 
-	if (pfm_create_evtsets(ctxid, my_sets, num_sets))
+	if (pfm_create_evtsets(ctxid, my_sets, num_sets)) {
 		fatal_error("cannot create sets\n");
-
+	}
 	eff_timeout = my_sets[0].set_timeout;
 
-	vbprintf("effective timeout %"PRIu64" nsecs\n", my_sets[0].set_timeout);
+	vbprintf("effective timeout %u usecs\n", my_sets[0].set_timeout);
 	/*
 	 * Now program the all the registers in one call
 	 *
 	 * Note that there is a limitation on the size of the argument vector
 	 * that can be passed. It is usually set to a page size (16KB).
 	 */
-	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1)
+	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1) {
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
+	}
 
 	/*
 	 * initialize the PMD registers.
@@ -474,8 +485,9 @@ measure_one_task(char **argv)
 	 * To be read, each PMD must be either written or declared
 	 * as being part of a sample (reg_smpl_pmds)
 	 */
-	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1)
+	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1) {
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
+	}
 
 	/*
 	 * now launch the child code
@@ -501,14 +513,16 @@ measure_one_task(char **argv)
 	 * now attach the context
 	 */
 	load_arg.load_pid = pid;
-	if (pfm_load_context(ctxid, &load_arg) == -1)
+	if (pfm_load_context(ctxid, &load_arg) == -1) {
 		fatal_error("pfm_load_context error errno %d\n",errno);
+	}
 
 	/*
 	 * start monitoring
 	 */
-	if (pfm_start(ctxid, NULL) == -1)
+	if (pfm_start(ctxid, NULL) == -1) {
 		fatal_error("pfm_start error errno %d\n",errno);
+	}
 
 	ptrace(PTRACE_DETACH, pid, NULL, 0);
 	vbprintf("child restarted\n");
@@ -532,9 +546,9 @@ measure_one_task(char **argv)
 	 * mainloop
 	 */
 	ret = read(ctxid, &msg, sizeof(msg));
-	if (ret < sizeof(msg))
+	if (ret < sizeof(msg)) {
 		fatal_error("interrupted read\n");
-
+	}
 	switch(msg.type) {
 		case PFM_MSG_OVFL:
 			fatal_error("unexpected ovfl message\n");
@@ -564,8 +578,7 @@ finish_line:
 		ptrace(PTRACE_DETACH, pid, NULL, 0);
 	}
 
-	if (time_to_quit < 2)
-		print_results(ctxid, &eff_timeout);
+	if (time_to_quit < 2) print_results(ctxid, &eff_timeout);
 
 	close(ctxid);
 
@@ -611,13 +624,17 @@ measure_one_cpu(char **argv)
 	/*
 	 * create the context
 	 */
-	ctxid = pfm_create_context(ctx, NULL, NULL, 0);
-	if (ctxid == -1) {
+	if (pfm_create_context(ctx, NULL, 0) == -1 ) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract context id
+	 */
+	ctxid = ctx[0].ctx_fd;
+
 	/*
 	 * set close-on-exec to ensure we will be getting the PFM_END_MSG, i.e.,
 	 * fd not visible to child.
@@ -632,8 +649,9 @@ measure_one_cpu(char **argv)
 	 * reason. However to avoid special casing set0 for creation, a PFM_CREATE_EVTSETS
 	 * for set0 does not complain and behaves as a PFM_CHANGE_EVTSETS
 	 */
-	if (pfm_create_evtsets(ctxid, my_sets, num_sets))
+	if (pfm_create_evtsets(ctxid, my_sets, num_sets)) {
 		fatal_error("cannot create sets\n");
+	}
 
 	/*
 	 * Now program the all the registers in one call
@@ -641,8 +659,9 @@ measure_one_cpu(char **argv)
 	 * Note that there is a limitation on the size of the argument vector
 	 * that can be passed. It is usually set to a page size (16KB).
 	 */
-	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1)
+	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1) {
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
+	}
 
 	/*
 	 * initialize the PMD registers.
@@ -650,8 +669,9 @@ measure_one_cpu(char **argv)
 	 * To be read, each PMD must be either written or declared
 	 * as being part of a sample (reg_smpl_pmds)
 	 */
-	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1)
+	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1) {
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
+	}
 
 	/*
 	 * now launch the child code
@@ -679,14 +699,16 @@ measure_one_cpu(char **argv)
 	 * now attach the context
 	 */
 	load_arg.load_pid = options.opt_is_system ? getpid() : pid;
-	if (pfm_load_context(ctxid, &load_arg) == -1)
+	if (pfm_load_context(ctxid, &load_arg) == -1) {
 		fatal_error("pfm_load_context error errno %d\n",errno);
+	}
 
 	/*
 	 * start monitoring
 	 */
-	if (pfm_start(ctxid, NULL) == -1)
+	if (pfm_start(ctxid, NULL) == -1) {
 		fatal_error("pfm_start error errno %d\n",errno);
+	}
 
 	if (pid) ptrace(PTRACE_DETACH, pid, NULL, 0);
 
@@ -715,11 +737,10 @@ mainloop(char **argv)
 	pfmlib_input_param_t inp;
 	pfmlib_output_param_t outp;
 	pfmlib_regmask_t impl_counters, used_pmcs;
-	pfmlib_event_t cycle_event;
-	unsigned int i, j;
+	unsigned int i, j, k;
 	char *p, *str;
 	int ret;
-	unsigned int max_counters, allowed_counters;
+	unsigned int max_counters, allowed_counters, cycles_event_idx;
 
 	pfm_get_num_counters(&max_counters);
 
@@ -758,23 +779,25 @@ mainloop(char **argv)
 		/*
 		 * look for our trigger event
 		 */
-		if (pfm_get_cycle_event(&cycle_event) != PFMLIB_SUCCESS)
+		if (pfm_get_cycle_event(&cycles_event_idx) != PFMLIB_SUCCESS) {
 			fatal_error("Cannot find cycle event\n");
+		}
 	}
 
 	vbprintf("total_events=%u\n", total_events);
 
-	/*
-	 * assumes number of pmds = number  of events
-	 * cannot assume number of pmcs = num of events (e.g., P4 2 PMCS per event)
-	 */
-	all_pmcs = calloc(NUM_PMCS, sizeof(pfarg_pmc_t));
-	all_pmds = calloc(total_events, sizeof(pfarg_pmd_t));
-	all_sets = calloc(num_sets, sizeof(pfarg_setdesc_t));
+	all_pmcs = malloc(sizeof(pfarg_pmc_t)*total_events);
+	all_pmds = malloc(sizeof(pfarg_pmd_t)*total_events);
+	all_sets = malloc(sizeof(pfarg_setdesc_t)*num_sets);
+
+	memset(all_pmcs, 0, sizeof(pfarg_pmc_t)*total_events);
+	memset(all_pmds, 0, sizeof(pfarg_pmd_t)*total_events);
+	memset(all_sets, 0, sizeof(pfarg_setdesc_t)*num_sets);
 
 	if (all_pmcs == NULL || all_pmds == NULL || all_sets == NULL)
 		fatal_error("cannot allocate event tables\n");
 
+
 	/*
 	 * use the library to figure out assignments for all events of all sets
 	 */
@@ -800,13 +823,14 @@ mainloop(char **argv)
 		for(j=0, p = str; p && j < allowed_counters; j++) {
 
 			p = strchr(str, ',');
-			if (p)
-				*p = '\0';
-			ret = pfm_find_full_event(str, &inp.pfp_events[j]);
-			if (ret != PFMLIB_SUCCESS)
-				fatal_error("event %s for set %d event %d: %s\n", str, i, j, pfm_strerror(ret));
-			if (p)
+			if (p) *p = '\0';
+
+			if (pfm_find_event(str, &inp.pfp_events[j].event) != PFMLIB_SUCCESS) {
+				fatal_error("Cannot find %s event for set %d event %d\n", str, i, j);
+			}
+			if (p) {
 				str = p + 1;
+			}
 		}
 		if (p) {
 			fatal_error("error in set %d: cannot have more than %d event(s) per set %s\n",
@@ -818,16 +842,16 @@ mainloop(char **argv)
 		 * add the cycle event as the last event when we switch on overflow
 		 */
 		if (options.opt_ovfl_switch) {
-			inp.pfp_events[j]   = cycle_event;
-			inp.pfp_event_count = j+1;
-			inp.pfp_dfl_plm     = options.opt_plm;
-			e->n_events	    = j+1;
+			inp.pfp_events[j].event = cycles_event_idx;
+			inp.pfp_event_count     = j+1;
+			inp.pfp_dfl_plm         = options.opt_plm;
+			e->n_events		= j+1;
 		} else {
-			e->n_events         = j;
-			inp.pfp_event_count = j;
+			e->n_events             = j;
+			inp.pfp_event_count     = j;
 		}
 
-		inp.pfp_dfl_plm = options.opt_plm;
+		inp.pfp_dfl_plm         = options.opt_plm;
 
 		if (options.opt_is_system) 
 			inp.pfp_flags = PFMLIB_PFP_SYSTEMWIDE;
@@ -836,22 +860,24 @@ mainloop(char **argv)
 		/*
 		 * let the library do the hard work
 		 */
-		if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS)
+		if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS) {
 			fatal_error("cannot configure events for set %d: %s\n", i, pfm_strerror(ret));
+		}
 
 		/*
 		 * propagate from libpfm to kernel data structures
 		 */
-		for (j=0; j < outp.pfp_pmc_count; j++, num_pmcs++) {
+		for (j=0; j < inp.pfp_event_count; j++, num_pmcs++) {
 			all_pmcs[num_pmcs].reg_num   = outp.pfp_pmcs[j].reg_num;
 			all_pmcs[num_pmcs].reg_value = outp.pfp_pmcs[j].reg_value;
 			all_pmcs[num_pmcs].reg_set   = i;
-		}
-		for (j=0; j < outp.pfp_pmd_count; j++, num_pmds++) {
-			all_pmds[num_pmds].reg_num = outp.pfp_pmds[j].reg_num;
-			all_pmds[num_pmds].reg_set = i;
-		}
 
+		}
+		for (j=0, k=0; j < inp.pfp_event_count; j++, num_pmds++) {
+			all_pmds[num_pmds].reg_num   = outp.pfp_pmcs[k].reg_pmd_num;;
+			all_pmds[num_pmds].reg_set   = i;
+			for(; k < outp.pfp_pmc_count; k++)  if (outp.pfp_pmcs[k].reg_evt_idx != j) break;
+		}
 		/*
 		 * setup event set properties
 		 */
@@ -867,7 +893,7 @@ mainloop(char **argv)
 			 * the first overflow of our trigger counter does
 			 * trigger a switch.
 			 */
-			all_pmds[num_pmds-1].reg_ovfl_switch_cnt = 1;
+			all_pmds[num_pmcs-1].reg_ovfl_switch_cnt = 1;
 
 			/*
 			 * We do this even in system-wide mode to ensure
@@ -880,16 +906,22 @@ mainloop(char **argv)
 			all_pmds[num_pmds-1].reg_long_reset  = - options.smpl_period;
 		} else {
 			/*
-			 * setup the switch timeout (in nanoseconds)
+			 * setup the switch timeout
 			 * Note that the actual timeout may be bigger than requested
 			 * due to timer tick granularity. It is always advised to
 			 * check the set_timeout value upon return from set creation.
 			 * The structure will by then contain the actual timeout.
 			 */
 			all_sets[i].set_flags    = PFM_SETFL_TIME_SWITCH;
-			all_sets[i].set_timeout  = 1000000000 / options.smpl_freq;
+			all_sets[i].set_timeout  = 1000000 / options.smpl_freq;
 		}
 #ifdef __ia64__
+		/*
+		 * setup additional per set flags
+		 */
+		if (options.opt_excl_idle && options.opt_is_system)
+			all_sets[i].set_flags  |= PFM_SETFL_EXCL_IDLE;
+
 		if (options.opt_excl_intr && options.opt_is_system)
 			all_sets[i].set_flags  |= PFM_ITA_SETFL_EXCL_INTR;
 
@@ -921,6 +953,7 @@ static struct option multiplex_options[]={
 	{ "us-counter-format", 0, &options.opt_us_format, 1},
 	{ "ovfl-switch", 0, &options.opt_ovfl_switch, 1},
 	{ "system-wide", 0, &options.opt_is_system, 1},
+	{ "excl-idle", 0, &options.opt_excl_idle, 1},
 #ifdef __ia64__
 	{ "excl-intr", 0, &options.opt_excl_intr, 1},
 	{ "intr-only", 0, &options.opt_intr_only, 1},
@@ -934,17 +967,17 @@ static void
 generate_default_sets(void)
 {
 	event_set_t *es, *tail = NULL;
-	pfmlib_event_t events[2];
+	unsigned int idx[2];
 	size_t len;
 	char *name;
 	unsigned int i;
 	int ret;
 	
-	ret = pfm_get_cycle_event(&events[0]);
+	ret = pfm_get_cycle_event(idx);
 	if (ret != PFMLIB_SUCCESS)
 		fatal_error("cannot find cycle event\n");
 
-	ret = pfm_get_inst_retired_event(&events[1]);
+	ret = pfm_get_inst_retired_event(idx+1);
 	if (ret != PFMLIB_SUCCESS)
 		fatal_error("cannot find instruction retired event\n");
 
@@ -952,10 +985,10 @@ generate_default_sets(void)
 
 	for (i=0; i < 2; i++) {
 		name = malloc(len+1);
-		if (name == NULL)
+		if (name == NULL) {
 			fatal_error("cannot allocate space for event name\n");
-
-		pfm_get_full_event_name(events+i, name, len+1);
+		}
+		pfm_get_event_name(idx[i], name, len+1);
 
 		es = (event_set_t *)malloc(sizeof(event_set_t));
 		if (es == NULL)
