@@ -80,8 +80,9 @@ sigio_handler(int n, struct siginfo *info, struct sigcontext *sc)
 	int fd = ctx_fd;
 	int r;
 
-	if (fd != ctx_fd)
+	if (fd != ctx_fd) {
 		fatal_error("handler does not get valid file descriptor\n");
+	}
 
 	if (event1_name && pfm_read_pmds(fd, pd+1, 1) == -1)
 		fatal_error("pfm_read_pmds: %s", strerror(errno));
@@ -108,14 +109,9 @@ retry:
 	 * XXX: risky to do printf() in signal handler!
 	 */
 	if (event1_name)
-		printf("Notification %lu: %"PRIu64" %s ip=0x%llx\n",
-			notification_received, pd[1].reg_value,
-			event1_name,
-			(unsigned long long)msg.pfm_ovfl_msg.msg_ovfl_ip);
+		printf("Notification %lu: %"PRIu64" %s\n", notification_received, pd[1].reg_value, event1_name);
 	else
-		printf("Notification %lu ip=0x%llx\n",
-			notification_received,
-			(unsigned long long)msg.pfm_ovfl_msg.msg_ovfl_ip);
+		printf("Notification %lu\n", notification_received);
 
 	/*
 	 * And resume monitoring
@@ -136,13 +132,6 @@ busyloop(void)
 	for(;notification_received < 3;) ;
 }
 
-#define BPL (sizeof(uint64_t)<<3)
-#define LBPL	6
-
-static inline void pfm_bv_set(uint64_t *bv, uint16_t rnum)
-{
-	bv[rnum>>LBPL] |= 1UL << (rnum&(BPL-1));
-}
 
 int
 main(int argc, char **argv)
@@ -154,7 +143,7 @@ main(int argc, char **argv)
 	pfarg_load_t load_args;
 	pfmlib_options_t pfmlib_options;
 	struct sigaction act;
-	unsigned int i, num_counters;
+	unsigned int i, j, num_counters;
 	size_t len;
 	int ret;
 
@@ -177,7 +166,6 @@ main(int argc, char **argv)
 	 */
 	memset(&pfmlib_options, 0, sizeof(pfmlib_options));
 	pfmlib_options.pfm_debug = 0; /* set to 1 for debug */
-	pfmlib_options.pfm_verbose = 1; /* set to 1 for verbose */
 	pfm_set_options(&pfmlib_options);
 
 	memset(pc, 0, sizeof(pc));
@@ -188,30 +176,30 @@ main(int argc, char **argv)
 
 	pfm_get_num_counters(&num_counters);
 
-	if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS)
+	if (pfm_get_cycle_event(&inp.pfp_events[0].event) != PFMLIB_SUCCESS)
 		fatal_error("cannot find cycle event\n");
 
-	if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS)
+	if (pfm_get_inst_retired_event(&inp.pfp_events[1].event) != PFMLIB_SUCCESS)
 		fatal_error("cannot find inst retired event\n");
 
 	i = 2;
 	
-	/*
-	 * set the default privilege mode for all counters:
-	 * 	PFM_PLM3 : user level only
-	 */
-	inp.pfp_dfl_plm = PFM_PLM3;
 	if (i > num_counters) {
 		i = num_counters;
 		printf("too many events provided (max=%d events), using first %d event(s)\n", num_counters, i);
 	}
 
-	inp.pfp_event_count = i;
+	/*
+	 * set the default privilege mode for all counters:
+	 * 	PFM_PLM3 : user level only
+	 */
+	inp.pfp_dfl_plm = PFM_PLM3;
 
 	/*
 	 * how many counters we use
 	 */
 	if (i > 1) {
+		inp.pfp_event_count = i;
 
 		pfm_get_max_event_name_len(&len);
 
@@ -219,19 +207,19 @@ main(int argc, char **argv)
 		if (event1_name == NULL)
 			fatal_error("cannot allocate event name\n");
 
-		pfm_get_full_event_name(&inp.pfp_events[1], event1_name, len+1);
+		pfm_get_event_name(inp.pfp_events[1].event, event1_name, len+1);
 	}
 
 	/*
 	 * now create the context for self monitoring/per-task
 	 */
-	ctx_fd = pfm_create_context(&ctx, NULL, NULL, 0);
-	if (ctx_fd == -1) {
+	if (pfm_create_context(&ctx, NULL, 0)) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	ctx_fd = ctx.ctx_fd;
 
 	/*
 	 * build the pfp_unavail_pmcs bitmask by looking
@@ -254,25 +242,33 @@ main(int argc, char **argv)
 
 	/*
 	 * Now prepare the argument to initialize the PMDs and PMCS.
+	 * We use pfp_pmc_count to determine the number of registers to
+	 * setup. Note that this field can be >= pfp_event_count.
 	 */
 
 	for (i=0; i < outp.pfp_pmc_count; i++) {
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
-	for (i=0; i < outp.pfp_pmd_count; i++)
-		pd[i].reg_num = outp.pfp_pmds[i].reg_num;
+
+	/*
+	 * figure out pmd mapping from output pmc
+	 */
+	for (i=0, j=0; i < inp.pfp_event_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
+		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
+	}
 	/*
 	 * We want to get notified when the counter used for our first
 	 * event overflows
 	 */
-	pd[0].reg_flags |= PFM_REGFL_OVFL_NOTIFY;
+	pd[0].reg_flags 	|= PFM_REGFL_OVFL_NOTIFY;
 
 	/*
 	 * nothing to sample when only one counter
 	 */
 	if (inp.pfp_event_count > 1)
-		pfm_bv_set(pd[0].reg_reset_pmds, pd[1].reg_num);
+		pd[0].reg_reset_pmds[0] |= 1UL << pd[1].reg_num;
 
 	/*
 	 * we arm the first counter, such that it will overflow
@@ -284,11 +280,19 @@ main(int argc, char **argv)
 
 	/*
 	 * Now program the registers
+	 *
+	 * We don't use the save variable to indicate the number of elements passed to
+	 * the kernel because, as we said earlier, pc may contain more elements than
+	 * the number of events we specified, i.e., contains more than counting monitors.
 	 */
 	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count))
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
 
-	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count))
+	/*
+	 * To be read, each PMD must be either written or declared
+	 * as being part of a sample (reg_smpl_pmds)
+	 */
+	if (pfm_write_pmds(ctx_fd, pd, inp.pfp_event_count))
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
 
 	/*
