@@ -48,8 +48,6 @@
 
 #include "detect_pmcs.h"
 
-#define SWITCH_TIMEOUT	1000000000 /* in nanoseconds */
-
 #define RTOP_VERSION "0.1"
 
 #define RTOP_MAX_CPUS		1024	/* maximum number of CPU supported */
@@ -121,7 +119,6 @@ typedef struct _setdesc_t {
 	pfmlib_output_param_t	outp;
 	uint16_t		set_id;
 	uint32_t		set_flags;
-	uint32_t		set_timeout; /* actual timeout */
 	int			(*handler)(int fd, FILE *fp, thread_desc_t *td, struct _setdesc_t *my_sdesc);
 	void 			*data;
 	eventdesc_t		*evt_desc;
@@ -145,6 +142,7 @@ typedef enum {
 typedef struct {
 	uint64_t prev_k_cycles;
 	uint64_t prev_u_cycles;
+	uint64_t prev_itc_cycles;
 } set0_data_t;
 
 static barrier_t 		barrier;
@@ -166,6 +164,7 @@ static int handler_set0(int fd, FILE *fp, thread_desc_t *td, setdesc_t *my_sdesc
 
 static setdesc_t setdesc_tab[]={
 	{ .set_id    = 0,
+	  .set_flags = PFM_SETFL_EXCL_IDLE,
 	  .evt_desc  = set0_evt,
 	  .handler   = handler_set0
 	}
@@ -480,15 +479,16 @@ handler_set0(int fd, FILE *fp, thread_desc_t *td, setdesc_t *my_sdesc)
 {
 	double k_cycles, u_cycles, i_cycles;
 	set0_data_t *sd1;
+	pfarg_setinfo_t info;
 	uint64_t itc_delta;
 	long mycpu;
 
 	mycpu = td->cpuid;
 
 	if (my_sdesc->data == NULL) {
-		my_sdesc->data = sd1 = calloc(1, sizeof(set0_data_t));
-		if (sd1 == NULL)
-			return -1;
+		my_sdesc->data = sd1 = malloc(sizeof(set0_data_t));
+		if (sd1 == NULL) return -1;
+		memset(sd1, 0, sizeof(*sd1));
 
 	}
 	sd1 = my_sdesc->data;
@@ -502,39 +502,41 @@ handler_set0(int fd, FILE *fp, thread_desc_t *td, setdesc_t *my_sdesc)
 	}
 
 	/*
-	 * expected maximum duration with monitoring active for this set
-	 * set_timeout is in nanoseconds, we need to divide mhz by 1000
-	 * to get cycles.
+	 * get elapsed cycles
 	 */
-	itc_delta =  (my_sdesc->set_timeout*(uint64_t)options.cpu_mhz)/1000;
+	info.set_id = my_sdesc->set_id;
+	if (pfm_getinfo_evtsets(fd, &info, 1) == -1) {
+		warning( "CPU%ld pfm_getinfo_evtsets error errno %d\n", mycpu, errno);
+		return -1;
+	}
 
-	k_cycles   = (double)(my_sdesc->pd[0].reg_value - sd1->prev_k_cycles)*100.0/ (double)itc_delta;
-	u_cycles   = (double)(my_sdesc->pd[1].reg_value - sd1->prev_u_cycles)*100.0/ (double)itc_delta;
-	i_cycles   = 100.0 - (k_cycles + u_cycles);
+	itc_delta = info.set_act_duration - sd1->prev_itc_cycles;
+
+	k_cycles   = (double)(my_sdesc->pd[0].reg_value - sd1->prev_k_cycles) / (double)itc_delta;
+	u_cycles   = (double)(my_sdesc->pd[1].reg_value - sd1->prev_u_cycles) / (double)itc_delta;
+	i_cycles   = 1.0 - (k_cycles + u_cycles);
 
 	/*
 	 * adjust for rounding errors
 	 */
 	if (i_cycles < 0.0) i_cycles = 0.0;
-	if (i_cycles > 100.0) i_cycles = 100.0;
-	if (k_cycles > 100.0) k_cycles = 100.0;
-	if (u_cycles > 100.0) u_cycles = 100.0;
+
+	sd1->prev_k_cycles   = my_sdesc->pd[0].reg_value;
+	sd1->prev_u_cycles   = my_sdesc->pd[1].reg_value;
+	sd1->prev_itc_cycles = info.set_act_duration;
 
 	printw("CPU%-2ld %6.2f%% usr %6.2f%% sys %6.2f%% idle\n",
 		mycpu,
-		u_cycles,
-		k_cycles,
-		i_cycles);
-
-	sd1->prev_k_cycles      = my_sdesc->pd[0].reg_value;
-	sd1->prev_u_cycles      = my_sdesc->pd[1].reg_value;
+		u_cycles*100.0,
+		k_cycles*100.0,
+		i_cycles*100.0);
 
 	if (fp)
 		fprintf(fp, "%"PRIu64" %6.2f %6.2f %6.2f\n",
 			td->nsamples,
-			u_cycles,
-			k_cycles,
-			i_cycles);
+			u_cycles*100.0,
+			k_cycles*100.0,
+			i_cycles*100.0);
 	td->nsamples++;
 	return 0;
 }
@@ -610,8 +612,7 @@ do_measure_one_cpu(void *data)
 
 	memcpy(my_sdesc_tab, setdesc_tab, sizeof(setdesc_t)*RTOP_NUM_SDESC);
 
-	fd = pfm_create_context(&ctx, NULL, NULL, 0);
-	if (fd == -1) {
+	if (pfm_create_context(&ctx, NULL, 0) == -1) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
@@ -619,6 +620,8 @@ do_measure_one_cpu(void *data)
 		goto error;
 	}
 
+	fd = ctx.ctx_fd;
+
 	/*
 	 * WARNING: on processors where the idle loop goes into some power-saving
 	 * state, the results of this program may be incorrect
@@ -629,13 +632,13 @@ do_measure_one_cpu(void *data)
 
 		setd.set_id    = my_sdesc->set_id;
 		setd.set_flags = my_sdesc->set_flags;
-		setd.set_timeout =  SWITCH_TIMEOUT; /* in nsecs */
+		setd.set_timeout =  500000;
+
 
 		if (pfm_create_evtsets(fd, &setd, 1) == -1) {
 			warning("CPU%ld cannot create set%u: %d\n", mycpu, j, errno);
 			goto error;
 		}
-		my_sdesc->set_timeout = setd.set_timeout;
 
 		if (pfm_write_pmcs(fd, my_sdesc->pc, my_sdesc->outp.pfp_pmc_count) == -1) {
 			warning("CPU%ld pfm_write_pmcs error errno %d\n", mycpu, errno);
@@ -652,12 +655,7 @@ do_measure_one_cpu(void *data)
 		}
 	}
 
-	/*
-	 * in system-wide mode, this field must provide the CPU the caller wants
-	 * to monitor. The kernel checks and if calling from the wrong CPU, the
-	 * call fails. The affinity is not affected.
-	 */
-	load_args.load_pid = mycpu;
+	load_args.load_pid = gettid();
 
 	if (pfm_load_context(fd, &load_args) == -1) {
 		warning("CPU%ld pfm_load_context error errno %d\n", mycpu, errno);
@@ -799,11 +797,7 @@ mainloop(void)
 				refresh();
 				barrier_wait(&barrier);
 				break;
-			case -1:
-				/* restart in case of signal */
-				if (errno == EINTR)
-					continue;
-				warning("polling error: %s\n", strerror(errno));
+			case -1: warning("polling error: %s\n", strerror(errno));
 				 /* fall through */
 			default: time_to_quit = 1;
 		}
@@ -813,8 +807,7 @@ mainloop(void)
 	/*
 	 * get worker thread out of their mainloop
 	 */
-	for (i=0; i < ncpus; i++)
-		sem_post(&thread_info[i].his_sem);
+	for (i=0; i < ncpus; i++) sem_post(&thread_info[i].his_sem);
 join_all:
 	for(i=0; i< ncpus; i++) {
 		ret = pthread_join(thread_info[i].tid, &retval);
@@ -839,8 +832,8 @@ setup_measurement(void)
 	pfmlib_options_t pfmlib_options;
 	eventdesc_t *evt;
 	setdesc_t *sdesc;
-	pfmlib_event_t trigger_event;
-	unsigned int i, j;
+	unsigned int i, j, k;
+	unsigned int trigger_event;
 	int ret;
 
 	/*
@@ -880,10 +873,11 @@ setup_measurement(void)
 		evt = sdesc->evt_desc;
 
 		for(j=0; evt[j].name ; j++) {
-			if (*evt[j].name == '*')
-				sdesc->inp.pfp_events[j] = trigger_event;
-			else if (pfm_find_full_event(evt[j].name, &sdesc->inp.pfp_events[j]) != PFMLIB_SUCCESS)
+			if (*evt[j].name == '*') {
+				sdesc->inp.pfp_events[j].event = trigger_event;
+			} else if (pfm_find_event(evt[j].name, &sdesc->inp.pfp_events[j].event) != PFMLIB_SUCCESS) {
 				fatal_error("cannot find %s event\n", evt[j].name);
+			}
 			sdesc->inp.pfp_events[j].plm   = evt[j].plm;
 		}
 
@@ -907,17 +901,19 @@ setup_measurement(void)
 		/*
 		 * let the library figure out the values for the PMCS
 		 */
-		if ((ret=pfm_dispatch_events(&sdesc->inp, NULL, &sdesc->outp, NULL)) != PFMLIB_SUCCESS)
+		if ((ret=pfm_dispatch_events(&sdesc->inp, NULL, &sdesc->outp, NULL)) != PFMLIB_SUCCESS) {
 			fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
+		}
 
 		for (j=0; j < sdesc->outp.pfp_pmc_count; j++) {
 			sdesc->pc[j].reg_set   = i;
 			sdesc->pc[j].reg_num   = sdesc->outp.pfp_pmcs[j].reg_num;
 			sdesc->pc[j].reg_value = sdesc->outp.pfp_pmcs[j].reg_value;
 		}
-		for (j=0; j < sdesc->outp.pfp_pmd_count; j++) {
-			sdesc->pd[j].reg_num = sdesc->outp.pfp_pmds[j].reg_num;
+		for (j=0, k=0; j < sdesc->inp.pfp_event_count; j++) {
 			sdesc->pd[j].reg_set = i;
+			sdesc->pd[j].reg_num = sdesc->outp.pfp_pmcs[k].reg_pmd_num;
+			for(; k < sdesc->outp.pfp_pmc_count; k++)  if (sdesc->outp.pfp_pmcs[k].reg_evt_idx != j) break;
 		}
 	}
 }
