@@ -43,8 +43,7 @@
 #define NUM_PMCS PFMLIB_MAX_PMCS
 #define NUM_PMDS PFMLIB_MAX_PMDS
 
-#define TABSIZE 1024
-static int a[TABSIZE], b[TABSIZE];
+#define MAX_EVT_NAME_LEN	128
 
 /*
  * our test code (function cannot be made static otherwise it is optimized away)
@@ -52,10 +51,8 @@ static int a[TABSIZE], b[TABSIZE];
 uint64_t
 noploop(uint64_t loop)
 {
-	unsigned int j = 0;
-
-	while (loop--) { a[j] += a[j]*loop + b[j];j = (j+1)%TABSIZE; }
-	return a[0];
+	while (loop--) {}
+	return loop;
 }
 
 static void fatal_error(char *fmt,...) __attribute__((noreturn));
@@ -76,7 +73,7 @@ int
 main(int argc, char **argv)
 {
 	char **p;
-	unsigned int i;
+	unsigned int i, j;
 	int ret, ctx_fd;
 	pfmlib_input_param_t inp;
 	pfmlib_output_param_t outp;
@@ -86,8 +83,7 @@ main(int argc, char **argv)
 	pfarg_load_t load_args;
 	pfmlib_options_t pfmlib_options;
 	unsigned int num_counters;
-	size_t len;
-	char *name;
+	char name[MAX_EVT_NAME_LEN];
 
 	/*
 	 * Initialize pfm library (required before we can use it)
@@ -96,11 +92,6 @@ main(int argc, char **argv)
 	if (ret != PFMLIB_SUCCESS)
 		fatal_error("Cannot initialize library: %s\n", pfm_strerror(ret));
 
-	pfm_get_max_event_name_len(&len);
-	name = malloc(len+1);
-	if (!name)
-		fatal_error("cannot allocate event name buffer\n");
-
 	pfm_get_num_counters(&num_counters);
 
 	/*
@@ -108,7 +99,7 @@ main(int argc, char **argv)
 	 */
 	memset(&pfmlib_options, 0, sizeof(pfmlib_options));
 	pfmlib_options.pfm_debug   = 0; /* set to 1 for debug */
-	pfmlib_options.pfm_verbose = 1; /* set to 1 for verbose */
+	pfmlib_options.pfm_verbose = 1; /* set to 1 for debug */
 	pfm_set_options(&pfmlib_options);
 
 	memset(pd, 0, sizeof(pd));
@@ -128,15 +119,14 @@ main(int argc, char **argv)
 	if (argc > 1) {
 		p = argv+1;
 		for (i=0; *p ; i++, p++) {
-			ret = pfm_find_full_event(*p, &inp.pfp_events[i]);
-			if (ret != PFMLIB_SUCCESS)
-				fatal_error("event %s: %s\n", *p, pfm_strerror(ret));
+			if (pfm_find_event(*p, &inp.pfp_events[i].event) != PFMLIB_SUCCESS)
+				fatal_error("Cannot find %s event\n", *p);
 		}
 	} else {
-		if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS)
+		if (pfm_get_cycle_event(&inp.pfp_events[0].event) != PFMLIB_SUCCESS)
 			fatal_error("cannot find cycle event\n");
 
-		if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS)
+		if (pfm_get_inst_retired_event(&inp.pfp_events[1].event) != PFMLIB_SUCCESS)
 			fatal_error("cannot find inst retired event\n");
 		i = 2;
 	}
@@ -162,12 +152,17 @@ main(int argc, char **argv)
 	 * This just creates a new context with some initial state, it is not
 	 * active nor attached to any process.
 	 */
-	ctx_fd = pfm_create_context(&ctx, NULL, NULL, 0);
-	if (ctx_fd == -1)  {
+	if (pfm_create_context(&ctx, NULL, 0) == -1 ) {
 		if (errno == ENOSYS)
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+
+	/*
+	 * extract the unique identifier for our context, a regular file descriptor
+	 */
+	ctx_fd = ctx.ctx_fd;
+
 	/*
 	 * build the pfp_unavail_pmcs bitmask by looking
 	 * at what perfmon has available. It is not always
@@ -189,28 +184,43 @@ main(int argc, char **argv)
 
 	/*
 	 * Now prepare the argument to initialize the PMDs and PMCS.
-	 * We use pfp_pmc_count to determine the number of PMC to intialize.
-	 * We use pfp_pmd_count to determine the number of PMD to initialize.
-	 * Some events/features may cause extra PMCs to be used, leading to:
-	 * 	- pfp_pmc_count may be >= pfp_event_count
-	 * 	- pfp_pmd_count may be >= pfp_event_count
+	 * We must pfp_pmc_count to determine the number of PMC to intialize.
+	 * We must use pfp_event_count to determine the number of PMD to initialize.
+	 * Some events causes extra PMCs to be used, so  pfp_pmc_count may be >= pfp_event_count.
+	 *
+	 * This step is new compared to libpfm-2.x. It is necessary because the library no
+	 * longer knows about the kernel data structures.
 	 */
+
 	for (i=0; i < outp.pfp_pmc_count; i++) {
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
 
-	for (i=0; i < outp.pfp_pmd_count; i++) {
-		pd[i].reg_num   = outp.pfp_pmds[i].reg_num;
+	/*
+	 * figure out pmd mapping from output pmc
+	 */
+	for (i=0, j=0; i < inp.pfp_event_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
+		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
 	}
 
 	/*
 	 * Now program the registers
+	 *
+	 * We don't use the same variable to indicate the number of elements passed to
+	 * the kernel because, as we said earlier, pc may contain more elements than
+	 * the number of events (pmd) we specified, i.e., contains more than counting
+	 * monitors.
 	 */
 	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count))
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
 
-	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count))
+	/*
+	 * To be read, each PMD must be either written or declared
+	 * as being part of a sample (reg_smpl_pmds)
+	 */
+	if (pfm_write_pmds(ctx_fd, pd, inp.pfp_event_count))
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
 
 	/*
@@ -232,9 +242,7 @@ main(int argc, char **argv)
 		fatal_error("pfm_stop error errno %d\n",errno);
 
 	/*
-	 * now read the results. We use pfp_event_count because
-	 * libpfm guarantees that counters for the events always
-	 * come first.
+	 * now read the results
 	 */
 	if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count))
 		fatal_error( "pfm_read_pmds error errno %d\n",errno);
@@ -243,13 +251,12 @@ main(int argc, char **argv)
 	 * print the results
 	 */
 	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_full_event_name(&inp.pfp_events[i], name, len+1);
-		printf("PMD%-3u %20"PRIu64" %s\n",
+		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
+		printf("PMD%u %20"PRIu64" %s\n",
 			pd[i].reg_num,
 			pd[i].reg_value,
 			name);
 	}
-	free(name);
 	/*
 	 * and destroy our context
 	 */
