@@ -7,10 +7,44 @@
 #ifndef __STANDALONE_H__
 #define __STANDALONE_H__ 1
 
+#ifdef __x86_64__
+#define __NR_pfm_create_context		279
+#elif defined(__i386__)
+#define __NR_pfm_create_context		317
+#elif defined( __ia64__)
+#define __NR_pfm_create_context		1303
+#elif defined(__mips__)
+/* Add 12 */
+#if (_MIPS_SIM == _ABIN32) || (_MIPS_SIM == _MIPS_SIM_NABI32)
+#define __NR_Linux 6000
+#define __NR_pfm_create_context		__NR_Linux+269
+#elif (_MIPS_SIM == _ABI32) || (_MIPS_SIM == _MIPS_SIM_ABI32)
+#define __NR_Linux 4000
+#define __NR_pfm_create_context		__NR_Linux+306
+#elif (_MIPS_SIM == _ABI64) || (_MIPS_SIM == _MIPS_SIM_ABI64)
+#define __NR_Linux 5000
+#define __NR_pfm_create_context		__NR_Linux+265
+#endif
+#else
+#error "you need to define based syscall number"
+#endif
+
+#define __NR_pfm_write_pmcs		(__NR_pfm_create_context+1)
+#define __NR_pfm_write_pmds		(__NR_pfm_create_context+2)
+#define __NR_pfm_read_pmds		(__NR_pfm_create_context+3)
+#define __NR_pfm_load_context		(__NR_pfm_create_context+4)
+#define __NR_pfm_start			(__NR_pfm_create_context+5)
+#define __NR_pfm_stop			(__NR_pfm_create_context+6)
+#define __NR_pfm_restart		(__NR_pfm_create_context+7)
+#define __NR_pfm_create_evtsets		(__NR_pfm_create_context+8)
+#define __NR_pfm_getinfo_evtsets	(__NR_pfm_create_context+9)
+#define __NR_pfm_delete_evtsets		(__NR_pfm_create_context+10)
+#define __NR_pfm_unload_context		(__NR_pfm_create_context+11)
+
 inline int
-pfm_create_context(pfarg_ctx_t *ctx, char *smpl_name, void *smpl_arg, size_t smpl_size)
+pfm_create_context(pfarg_ctx_t *ctx, void *smpl_arg, size_t smpl_size)
 {
-  return syscall(__NR_pfm_create_context, ctx, smpl_name, smpl_arg, smpl_size);
+  return syscall(__NR_pfm_create_context, ctx, smpl_arg, smpl_size);
 }
 
 inline int
@@ -107,12 +141,12 @@ static inline int cpu_detect(void)
 	ret = access("/sys/kernel/perfmon/pmu_desc", F_OK);
 	if (ret == -1) {
 		memset(&ctx, 0, sizeof(ctx));
-		ret = pfm_create_context(&ctx, NULL, NULL, 0);
+		ret = pfm_create_context(&ctx, NULL, 0);
 		if (ret == -1)
 			return -1;
-		close(ret);
+		close(ctx.ctx_fd);
 	}
-	fp = fopen("/sys/kernel/perfmon/pmu_desc/model", "r");
+	fp = fopen("/sys/kernel/perfmon/pmu_model", "r");
 	if (fp == NULL) return -1;
 
 	p  = fgets(buffer, 255, fp);
