@@ -85,7 +85,7 @@ parent(char **arg)
 	pfarg_pmc_t pc[NUM_PMCS];
 	pfarg_pmd_t pd[NUM_PMDS];
 	pfarg_load_t load_args;
-	unsigned int i, num_counters;
+	unsigned int i, j, num_counters;
 	int status, ret;
 	int ctx_fd;
 	pid_t pid;
@@ -100,10 +100,10 @@ parent(char **arg)
 
 	pfm_get_num_counters(&num_counters);
 
-	if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS)
+	if (pfm_get_cycle_event(&inp.pfp_events[0].event) != PFMLIB_SUCCESS)
 		fatal_error("cannot find cycle event\n");
 
-	if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS)
+	if (pfm_get_inst_retired_event(&inp.pfp_events[1].event) != PFMLIB_SUCCESS)
 		fatal_error("cannot find inst retired event\n");
 	i = 2;
 
@@ -126,13 +126,17 @@ parent(char **arg)
 	/*
 	 * now create a context. we will later attach it to the task we are creating.
 	 */
-	ctx_fd = pfm_create_context(ctx, NULL, NULL, 0);
-	if (ctx_fd == -1) {
+	if (pfm_create_context(ctx, NULL, 0) == -1) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract the identifier for our context
+	 */
+	ctx_fd = ctx[0].ctx_fd;
+
 	/*
 	 * build the pfp_unavail_pmcs bitmask by looking
 	 * at what perfmon has available. It is not always
@@ -154,27 +158,44 @@ parent(char **arg)
 	}
 	/*
 	 * Now prepare the argument to initialize the PMDs and PMCS.
-	 * We use pfp_pmc_count to determine the number of PMC to intialize.
-	 * We use pfp_pmd_count to determine the number of PMD to initialize.
-	 * Some events/features may cause extra PMCs to be used, leading to:
-	 * 	- pfp_pmc_count may be >= pfp_event_count
-	 * 	- pfp_pmd_count may be >= pfp_event_count
+	 * We must pfp_pmc_count to determine the number of PMC to intialize.
+	 * We must use pfp_event_count to determine the number of PMD to initialize.
+	 * Some events causes extra PMCs to be used, so  pfp_pmc_count may be >= pfp_event_count.
+	 *
+	 * This step is new compared to libpfm-2.x. It is necessary because the library no
+	 * longer knows about the kernel data structures.
 	 */
 	for (i=0; i < outp.pfp_pmc_count; i++) {
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
-	for(i=0; i < outp.pfp_pmd_count; i++)
-		pd[i].reg_num = outp.pfp_pmds[i].reg_num;
 
 	/*
+	 * figure out pmd mapping from output pmc
+	 */
+	for (i=0, j=0; i < inp.pfp_event_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
+		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
+	}
+	/*
 	 * Now program the registers
+	 *
+	 * We don't use the save variable to indicate the number of elements passed to
+	 * the kernel because, as we said earlier, pc may contain more elements than
+	 * the number of events we specified, i.e., contains more thann counting monitors.
 	 */
-	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1)
-		fatal_error("pfm_write_pmcs error errno %d\n",errno);
 
-	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count) == -1)
+	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1) {
+		fatal_error("pfm_write_pmcs error errno %d\n",errno);
+	}
+
+	/*
+	 * To be read, each PMD must be either written or declared
+	 * as being part of a sample (reg_smpl_pmds)
+	 */
+	if (pfm_write_pmds(ctx_fd, pd, inp.pfp_event_count) == -1) {
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
+	}
 
 	/*
 	 * Create the child task
@@ -194,26 +215,30 @@ parent(char **arg)
 	/*
 	 * check if process exited early
 	 */
-	if (WIFEXITED(status))
+	if (WIFEXITED(status)) {
 		fatal_error("command %s exited too early with status %d\n", arg[0], WEXITSTATUS(status));
-
+	}
 	/*
 	 * the task is stopped at this point
 	 */
 	
+	
 	/*
 	 * now we load (i.e., attach) the context to ourself
 	 */
 	load_args.load_pid = pid;
-	if (pfm_load_context(ctx_fd, &load_args) == -1)
+
+	if (pfm_load_context(ctx_fd, &load_args) == -1) {
 		fatal_error("pfm_load_context error errno %d\n",errno);
+	}
 
 	/*
 	 * activate monitoring. The task is still STOPPED at this point. Monitoring
 	 * will not take effect until the execution of the task is resumed.
 	 */
-	if (pfm_start(ctx_fd, NULL) == -1)
+	if (pfm_start(ctx_fd, NULL) == -1) {
 		fatal_error("pfm_start error errno %d\n",errno);
+	}
 
 	/*
 	 * now resume execution of the task, effectively activating
@@ -238,9 +263,10 @@ parent(char **arg)
 	/*
 	 * now simply read the results.
 	 */
-	if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count) == -1)
+	if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count) == -1) {
 		fatal_error("pfm_read_pmds error errno %d\n",errno);
-
+		return -1;
+	}
 	/*
 	 * print the results
 	 *
@@ -250,8 +276,8 @@ parent(char **arg)
 	 *
 	 */
 	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_full_event_name(&inp.pfp_events[i], name, MAX_EVT_NAME_LEN);
-		printf("PMD%-3u %20"PRIu64" %s\n",
+		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
+		printf("PMD%u %20"PRIu64" %s\n",
 			pd[i].reg_num,
 			pd[i].reg_value,
 			name);
@@ -285,7 +311,6 @@ main(int argc, char **argv)
 	 */
 	memset(&pfmlib_options, 0, sizeof(pfmlib_options));
 	pfmlib_options.pfm_debug = 0; /* set to 1 for debug */
-	pfmlib_options.pfm_verbose= 0; /* set to 1 for verbose */
 	pfm_set_options(&pfmlib_options);
 
 	return parent(argv+1);
