@@ -69,7 +69,7 @@ parent(pid_t pid, unsigned long delay)
 	pfarg_load_t load_args;
 	struct pollfd pollfd;
 	pfm_msg_t msg;
-	unsigned int i, num_counters;
+	unsigned int i, j, num_counters;
 	int status, ret;
 	int ctx_fd;
 	char name[MAX_EVT_NAME_LEN];
@@ -83,11 +83,11 @@ parent(pid_t pid, unsigned long delay)
 
 	pfm_get_num_counters(&num_counters);
 
-	if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS) {
+	if (pfm_get_cycle_event(&inp.pfp_events[0].event) != PFMLIB_SUCCESS) {
 		fatal_error("cannot find cycle event\n");
 	}
 
-	if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS) {
+	if (pfm_get_inst_retired_event(&inp.pfp_events[1].event) != PFMLIB_SUCCESS) {
 		fatal_error("cannot find inst retired event\n");
 	}
 	i = 2;
@@ -111,13 +111,17 @@ parent(pid_t pid, unsigned long delay)
 	/*
 	 * now create a context. we will later attach it to the task we are creating.
 	 */
-	ctx_fd = pfm_create_context(ctx, NULL, NULL, 0);
-	if (ctx_fd == -1) {
+	if (pfm_create_context(ctx, NULL, 0) == -1) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
+	/*
+	 * extract the identifier for our context
+	 */
+	ctx_fd = ctx[0].ctx_fd;
+
 	/*
 	 * build the pfp_unavail_pmcs bitmask by looking
 	 * at what perfmon has available. It is not always
@@ -157,8 +161,14 @@ parent(pid_t pid, unsigned long delay)
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
-	for (i=0; i < outp.pfp_pmd_count; i++)
-		pd[i].reg_num = outp.pfp_pmds[i].reg_num;
+	/*
+	 * figure out pmd mapping from output pmc
+	 */
+	for (i=0, j=0; i < inp.pfp_event_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmcs[j].reg_pmd_num;
+		for(; j < outp.pfp_pmc_count; j++)  if (outp.pfp_pmcs[j].reg_evt_idx != i) break;
+	}
+
 	/*
 	 * Now program the registers
 	 *
@@ -167,15 +177,17 @@ parent(pid_t pid, unsigned long delay)
 	 * the number of events we specified, i.e., contains more thann counting monitors.
 	 */
 
-	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1)
+	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1) {
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
+	}
 
 	/*
 	 * To be read, each PMD must be either written or declared
 	 * as being part of a sample (reg_smpl_pmds)
 	 */
-	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count) == -1)
+	if (pfm_write_pmds(ctx_fd, pd, inp.pfp_event_count) == -1) {
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
+	}
 
 	ret = ptrace(PTRACE_ATTACH, pid, NULL, 0);
 	if (ret == -1) {
@@ -310,8 +322,8 @@ read_results:
 	 *
 	 */
 	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_full_event_name(&inp.pfp_events[i], name, MAX_EVT_NAME_LEN);
-		printf("PMD%-3u %20"PRIu64" %s\n",
+		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
+		printf("PMD%u %20"PRIu64" %s\n",
 			pd[i].reg_num,
 			pd[i].reg_value,
 			name);
