@@ -11,7 +11,6 @@
 
 #include <sys/types.h>
 #include <stdint.h>
-#include <syscall.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -34,10 +33,20 @@ extern "C" {
 #define PFM_PMD_BV      PFM_BVSIZE(PFM_MAX_PMDS)
 #define PFM_PMC_BV      PFM_BVSIZE(PFM_MAX_PMCS)
 
+typedef unsigned char pfm_uuid_t[16];	/* custom sampling buffer identifier type */
+
 #ifdef __ia64__
 #include <perfmon/perfmon_ia64.h>
 #endif
 
+#if defined(__i386__) || defined(__x86_64__)
+#include <perfmon/perfmon_i386.h>
+#endif
+
+#ifdef __mips__
+#include <perfmon/perfmon_mips64.h>
+#endif
+
 /*
  * PMC/PMD flags to use with pfm_write_pmds() or pfm_write_pmcs()
  *
@@ -56,6 +65,7 @@ extern "C" {
 #define PFM_SETFL_OVFL_SWITCH	0x01 /* enable switch on overflow (subject to individual switch_cnt */
 #define PFM_SETFL_TIME_SWITCH	0x02 /* switch set on timeout */
 #define PFM_SETFL_EXPL_NEXT	0x04 /* use set_id_next as the next set */
+#define PFM_SETFL_EXCL_IDLE	0x08 /* exclude idle task (system-wide only) */
 
 /*
  * PMD/PMC return flags in case of error (ignored on input)
@@ -71,13 +81,16 @@ extern "C" {
 
 #define PFM_REG_HAS_ERROR(flag)	(((flag) & PFM_REG_RETFL_MASK) != 0)
 
+
 /*
- * argument to pfm_create_context()
+ * Request structure used to define a context
  */
 typedef struct {
+	pfm_uuid_t	ctx_smpl_buf_id;	/* which buffer format to use (if needed) */
 	uint32_t	ctx_flags;	   /* noblock/block/syswide */
-	uint32_t	ctx_reserved1;	   /* for future use */
-	uint64_t	ctx_reserved3[7];  /* for future use */
+	int		ctx_fd;		 	/* return arg: unique identification for context */
+	uint64_t	ctx_smpl_buf_size;	/* return arg: actual smpl buffer size */
+	uint64_t	ctx_reserved3[12];	/* for future use */
 } pfarg_ctx_t;
 
 /*
@@ -89,8 +102,11 @@ typedef struct {
 #define PFM_FL_OVFL_NO_MSG	 0x80   /* no overflow msgs */
 #define PFM_FL_MAP_SETS		 0x10	/* event sets are remapped */
 
+
 /*
- * argument for pfm_write_pmcs()
+ * argument structure for PFM_WRITE_PMCS2/PFM_GET_DEFAULT_PMC_VAL2 (only).
+ *
+ * For PFM_WRITE_PMCS and PFM_WRITE_PMDS, see perfmon_compat.h
  */
 typedef struct {
 	uint16_t reg_num;	   			/* which register */
@@ -101,7 +117,9 @@ typedef struct {
 } pfarg_pmc_t;
 
 /*
- * argument pfm_write_pmds() and pfm_read_pmds()
+ * argument structure for PFM_WRITE_PMDS2/PFM_READ_PMDS2
+ *
+ * For PFM_WRITE_PMCS and PFM_WRITE_PMDS, see perfmon_compat.h
  */
 typedef struct {
 	uint16_t reg_num;	   	/* which register */
@@ -114,14 +132,15 @@ typedef struct {
 	uint64_t reg_ovfl_switch_cnt;	/* how many overflow before switch for next set */
 	uint64_t reg_reset_pmds[PFM_PMD_BV]; /* which other PMDS to reset on overflow */
 	uint64_t reg_smpl_pmds[PFM_PMD_BV];  /* which other PMDS to record when the associated PMD overflows */
-	uint64_t reg_smpl_eventid;  	/* opaque sampling event identifier */
+	uint64_t reg_smpl_eventid;  			/* opaque sampling event identifier */
 	uint64_t reg_random_mask; 	/* bitmask used to limit random value */
-	uint32_t reg_random_seed;   	/* seed for randomization (DEPRECATED) */
+	uint32_t reg_random_seed;   	/* seed value when randomization is used */
 	uint32_t reg_reserved2[7];	/* for future use */
 } pfarg_pmd_t;
 
+
 /*
- * optional argument to pfm_start(), pass NULL if no arg needed
+ * optional argument to PFM_START, pass NULL if no arg needed
  */
 typedef struct {
 	uint16_t start_set;		/* event set to start with */
@@ -131,29 +150,30 @@ typedef struct {
 } pfarg_start_t;
 
 /*
- * argument to pfm_load_context()
+ * argument to PFM_LOAD_CONTEXT
  */
 typedef struct {
-	uint32_t	load_pid;          /* thread or CPU to attach to */
+	uint32_t	load_pid;          /* thread to attach to */
 	uint16_t        load_set;          /* set to load first */
 	uint16_t        load_reserved1;    /* for future use */
 	uint64_t        load_reserved2[3]; /* for future use */
 } pfarg_load_t;
 
 /*
- * argument to pfm_create_evtsets()/pfm_delete_evtsets()
+ * argument to PFM_CREATE_EVTSETS/PFM_DELETE_EVTSETS
  */
 typedef struct {
 	uint16_t	set_id;		  /* which set */
 	uint16_t	set_id_next;	  /* next set to go to (must use PFM_SETFL_EXPL_NEXT) */
 	uint32_t    	set_flags; 	  /* input: flags for set, output: err flag */
-	uint64_t	set_timeout;	  /* requested/effective switch timeout in nsecs */
+	uint32_t	set_timeout;	  /* requested/effective switch timeout in usecs */
+	uint32_t	set_reserved1;	  /* for future use */
 	uint64_t	set_mmap_offset;  /* cookie to pass as mmap offset to access 64-bit virtual PMD */
 	uint64_t	reserved[5];	  /* for future use */
 } pfarg_setdesc_t;
 
 /*
- * argument to pfm_getinfo_evtsets()
+ * argument to PFM_GETINFO_EVTSETS
  */
 typedef struct {
         uint16_t	set_id;             /* which set */
@@ -161,7 +181,8 @@ typedef struct {
         uint32_t	set_flags;          /* output:flags or error */
         uint64_t 	set_ovfl_pmds[PFM_PMD_BV]; /* output: last ovfl PMDs which triggered a switch from set */
         uint64_t	set_runs;           /* output: number of times the set was active */
-        uint64_t	set_timeout;        /* output:effective/leftover switch timeout in nsecs */
+        uint32_t	set_timeout;        /* output:effective switch timeout     */
+	uint32_t	set_reserved1;	    /* for future use */
         uint64_t	set_act_duration;   /* DO NOT USE: number of cycles set was active (syswide only) */
 	uint64_t	set_mmap_offset;    /* cookie to pass as mmap offset to access 64-bit virtual PMD */
 	uint64_t	set_avail_pmcs[PFM_PMC_BV];
@@ -200,12 +221,12 @@ typedef union {
  * perfmon version number
  */
 #define PFM_VERSION_MAJ		 2U
-#define PFM_VERSION_MIN		 4U
+#define PFM_VERSION_MIN		 2U
 #define PFM_VERSION		 (((PFM_VERSION_MAJ&0xffff)<<16)|(PFM_VERSION_MIN & 0xffff))
 #define PFM_VERSION_MAJOR(x)	 (((x)>>16) & 0xffff)
 #define PFM_VERSION_MINOR(x)	 ((x) & 0xffff)
 
-extern int pfm_create_context(pfarg_ctx_t *ctx, char *smpl_name, void *smpl_arg, size_t smpl_size);
+extern int pfm_create_context(pfarg_ctx_t *ctx, void *smpl_arg, size_t smpl_size);
 extern int pfm_write_pmcs(int fd, pfarg_pmc_t *pmcs, int count);
 extern int pfm_write_pmds(int fd, pfarg_pmd_t *pmds, int count);
 extern int pfm_read_pmds(int fd, pfarg_pmd_t *pmds, int count);
@@ -218,54 +239,6 @@ extern int pfm_getinfo_evtsets(int fd, pfarg_setinfo_t *info, int count);
 extern int pfm_delete_evtsets(int fd, pfarg_setdesc_t *setd, int count);
 extern int pfm_unload_context(int fd);
 
-/*
- * until the syscall stubs are implemented by glibc
- * we define them here
- */
-#ifndef __NR_pfm_create_context
-#ifdef __x86_64__
-#define __NR_pfm_create_context		280
-#endif /* __x86_64__ */
-
-#ifdef __i386__
-#define __NR_pfm_create_context		320
-#endif
-
-#ifdef __ia64__
-#define __NR_pfm_create_context		1305
-#endif
-
-#if defined(__mips__)
-/* Add 12 */
-#if (_MIPS_SIM == _ABIN32) || (_MIPS_SIM == _MIPS_SIM_NABI32)
-#define __NR_Linux 6000
-#define __NR_pfm_create_context         __NR_Linux+279
-#elif (_MIPS_SIM == _ABI32) || (_MIPS_SIM == _MIPS_SIM_ABI32)
-#define __NR_Linux 4000
-#define __NR_pfm_create_context         __NR_Linux+316
-#elif (_MIPS_SIM == _ABI64) || (_MIPS_SIM == _MIPS_SIM_ABI64)
-#define __NR_Linux 5000
-#define __NR_pfm_create_context         __NR_Linux+275
-#endif
-#endif
-
-#ifdef __powerpc__
-#define __NR_pfm_create_context		302
-#endif
-
-#define __NR_pfm_write_pmcs		(__NR_pfm_create_context+1)
-#define __NR_pfm_write_pmds		(__NR_pfm_create_context+2)
-#define __NR_pfm_read_pmds		(__NR_pfm_create_context+3)
-#define __NR_pfm_load_context		(__NR_pfm_create_context+4)
-#define __NR_pfm_start			(__NR_pfm_create_context+5)
-#define __NR_pfm_stop			(__NR_pfm_create_context+6)
-#define __NR_pfm_restart		(__NR_pfm_create_context+7)
-#define __NR_pfm_create_evtsets		(__NR_pfm_create_context+8)
-#define __NR_pfm_getinfo_evtsets	(__NR_pfm_create_context+9)
-#define __NR_pfm_delete_evtsets		(__NR_pfm_create_context+10)
-#define __NR_pfm_unload_context		(__NR_pfm_create_context+11)
-#endif /* __NR_pfm_create_context */
-
 #ifdef __cplusplus
 };
 #endif
