@@ -1,5 +1,5 @@
 /*
- * Intel IA-32 architectural PMU
+ * IA-32 architected PMU
  *
  * Copyright (c) 2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
@@ -25,14 +25,7 @@
 #define __PFMLIB_GEN_IA32_H__
 
 #include <perfmon/pfmlib.h>
-/*
- * privilege level mask usage for architected IA-32 PMU:
- *
- * PFM_PLM0 = OS (kernel, hypervisor, ..)
- * PFM_PLM1 = unused (ignored)
- * PFM_PLM2 = unused (ignored)
- * PFM_PLM3 = USR (user level)
- */
+#include <endian.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -51,6 +44,11 @@ extern "C" {
  */
 #define PMU_GEN_IA32_COUNTER_WIDTH       32
 
+/*
+ * This structure provides a detailed way to setup a PMC register.
+ * Once value is loaded, it must be copied (via pmu_reg) to the
+ * perfmon_req_t and passed to the kernel via perfmonctl().
+ */
 typedef union {
 	unsigned long long val;			/* complete register value */
 	struct {
@@ -78,7 +76,7 @@ typedef struct {
 #define PFM_GEN_IA32_SEL_EDGE	0x2	/* edge detect */
 
 /*
- * model-specific parameters for the library
+ * P6-specific parameters for the library
  */
 typedef struct {
 	pfmlib_gen_ia32_counter_t	pfp_gen_ia32_counters[PMU_GEN_IA32_MAX_COUNTERS];
@@ -93,4 +91,4 @@ typedef struct {
 }
 #endif
 
-#endif /* __PFMLIB_GEN_IA32__H__ */
+#endif /* __PFMLIB_I386_P6_H__ */
