@@ -26,33 +26,28 @@
 #ifndef __PFMLIB_GEN_MIPS64_H__
 #define __PFMLIB_GEN_MIPS64_H__
 
-#include <endian.h> /* MIPS are bi-endian */
-
 #include <perfmon/pfmlib.h>
-/*
- * privilege level mask usage for MIPS:
- *
- * PFM_PLM0 = KERNEL
- * PFM_PLM1 = SUPERVISOR
- * PFM_PLM2 = INTERRUPT
- * PFM_PLM3 = USER
- */
+#include <endian.h>
+
+#if BYTE_ORDER != LITTLE_ENDIAN
+#error "this file only supports little endian environments"
+#endif
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#define PMU_GEN_MIPS64_NUM_COUNTERS		4	/* total numbers of EvtSel/EvtCtr */
+#define PMU_GEN_MIPS64_NUM_COUNTERS		2	/* total numbers of EvtSel/EvtCtr */
+#define PMU_GEN_MIPS64_NUM_PERFSEL		2	/* total number of EvtSel defined */
+#define PMU_GEN_MIPS64_NUM_PERFCTR		2	/* total number of EvtCtr defined */
 #define PMU_GEN_MIPS64_COUNTER_WIDTH		32	/* hardware counter bit width   */
+
+#define PMU_GEN_MIPS64_CNT_MASK_MAX		2 	/* max cnt_mask value */
 /*
  * This structure provides a detailed way to setup a PMC register.
  * Once value is loaded, it must be copied (via pmu_reg) to the
  * perfmon_req_t and passed to the kernel via perfmonctl().
- *
- * It needs to be adjusted based on endianess
  */
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-
 typedef union {
 	uint64_t	val;				/* complete register value */
 	struct {
@@ -63,30 +58,9 @@ typedef union {
 	        unsigned long sel_int:1;		/* enable intr */
 	  	unsigned long sel_event_mask:4;		/* event mask */
 		unsigned long sel_res1:23;		/* reserved */
-		unsigned long sel_res2:32;		/* reserved */
 	} perfsel;
 } pfm_gen_mips64_sel_reg_t;
 
-#elif __BYTE_ORDER == __BIG_ENDIAN
-
-typedef union {
-	uint64_t	val;				/* complete register value */
-	struct {
-		unsigned long sel_res2:32;		/* reserved */
-		unsigned long sel_res1:23;		/* reserved */
-	  	unsigned long sel_event_mask:4;		/* event mask */
-	        unsigned long sel_int:1;		/* enable intr */
-		unsigned long sel_usr:1;		/* user level */
-		unsigned long sel_sup:1;		/* supervisor level */
-		unsigned long sel_os:1;			/* system level */
-	        unsigned long sel_exl:1;		/* int level */
-	} perfsel;
-} pfm_gen_mips64_sel_reg_t;
-
-#else
-#error "cannot determine endianess"
-#endif
-
 typedef union {
 	uint64_t val;	/* counter value */
 	/* counting perfctr register */
