@@ -21,7 +21,6 @@
  * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#define _GNU_SOURCE /* for getline */
 #include <sys/types.h>
 #include <ctype.h>
 #include <string.h>
@@ -34,6 +33,16 @@
 
 #include "pfmlib_priv.h"
 
+extern pfm_pmu_support_t montecito_support;
+extern pfm_pmu_support_t itanium2_support;
+extern pfm_pmu_support_t itanium_support;
+extern pfm_pmu_support_t generic_ia64_support;
+extern pfm_pmu_support_t amd_x86_64_support;
+extern pfm_pmu_support_t i386_p6_support;
+extern pfm_pmu_support_t i386_pm_support;
+extern pfm_pmu_support_t gen_ia32_support;
+extern pfm_pmu_support_t generic_mips64_support;
+
 static pfm_pmu_support_t *supported_pmus[]=
 {
 #ifdef CONFIG_PFMLIB_MONTECITO
@@ -48,27 +57,17 @@ static pfm_pmu_support_t *supported_pmus[]=
 	&itanium_support,
 #endif
 
-#ifdef CONFIG_PFMLIB_AMD64
-	&amd64_support,
+#ifdef CONFIG_PFMLIB_AMD_X86_64
+	&amd_x86_64_support,
 #endif
 
-
 #ifdef CONFIG_PFMLIB_I386_P6
 	&i386_p6_support,
 	&i386_pm_support,
 #endif
 
-#ifdef CONFIG_PFMLIB_PENTIUM4
-	&pentium4_support,
-#endif
-
-#ifdef CONFIG_PFMLIB_CORE
-	&core_support,
-#endif
-
 #ifdef CONFIG_PFMLIB_GEN_IA32
-	&coreduo_support,
-	&gen_ia32_support,
+	&gen_ia32_support, /* must always be last of i386 arch */
 #endif
 
 #ifdef CONFIG_PFMLIB_GEN_MIPS64
@@ -91,22 +90,12 @@ int
 pfm_initialize(void)
 {
 	pfm_pmu_support_t **p = supported_pmus;
-
-	while(*p) {
-		if ((*p)->pmu_detect() == PFMLIB_SUCCESS)
-			goto found;
+	while (*p) {
+		if ((*p)->pmu_detect() == PFMLIB_SUCCESS) goto found;
 		p++;
 	}
 	return PFMLIB_ERR_NOTSUPP;
 found:
-	/*
-	 * run a few sanity checks
-	 */
-	if ((*p)->pmc_count >= PFMLIB_MAX_PMCS)
-		return PFMLIB_ERR_NOTSUPP;
-	if ((*p)->pmd_count >= PFMLIB_MAX_PMDS)
-		return PFMLIB_ERR_NOTSUPP;
-
 	pfm_current = *p;
 	return PFMLIB_SUCCESS;
 }
@@ -218,43 +207,44 @@ pfm_force_pmu(int type)
 	}
 	return PFMLIB_ERR_NOTSUPP;
 found:
+	/* verify that this is valid */
+	if ((*p)->pmu_detect() != PFMLIB_SUCCESS) return PFMLIB_ERR_NOTSUPP;
+
 	pfm_current = *p;
 
 	return PFMLIB_SUCCESS;
 }
 
+static int
+pfm_gen_event_code(const char *str, unsigned long *code)
+{
+	long ret;
+	char *endptr = NULL;
+
+	if (str == NULL || code == NULL) return PFMLIB_ERR_INVAL;
+
+	ret = strtoul(str,&endptr, 0);
+
+	/* check for errors */
+	if (*endptr!='\0') return PFMLIB_ERR_INVAL;
+
+	*code = ret;
+
+	return PFMLIB_SUCCESS;
+}
+
 int
 pfm_find_event_byname(const char *n, unsigned int *idx)
 {
-	char *p, *e;
-	unsigned int i;
-	size_t len;
+	 unsigned int i;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (n == NULL || idx == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (n == NULL || idx == NULL) return PFMLIB_ERR_INVAL;
 
-	/*
-	 * this function ignores any ':' separator
-	 */
-	p = strchr(n, ':');
-	if (!p)
-		len = strlen(n);
-	else
-		len = p - n;
-
-	/*
-	 * we do case insensitive comparisons
-	 *
-	 * event names must match completely
-	 */
+	/* we do case insensitive comparisons */
 	for(i=0; i < pfm_current->pme_count; i++) {
-		e = pfm_current->get_event_name(i);
-		if (!strncasecmp(e, n, len)
-		    && len == strlen(e))
-			goto found;
+		if (!strcasecmp(pfm_current->get_event_name(i), n)) goto found;
 	}
 	return PFMLIB_ERR_NOTFOUND;
 found:
@@ -274,6 +264,7 @@ pfm_find_event_bycode(int code, unsigned int *idx)
 	if (idx == NULL) return PFMLIB_ERR_INVAL;
 
 	if (pfm_current->flags & PFMLIB_MULT_CODE_EVENT) {
+
 		pfm_current->get_impl_counters(&impl_cnt);
 		num_cnt = pfm_current->num_cnt;
 
@@ -303,27 +294,24 @@ int
 pfm_find_event(const char *v, unsigned int *ev)
 {
 	unsigned long number;
-	char *endptr = NULL;
-	int ret = PFMLIB_ERR_INVAL;
+	int ret;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (v == NULL || ev == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (v == NULL || ev == NULL) return PFMLIB_ERR_INVAL;
 
 	if (isdigit((int)*v)) {
-		number = strtoul(v,&endptr, 0);
-		/* check for errors */
-		if (*endptr!='\0')
-			return PFMLIB_ERR_INVAL;
+		ret = pfm_gen_event_code(v, &number);
+		if (ret != PFMLIB_SUCCESS) return ret;
 
 		if (number <= INT_MAX) {
 			int the_int_number = (int)number;
+
 			ret = pfm_find_event_bycode(the_int_number, ev);
 		}
-	} else 
+	} else {
 		ret = pfm_find_event_byname(v, ev);
+	}
 	return ret;
 }
 
@@ -332,11 +320,9 @@ pfm_find_event_bycode_next(int code, unsigned int i, unsigned int *next)
 {
 	int code2;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (!next)
-		return PFMLIB_ERR_INVAL;
+	if (next == NULL) return PFMLIB_ERR_INVAL;
 
 	for(++i; i < pfm_current->pme_count; i++) {
 		pfm_current->get_event_code(i, PFMLIB_CNT_FIRST, &code2);
@@ -349,31 +335,30 @@ found:
 }
 
 int
-pfm_find_event_mask(unsigned int ev, const char *str, unsigned int *mask_idx)
+pfm_find_event_mask(unsigned int event_idx, const char *str, unsigned int *mask_idx)
 {
 	unsigned int i, num_masks = 0;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (str == NULL || mask_idx == NULL || ev >= pfm_current->pme_count)
-		return PFMLIB_ERR_INVAL;
+	if (str == NULL || mask_idx == NULL) return PFMLIB_ERR_INVAL;
 
-	num_masks = pfm_num_masks(ev);
-	for (i = 0; i < num_masks; i++) {
-		if (!strcasecmp(pfm_current->get_event_mask_name(ev, i), str)) {
-			*mask_idx = i;
-			return PFMLIB_SUCCESS;
+	if (PFMLIB_HAS_EVENT_MASKS()) {
+		pfm_current->get_num_event_masks(event_idx, &num_masks);
+		for (i = 0; i < num_masks; i++) {
+			if (!strcasecmp(pfm_current->get_event_mask_name(event_idx, i), str)) {
+				*mask_idx = i;
+				return PFMLIB_SUCCESS;
+			}
 		}
 	}
-	return PFMLIB_ERR_UMASK;
+
+	return PFMLIB_ERR_NOTFOUND;
 }
 
 int
 pfm_get_event_name(unsigned int i, char *name, size_t maxlen)
 {
-	size_t l, j;
-
 	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
 	if (i >= pfm_current->pme_count || name == NULL || maxlen < 1) return PFMLIB_ERR_INVAL;
@@ -381,9 +366,6 @@ pfm_get_event_name(unsigned int i, char *name, size_t maxlen)
 	strncpy(name, pfm_current->get_event_name(i), maxlen-1);
 
 	name[maxlen-1] = '\0';
-	l = strlen(name);
-	for(j=0; j < l; j++)
-		name[j] = (char)toupper(name[j]);
 
 	return PFMLIB_SUCCESS;
 }
@@ -422,42 +404,19 @@ pfm_get_event_counters(unsigned int i, pfmlib_regmask_t *counters)
 }
 
 int
-pfm_get_event_mask_name(unsigned int ev, unsigned int mask, char *name, size_t maxlen)
+pfm_get_event_mask_name(unsigned int event_idx, unsigned int mask_idx, char *name, size_t maxlen)
 {
-	char *n;
-	unsigned int num;
-	size_t l, j;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (event_idx >= pfm_current->pme_count || name == NULL || maxlen < 1) return PFMLIB_ERR_INVAL;
 
-	if (ev >= pfm_current->pme_count || name == NULL || maxlen < 1)
-		return PFMLIB_ERR_INVAL;
-
-	num = pfm_num_masks(ev);
-	if (num == 0)
+	if (!PFMLIB_HAS_EVENT_MASKS()) {
 		return PFMLIB_ERR_NOTSUPP;
+	}
 
-	if (mask >= num)
-		return PFMLIB_ERR_INVAL;
+	strncpy(name, pfm_current->get_event_mask_name(event_idx, mask_idx), maxlen-1);
+	name[maxlen-1] = '\0';
 
-	n = pfm_current->get_event_mask_name(ev, mask);
-	if (n == NULL)
-		return PFMLIB_ERR_INVAL;
-
-	l = strlen(n);
-	if (l >= (maxlen-1))
-		return PFMLIB_ERR_FULL;
-
-	strcpy(name, n);
-
-	/*
-	 * present nice uniform names
-	 */
-	l = strlen(name);
-	for(j=0; j < l; j++)
-		if (islower(name[j]))
-				name[j] = (char)toupper(name[j]);
 	return PFMLIB_SUCCESS;
 }
 
@@ -474,16 +433,145 @@ pfm_get_num_events(unsigned int *count)
 }
 
 int
-pfm_get_num_event_masks(unsigned int ev, unsigned int *count)
+pfm_get_num_event_masks(unsigned int event_idx, unsigned int *count)
 {
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (ev >= pfm_current->pme_count || count == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (event_idx >= pfm_current->pme_count || count == NULL) return PFMLIB_ERR_INVAL;
 
-	*count = pfm_num_masks(ev);
+	if (!PFMLIB_HAS_EVENT_MASKS()) {
+		*count = 0;
+		return PFMLIB_SUCCESS;
+	}
 
+	return pfm_current->get_num_event_masks(event_idx, count);
+}
+
+/*
+ * Function used to print information about a specific events. More than
+ * one event can be printed in case an event code is given rather than
+ * a specific name. A callback function is used for printing.
+ */
+int
+pfm_print_event_info(const char *name, int (*pf)(const char *fmt,...))
+{
+	unsigned long number;
+	pfmlib_regmask_t cmask, impl_counters;
+        int code, ret, n;
+	int event_is_digit = 0;
+	unsigned int idx, next_idx, num_counters, i;
+	unsigned int num_masks;
+
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
+
+	if (name == NULL || pf == NULL) return PFMLIB_ERR_INVAL;
+
+	num_counters = pfm_current->num_cnt;
+
+	/* we can't quite use pfm_findevent() because we need to try
+	 * both ways systematically.
+	 */
+	if (isdigit((int)*name)) {
+
+		event_is_digit = 1;
+
+		ret = pfm_gen_event_code(name, &number);
+		if (ret != PFMLIB_SUCCESS) return ret;
+
+		if (number <= INT_MAX) {
+			int the_int_number = (int)number;
+
+			ret = pfm_find_event_bycode(the_int_number, &idx);
+			if (ret == PFMLIB_SUCCESS) goto start_loop;
+		}
+	} else {
+		ret  = pfm_find_event_byname(name, &idx);
+	}
+
+	if (ret != PFMLIB_SUCCESS) return PFMLIB_ERR_NOTFOUND;
+
+
+start_loop:
+	do {
+		pfm_current->get_event_code(idx, PFMLIB_CNT_FIRST, &code);
+
+		(*pf)(	"Name   : %s\n"
+			"Code   : 0x%x\n",
+			pfm_current->get_event_name(idx),
+			code);
+		
+		(*pf)(	"counter: [ ");
+
+		pfm_current->get_event_counters(idx, &cmask);
+		n = num_counters;
+		for (i=0; n; i++) {
+			if (pfm_regmask_isset(&impl_counters, i)) n--;
+			if (pfm_regmask_isset(&cmask, i)) (*pf)("%d ", i);
+		}
+		(*pf)(	"]\n");
+
+		if (PFMLIB_HAS_EVENT_MASKS()) {
+			pfm_current->get_num_event_masks(idx, &num_masks);
+			for (i = 0; i < num_masks; i++) {
+				(*pf)("Unit-mask %d: %s\n",
+					i, pfm_current->get_event_mask_name(idx, i));
+			}
+		}
+
+		/* print PMU specific information */
+		if (pfm_current->print_info) {
+			pfm_current->print_info(idx, pf);
+		}
+		ret = pfm_find_event_bycode_next(code, idx, &next_idx);
+		idx = next_idx;
+
+	} while (event_is_digit && ret == PFMLIB_SUCCESS);
+
+	return PFMLIB_SUCCESS;
+}
+
+int
+pfm_print_event_info_byindex(unsigned int v, int (*pf)(const char *fmt,...))
+{
+	pfmlib_regmask_t cmask, impl_counters;
+	unsigned int i, n, num_masks;
+	int code;
+
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
+
+	if (v >= pfm_current->pme_count || pf == NULL) return PFMLIB_ERR_INVAL;
+
+	pfm_current->get_event_code(v, PFMLIB_CNT_FIRST, &code);
+
+	(*pf)(	"Name   : %s\n"
+		"Code   : 0x%x\n",
+		pfm_current->get_event_name(v),
+		code);
+	
+	(*pf)(	"counter: [ ");
+
+	n = pfm_current->num_cnt;
+	pfm_current->get_event_counters(v, &cmask);
+	pfm_current->get_impl_counters(&impl_counters);
+
+	for (i=0; n; i++) {
+		if (pfm_regmask_isset(&impl_counters, i)) n--;
+		if (pfm_regmask_isset(&cmask, i)) (*pf)("%d ", i);
+	}
+	(*pf)(	"]\n");
+
+	if (PFMLIB_HAS_EVENT_MASKS()) {
+		pfm_current->get_num_event_masks(v, &num_masks);
+		for (i = 0; i < num_masks; i++) {
+			(*pf)("Unit-mask %d: %s\n",
+				v, pfm_current->get_event_mask_name(v, i));
+		}
+	}
+
+	/* print PMU specific information */
+	if (pfm_current->print_info) {
+		pfm_current->print_info(v, pf);
+	}
 	return PFMLIB_SUCCESS;
 }
 
@@ -507,7 +595,6 @@ pfm_check_unavail_pmcs(pfmlib_regmask_t *pmcs)
 }
 #endif
 
-
 /*
  * we do not check if pfp_unavail_pmcs contains only implemented PMC
  * registers. In other words, invalid registers are ignored
@@ -515,39 +602,32 @@ pfm_check_unavail_pmcs(pfmlib_regmask_t *pmcs)
 int
 pfm_dispatch_events(pfmlib_input_param_t *inp, void *model_in, pfmlib_output_param_t *outp, void *model_out)
 {
-	unsigned count;
+	unsigned int max_count, count;
 	unsigned int i;
-	int ret;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 
-	if (inp == NULL || outp == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (inp == NULL || outp == NULL) return PFMLIB_ERR_INVAL;
 
 	/*
 	 * the default priv level must be set to something
 	 */
-	if (inp->pfp_dfl_plm == 0)
-		return PFMLIB_ERR_INVAL;
+	if (inp->pfp_dfl_plm == 0) return PFMLIB_ERR_INVAL;
 
-	if (inp->pfp_event_count >= PFMLIB_MAX_PMCS)
-		return PFMLIB_ERR_INVAL;
+	if (inp->pfp_event_count >= PFMLIB_MAX_PMCS) return PFMLIB_ERR_INVAL;
 
-	count = inp->pfp_event_count;
-	if (count > pfm_current->num_cnt)
-		return PFMLIB_ERR_NOASSIGN;
+	max_count = pfm_current->pme_count;
+	count     = inp->pfp_event_count;
 
 	/*
-	 * check that event and unit masks descriptors are correct
+	 * check that event descriptors are correct
 	 */
 	for (i=0; i < count; i++) {
-		ret = __pfm_check_event(inp->pfp_events+i);
-		if (ret != PFMLIB_SUCCESS)
-			return ret;
+		if (inp->pfp_events[i].event >= max_count) return PFMLIB_ERR_INVAL;
 	}
-	/* reset output data structure */
-	memset(outp, 0, sizeof(*outp));
+
+	/* reset pc counter */
+	outp->pfp_pmc_count = 0;
 
 	return pfm_current->dispatch_events(inp, model_in, outp, model_out);
 }
@@ -597,8 +677,6 @@ pfm_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 	if (impl_pmcs == NULL) return PFMLIB_ERR_INVAL;
 
-	memset(impl_pmcs , 0, sizeof(*impl_pmcs));
-
 	pfm_current->get_impl_pmcs(impl_pmcs);
 
 	return PFMLIB_SUCCESS;
@@ -610,8 +688,6 @@ pfm_get_impl_pmds(pfmlib_regmask_t *impl_pmds)
 	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 	if (impl_pmds == NULL) return PFMLIB_ERR_INVAL;
 
-	memset(impl_pmds, 0, sizeof(*impl_pmds));
-
 	pfm_current->get_impl_pmds(impl_pmds);
 
 	return PFMLIB_SUCCESS;
@@ -623,8 +699,6 @@ pfm_get_impl_counters(pfmlib_regmask_t *impl_counters)
 	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
 	if (impl_counters == NULL) return PFMLIB_ERR_INVAL;
 
-	memset(impl_counters, 0, sizeof(*impl_counters));
-
 	pfm_current->get_impl_counters(impl_counters);
 
 	return PFMLIB_SUCCESS;
@@ -649,15 +723,15 @@ static char *pfmlib_err_list[]=
 	"not supported",
 	"invalid parameters",
 	"pfmlib not initialized",
-	"event not found",
+	"object not found",
 	"cannot assign events to counters",
-	"buffer is full or too small",
+	"buffer is full (obsolete)",
 	"event used more than once",
 	"invalid model specific magic number",
 	"invalid combination of model specific features",
 	"incompatible event sets",
 	"incompatible events combination",
-	"too many events or unit masks",
+	"too many events",
 	"code range too big",
 	"empty code range",
 	"invalid code range",
@@ -667,8 +741,6 @@ static char *pfmlib_err_list[]=
 	"not supported by host cpu",
 	"code range is not bundle-aligned",
 	"code range requires some flags in rr_flags",
-	"invalid or missing unit mask",
-	"out of memory"
 };
 static size_t pfmlib_err_count = sizeof(pfmlib_err_list)/sizeof(char *);
 
@@ -680,6 +752,18 @@ pfm_strerror(int code)
 	return pfmlib_err_list[code];
 }
 
+void
+__pfm_vbprintf(const char *fmt, ...)
+{
+	va_list ap;
+
+	if (pfm_config.options.pfm_verbose == 0) return;
+
+	va_start(ap, fmt);
+	vprintf(fmt, ap);
+	va_end(ap);
+}
+
 int
 pfm_get_version(unsigned int *version)
 {
@@ -689,71 +773,69 @@ pfm_get_version(unsigned int *version)
 }
 
 int
-pfm_get_max_event_name_len(size_t *len)
+pfm_get_max_name_len(size_t *len)
 {
 	unsigned int i, j, num_masks;
 	size_t max = 0, l;
 
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
-	if (len == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
+	if (len == NULL) return PFMLIB_ERR_INVAL;
 
 	for(i=0; i < pfm_current->pme_count; i++) {
 		l = strlen(pfm_current->get_event_name(i));
 		if (l > max) max = l;
 
-		num_masks = pfm_num_masks(i);
-		/*
-		 * we need to add up all length because unit masks can
-		 * be combined typically. We add 1 to account for ':'
-		 * which is inserted as the unit mask separator
-		 */
-		for (j = 0; j < num_masks; j++) {
-			l += 1 + strlen(pfm_current->get_event_mask_name(i, j));
+		if (PFMLIB_HAS_EVENT_MASKS()) {
+			pfm_current->get_num_event_masks(i, &num_masks);
+			for (j = 0; j < num_masks; j++) {
+				l = strlen(pfm_current->get_event_mask_name(i, j));
+				if (l > max) max = l;
+			}
 		}
-		if (l > max) max = l;
 	}
 	*len = max;
+
 	return PFMLIB_SUCCESS;
 }
 
+int
+pfm_get_max_event_name_len(size_t *len)
+{
+	return pfm_get_max_name_len(len);
+}
+
 /*
  * return the index of the event that counts elapsed cycles
  */
 int
-pfm_get_cycle_event(pfmlib_event_t *e)
+pfm_get_cycle_event(unsigned int *ev)
 {
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
-	if (e == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
+	if (ev == NULL) return PFMLIB_ERR_INVAL;
 
-	if (!pfm_current->get_cycle_event)
+	if (pfm_current->cycle_event == PFMLIB_NO_EVT)
 		return PFMLIB_ERR_NOTSUPP;
 
-	memset(e, 0, sizeof(*e));
+	*ev = pfm_current->cycle_event;
 
-	return pfm_current->get_cycle_event(e);
+	return PFMLIB_SUCCESS;
 }
 
 /*
  * return the index of the event that retired instructions
  */
 int
-pfm_get_inst_retired_event(pfmlib_event_t *e)
+pfm_get_inst_retired_event(unsigned int *ev)
 {
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
-	if (e == NULL)
-		return PFMLIB_ERR_INVAL;
+	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
+	if (ev == NULL) return PFMLIB_ERR_INVAL;
 
-	if (!pfm_current->get_inst_retired_event)
+	if (pfm_current->inst_retired_event == PFMLIB_NO_EVT)
 		return PFMLIB_ERR_NOTSUPP;
 
-	memset(e, 0, sizeof(*e));
+	*ev = pfm_current->inst_retired_event;
 
-	return pfm_current->get_inst_retired_event(e);
+	return PFMLIB_SUCCESS;
 }
 
 int
@@ -764,7 +846,7 @@ pfm_get_event_description(unsigned int i, char **str)
 	if (i >= pfm_current->pme_count || str == NULL) return PFMLIB_ERR_INVAL;
 
 	if (pfm_current->get_event_desc == NULL) {
-		*str = strdup("no description available");
+		*str = NULL;	
 		return PFMLIB_SUCCESS;
 	}
 	return pfm_current->get_event_desc(i, str);
@@ -778,166 +860,10 @@ pfm_get_event_mask_description(unsigned int event_idx, unsigned int mask_idx, ch
 	if (event_idx >= pfm_current->pme_count || desc == NULL) return PFMLIB_ERR_INVAL;
 
 	if (pfm_current->get_event_mask_desc == NULL) {
-		*desc = strdup("no description available");
+		*desc = NULL;
 		return PFMLIB_SUCCESS;
 	}
-	if (mask_idx >= pfm_current->get_num_event_masks(event_idx))
-		return PFMLIB_ERR_INVAL;
 
 	return pfm_current->get_event_mask_desc(event_idx, mask_idx, desc);
 }
 
-int
-pfm_get_event_mask_code(unsigned int event_idx, unsigned int mask_idx, unsigned int *code)
-{
-	if (PFMLIB_INITIALIZED() == 0) return PFMLIB_ERR_NOINIT;
-
-	if (event_idx >= pfm_current->pme_count || code == NULL) return PFMLIB_ERR_INVAL;
-
-	if (pfm_current->get_event_mask_code == NULL) {
-		*code = 0;
-		return PFMLIB_SUCCESS;
-	}
-	if (mask_idx >= pfm_current->get_num_event_masks(event_idx))
-		return PFMLIB_ERR_INVAL;
-
-	return pfm_current->get_event_mask_code(event_idx, mask_idx, code);
-}
-	
-int
-pfm_get_full_event_name(pfmlib_event_t *e, char *name, size_t maxlen)
-{
-	char *n;
-	size_t l, j;
-	int ret;
-
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
-
-	if (e == NULL || name == NULL || maxlen < 1)
-		return PFMLIB_ERR_INVAL;
-
-	ret = __pfm_check_event(e);
-	if (ret != PFMLIB_SUCCESS)
-		return ret;
-
-	/*
-	 * make sure the string is at least empty
-	 * important for programs that do not check return value
-	 * from this function!
-	 */
-	*name = '\0';
-
-	n = pfm_current->get_event_name(e->event);
-	l = strlen(n);
-	if (l > (maxlen-1))
-		return PFMLIB_ERR_FULL;
-
-	strcpy(name, n);
-	maxlen -= l + 1;
-	for(j=0; j < e->num_masks; j++) {
-		n = pfm_current->get_event_mask_name(e->event, e->unit_masks[j]);
-		l = strlen(n);
-		if (l > (maxlen-1))
-			return PFMLIB_ERR_FULL;
-
-		strcat(name, ":");
-		strcat(name, n);
-		maxlen -= l + 1;
-	}
-	/*
-	 * present nice uniform names
-	 */
-	l = strlen(name);
-	for(j=0; j < l; j++)
-		if (islower(name[j]))
-				name[j] = (char)toupper(name[j]);
-	return PFMLIB_SUCCESS;
-}
-	
-int
-pfm_find_full_event(const char *v, pfmlib_event_t *e)
-{
-	char *str, *p, *q;
-	unsigned int j, mask;
-	int ret = PFMLIB_SUCCESS;
-
-	if (PFMLIB_INITIALIZED() == 0)
-		return PFMLIB_ERR_NOINIT;
-
-	if (v == NULL || e == NULL)
-		return PFMLIB_ERR_INVAL;
-
-	memset(e, 0, sizeof(*e));
-
-	/*
-	 * must copy string because we modify it when parsing
-	 */
-	str = strdup(v);
-	if (!str)
-		return PFMLIB_ERR_NOMEM;
-
-	/*
-	 * find event. this function ignores ':' separator
-	 */
-	ret = pfm_find_event_byname(str, &e->event);
-	if (ret)
-		goto error;
-
-	/*
-	 * get number of unit masks for event
-	 */
-	j = pfm_num_masks(e->event);
-
-	/*
-	 * look for colon (unit mask separator)
-	 */
-	p = strchr(str, ':');
-
-	/*
-	 * error if:
-	 * 	- event has unit masks and non is passed
-	 * 	- event has no unit mask and at least one is passed
-	 */
-	if ((!p && j) || (p && !j)) {
-		ret = PFMLIB_ERR_UMASK;
-		goto error;
-	}
-
-	/*
-	 * if no unit mask, then we are done
-	 */
-	if (!j) {
-		free(str);
-		return PFMLIB_SUCCESS;
-	}
-
-	p++;
-
-	for( q = p; q ; p = q) {
-		q = strchr(p,':');
-		if (q)
-			*q++ = '\0';
-
-		ret = pfm_find_event_mask(e->event, p, &mask);
-		if (ret != PFMLIB_SUCCESS)
-			break;
-
-		/* avoid duplicates */
-		for(j=0; j < e->num_masks; j++)
-			if (e->unit_masks[j] == mask)
-				break;
-
-		if (j == e->num_masks) {
-			if (e->num_masks == PFMLIB_MAX_MASKS_PER_EVENT) {
-				ret = PFMLIB_ERR_TOOMANY;
-				break;
-			}
-			e->unit_masks[e->num_masks] = mask;
-			e->num_masks++;
-		}
-	}
-error:
-	free(str);
-	return ret;
-}
