@@ -1,5 +1,5 @@
 /*
- * pfmlib_gen_ia32.c : IA-32 architectural PMU 
+ * pfmlib_gen_ia32.c : support for architected IA-32 PMU
  *
  * Copyright (c) 2005-2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
@@ -22,8 +22,8 @@
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *
- * This file implements supports for the IA-32 architectural PMU as specified
- * in the following document:
+ * This file implements supports for the IA-32 architected PMU as specified in the
+ * following document:
  * 	"IA-32 Intel Architecture Software Developer's Manual - Volume 3B: System
  * 	Programming Guide, Part 2"
  * 	Order Number: 253669-019
@@ -33,7 +33,6 @@
 #include <sys/types.h>
 #include <ctype.h>
 #include <string.h>
-#include <stdlib.h>
 #include <stdio.h>
 
 /* public headers */
@@ -42,8 +41,6 @@
 /* private headers */
 #include "pfmlib_priv.h"			/* library private */
 #include "pfmlib_gen_ia32_priv.h"		/* architecture private */
-#include "gen_ia32_events.h"
-#include "coreduo_events.h"
 
 /* let's define some handy shortcuts! */
 #define sel_event_select perfevtsel.sel_event_select
@@ -57,35 +54,58 @@
 #define sel_inv		 perfevtsel.sel_inv
 #define sel_cnt_mask	 perfevtsel.sel_cnt_mask
 
-pfm_pmu_support_t coreduo_support;
 pfm_pmu_support_t gen_ia32_support;
-pfm_pmu_support_t *gen_support;
-
-/*
- * Description of the PMC register mappings use by
- * this module (as reported in pfmlib_reg_t.reg_num):
- *
- * 0 -> PMC0 -> PERFEVTSEL0 -> MSR @ 0x186
- * 1 -> PMC1 -> PERFEVTSEL1 -> MSR @ 0x187
- * n -> PMCn -> PERFEVTSELn -> MSR @ 0x186+n
- * We do not use a mapping table, instead we make up the
- * values on the fly given the base.
- */
-#define GEN_IA32_SEL_BASE 0x186
-#define GEN_IA32_CTR_BASE 0xc1
-
-#define PFMLIB_GEN_IA32_ALL_FLAGS \
-	(PFM_GEN_IA32_SEL_INV|PFM_GEN_IA32_SEL_EDGE)
 
 static char * pfm_gen_ia32_get_event_name(unsigned int i);
 
+static pme_gen_ia32_entry_t gen_ia32_all_pe[]={
+#define PME_GEN_IA32_UNHALTED_CORE_CYCLES 0
+	{.pme_name = "UNHALTED_CORE_CYCLES",
+	 .pme_entry_code.pme_vcode = 0x003c,
+	 .pme_desc =  "count core clock cycles whenever the clock signal on the specific core is running (not halted)"
+	},
+#define PME_GEN_IA32_INSTRUCTIONS_RETIRED 1
+	{.pme_name = "INSTRUCTIONS_RETIRED",
+	 .pme_entry_code.pme_vcode = 0x00c0,
+	 .pme_desc =  "count the number of instructions at retirement. For instructions that consists of multiple mnicro-ops, this event counts the retirement of the last micro-op of the instruction",
+	},
+	{.pme_name = "UNHALTED_REFERENCE_CYCLES",
+	 .pme_entry_code.pme_vcode = 0x013c,
+	 .pme_desc =  "count reference clock cycles while the clock signal on the specific core is running. The reference clock operates at a fixed frequency, irrespective of core freqeuncy changes due to performance state transitions",
+	},
+	{.pme_name = "LAST_LEVEL_CACHE_REFERENCES",
+	 .pme_entry_code.pme_vcode = 0x4f2e,
+	 .pme_desc =  "count each request originating from the core to reference a cache line in the last level cache. The count may include speculation, but excludes cache line fills due to hardware prefetch",
+	},
+	{.pme_name = "LAST_LEVEL_CACHE_MISSES",
+	 .pme_entry_code.pme_vcode = 0x412e,
+	 .pme_desc =  "count each cache miss condition for references to the last level cache. The event count may include speculation, but excludes cache line fills due to hardware prefetch",
+	},
+	{.pme_name = "BRANCH_INSTRUCTIONS_RETIRED",
+	 .pme_entry_code.pme_vcode = 0x00c4,
+	 .pme_desc =  "count branch instructions at retirement. Specifically, this event counts the retirement of the last micro-op of a branch instruction",
+	},
+	{.pme_name = "ALL_BRANCH_MISPREDICT_RETIRED",
+	 .pme_entry_code.pme_vcode = 0x00c5,
+	 .pme_desc =  "count mispredicted branch instructions at retirement. Specifically, this event counts at retirement of the last micro-op of a branch instruction in the architectural path of the execution and experienced misprediction in the branch prediction hardware",
+	}
+};
+#define PFMLIB_GEN_IA32_DEF_NUM_EVENTS	(sizeof(gen_ia32_all_pe)/sizeof(pme_gen_ia32_entry_t))
+
 static pme_gen_ia32_entry_t *gen_ia32_pe;
 
-static int gen_ia32_cycle_event, gen_ia32_inst_retired_event;
+static inline unsigned int cpuid_eax(unsigned int op)
+{
+	unsigned int eax;
 
-#ifdef __i386__
-static inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx,
-			 unsigned int *ecx, unsigned int *edx)
+	__asm__("pushl %%ebx; cpuid; popl %%ebx"
+			: "=a" (eax)
+			: "0" (op)
+			: "ecx", "edx");
+	return eax;
+}
+
+static inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx)
 {
 	/*
 	 * because ebx is used in Pic mode, we need to save/restore because
@@ -103,95 +123,34 @@ static inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx,
 			: "a" (op)
 			: "ecx", "edx");
 }
-#else
-static inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx,
-			 unsigned int *ecx, unsigned int *edx)
-{
-        __asm__("cpuid"
-                        : "=a" (*eax),
-                        "=b" (*ebx),
-                        "=c" (*ecx),
-                        "=d" (*edx)
-                        : "0" (op), "c"(0));
-}
-#endif
 
 /*
- * create architected event table
+ * detect presence of architected PMU
  */
 static int
-create_arch_event_table(unsigned int mask)
-{
-	pme_gen_ia32_entry_t *pe;
-	unsigned int i, num_events = 0;
-	unsigned int m;
-
-	/*
-	 * first pass: count the number of supported events
-	 */
-	m = mask;
-	for(i=0; i < 7; i++, m>>=1) {
-		if ((m & 0x1)  == 0)
-			num_events++;
-	}
-	gen_ia32_support.pme_count = num_events;
-
-	gen_ia32_pe = calloc(num_events, sizeof(pme_gen_ia32_entry_t));
-	if (gen_ia32_pe == NULL)
-		return PFMLIB_ERR_NOTSUPP;
-
-	/*
-	 * second pass: populate the table
-	 */
-	gen_ia32_cycle_event = gen_ia32_inst_retired_event = -1;
-	m = mask;
-	for(i=0, pe = gen_ia32_pe; i < 7; i++, m>>=1) {
-		if ((m & 0x1)  == 0) {
-			*pe = gen_ia32_all_pe[i];
-			/*
-			 * setup default event: cycles and inst_retired
-			 */
-			if (i == PME_GEN_IA32_UNHALTED_CORE_CYCLES)
-				gen_ia32_cycle_event = pe - gen_ia32_pe;
-			if (i == PME_GEN_IA32_INSTRUCTIONS_RETIRED)
-				gen_ia32_inst_retired_event = pe - gen_ia32_pe;
-			pe++;
-		}
-	}
-	return PFMLIB_SUCCESS;
-}
-
-static int
-check_arch_pmu(int family)
+pfm_gen_ia32_detect(void)
 {
 	union {
 		unsigned int val;
 		pmu_eax_t eax;
-	} eax, ecx, edx, ebx;
-	int ret;
+	} eax;
+	pme_gen_ia32_entry_t *pe;
+	unsigned int ebx = 0, mask;
+	unsigned int i, num_events;
 
 	/*
-	 * check family number to reject for processors
-	 * older than Pentium (family=5). Those processors
-	 * did not have the CPUID instruction
-	 */
-	if (family < 5)
-		return PFMLIB_ERR_NOTSUPP;
-
-	/*
-	 * check if CPU supports 0xa function of CPUID
+	 * check if CPU supoprt 0xa function of CPUID
 	 * 0xa started with Core Duo. Needed to detect if
 	 * architected PMU is present
 	 */
-	cpuid(0x0, &eax.val, &ebx.val, &ecx.val, &edx.val);
+	cpuid(0x0, &eax.val, &ebx);
+
 	if (eax.val < 0xa)
 		return PFMLIB_ERR_NOTSUPP;
-
 	/*
 	 * extract architected PMU information
 	 */
-	cpuid(0xa, &eax.val, &ebx.val, &ecx.val, &edx.val);
-
+	cpuid(0xa, &eax.val, &ebx);
 	/*
 	 * check version. must be greater than zero
 	 */
@@ -204,77 +163,50 @@ check_arch_pmu(int family)
 	if (eax.eax.num_cnt == 0)
 		return PFMLIB_ERR_NOTSUPP;
 
+	/*
+	 * only support counter which air paired PERFEVTSEL/PMC
+	 * In the Intel terminology a counting PMD register is a PMC
+	 */
 	gen_ia32_support.pmc_count = eax.eax.num_cnt;
 	gen_ia32_support.pmd_count = eax.eax.num_cnt;
 	gen_ia32_support.num_cnt   = eax.eax.num_cnt;
 
-	ret = create_arch_event_table(ebx.val);
-	if (ret != PFMLIB_SUCCESS)
-		return ret;
+	num_events = 0;
+	mask = ebx;
+	/*
+	 * first pass: count the number of supported events
+	 */
+	for(i=0; i < 7; i++, mask>>=1) {
+		if ((mask & 0x1)  == 0)
+			num_events++;
+	}
 
-	gen_support = &gen_ia32_support;
-
-	return PFMLIB_SUCCESS;
-}
-
-static int
-pfm_gen_ia32_detect(void)
-{
-	int ret, family;
-	char buffer[128];
-
-	ret = __pfm_getcpuinfo_attr("vendor_id", buffer, sizeof(buffer));
-	if (ret == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	if (strcmp(buffer, "GenuineIntel"))
-		return PFMLIB_ERR_NOTSUPP;
-
-	ret = __pfm_getcpuinfo_attr("cpu family", buffer, sizeof(buffer));
-	if (ret == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	family = atoi(buffer);
-
-	return check_arch_pmu(family);
-}
-
-static int
-pfm_coreduo_detect(void)
-{
-	int ret, family, model;
-	char buffer[128];
-
-	ret = __pfm_getcpuinfo_attr("vendor_id", buffer, sizeof(buffer));
-	if (ret == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	if (strcmp(buffer, "GenuineIntel"))
-		return PFMLIB_ERR_NOTSUPP;
-
-	ret = __pfm_getcpuinfo_attr("cpu family", buffer, sizeof(buffer));
-	if (ret == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	family = atoi(buffer);
-
-	ret = __pfm_getcpuinfo_attr("model", buffer, sizeof(buffer));
-	if (ret == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	model = atoi(buffer);
+	gen_ia32_support.pme_count = num_events;
 
 	/*
-	 * check for core solo/core duo
+	 * alloc event table
 	 */
-	if (family == 6 && model == 14) {
-		gen_ia32_pe = coreduo_pe;
-		gen_support = &coreduo_support;
-		gen_ia32_cycle_event = PME_COREDUO_UNHALTED_CORE_CYCLES;
-		gen_ia32_inst_retired_event = PME_COREDUO_INSTRUCTIONS_RETIRED;
-		return PFMLIB_SUCCESS;
+	gen_ia32_pe = malloc(num_events*sizeof(pme_gen_ia32_entry_t));
+	if (gen_ia32_pe == NULL)
+		return PFMLIB_ERR_NOTSUPP;
+	/*
+	 * second pass: populate event table
+	 */
+	mask = ebx;
+	for(i=0, pe = gen_ia32_pe; i < 7; i++, mask>>=1) {
+		if ((mask & 0x1)  == 0) {
+			*pe = gen_ia32_all_pe[i];
+			/*
+			 * setup default event: cycles and inst_retired
+			 */
+			if (i == PME_GEN_IA32_UNHALTED_CORE_CYCLES)
+				gen_ia32_support.cycle_event = pe - gen_ia32_pe;
+			if (i == PME_GEN_IA32_INSTRUCTIONS_RETIRED)
+				gen_ia32_support.inst_retired_event = pe - gen_ia32_pe;
+			pe++;
+		}
 	}
-	return PFMLIB_ERR_NOTSUPP;
+	return PFMLIB_SUCCESS;
 }
 
 static int
@@ -284,15 +216,14 @@ pfm_gen_ia32_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_gen_ia32_input_
 	pfmlib_gen_ia32_counter_t *cntrs;
 	pfm_gen_ia32_sel_reg_t reg;
 	pfmlib_event_t *e;
-	pfmlib_reg_t *pc, *pd;
+	pfmlib_reg_t *pc;
 	pfmlib_regmask_t *r_pmcs;
 	unsigned long plm;
-	unsigned int i, j, cnt, k, umask;
+	unsigned int i, j, cnt;
 	unsigned int assign[PMU_GEN_IA32_MAX_COUNTERS];
 
 	e      = inp->pfp_events;
 	pc     = outp->pfp_pmcs;
-	pd     = outp->pfp_pmds;
 	cnt    = inp->pfp_event_count;
 	r_pmcs = &inp->pfp_unavail_pmcs;
 	cntrs  = param ? param->pfp_gen_ia32_counters : NULL;
@@ -303,7 +234,7 @@ pfm_gen_ia32_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_gen_ia32_input_
 		}
 	}
 
-	if (cnt > gen_support->pmc_count) return PFMLIB_ERR_TOOMANY;
+	if (cnt > gen_ia32_support.pmc_count) return PFMLIB_ERR_TOOMANY;
 
 	for(i=0, j=0; j < cnt; j++) {
 		/*
@@ -313,18 +244,13 @@ pfm_gen_ia32_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_gen_ia32_input_
 			DPRINT(("event=%d invalid plm=%d\n", e[j].event, e[j].plm));
 			return PFMLIB_ERR_INVAL;
 		}
-
-		if (e[j].flags & ~PFMLIB_GEN_IA32_ALL_FLAGS) {
-			DPRINT(("event=%d invalid flags=0x%lx\n", e[j].event, e[j].flags));
-			return PFMLIB_ERR_INVAL;
-		}
-
+		
 		/*
 		 * exclude restricted registers from assignement
 		 */
-		while(i < gen_support->pmc_count && pfm_regmask_isset(r_pmcs, i)) i++;
+		while(i < gen_ia32_support.pmc_count && pfm_regmask_isset(r_pmcs, i)) i++;
 
-		if (i == gen_support->pmc_count)
+		if (i == gen_ia32_support.pmc_count)
 			return PFMLIB_ERR_NOASSIGN;
 
 		/*
@@ -334,39 +260,36 @@ pfm_gen_ia32_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_gen_ia32_input_
 	}
 
 	for (j=0; j < cnt ; j++ ) {
-		reg.val = 0; /* assume reserved bits are zerooed */
+		reg.val    = 0; /* assume reserved bits are zerooed */
 
 		/* if plm is 0, then assume not specified per-event and use default */
 		plm = e[j].plm ? e[j].plm : inp->pfp_dfl_plm;
 
-		reg.sel_event_select = gen_ia32_pe[e[j].event].pme_code & 0xff;
-
-		umask = (gen_ia32_pe[e[j].event].pme_code >> 8) & 0xff;
-
-		for(k=0; k < e[j].num_masks; k++) {
-			umask |= gen_ia32_pe[e[j].event].pme_umasks[e[j].unit_masks[k]].pme_ucode;
-		}
-		reg.sel_unit_mask  = umask;
-		reg.sel_usr        = plm & PFM_PLM3 ? 1 : 0;
-		reg.sel_os         = plm & PFM_PLM0 ? 1 : 0;
-		reg.sel_en         = 1; /* force enable bit to 1 */
-		reg.sel_int        = 1; /* force APIC int to 1 */
-
+		reg.sel_event_select = gen_ia32_pe[e[j].event].pme_entry_code.pme_code.pme_sel;
+		reg.sel_unit_mask    = gen_ia32_pe[e[j].event].pme_entry_code.pme_code.pme_umask;
+		reg.sel_usr          = plm & PFM_PLM3 ? 1 : 0;
+		reg.sel_os           = plm & PFM_PLM0 ? 1 : 0;
+		reg.sel_en           = 1; /* force enable bit to 1 */
+		reg.sel_int          = 1; /* force APIC int to 1 */
 		if (cntrs) {
-			reg.sel_cnt_mask = cntrs[j].cnt_mask;
-			reg.sel_edge	 = cntrs[j].flags & PFM_GEN_IA32_SEL_EDGE ? 1 : 0;
-			reg.sel_inv	 = cntrs[j].flags & PFM_GEN_IA32_SEL_INV ? 1 : 0;
+			reg.sel_cnt_mask   = cntrs[j].cnt_mask;
+
+			/*
+			 * certain events require edge to be set
+			 */
+			reg.sel_edge	   = cntrs[j].flags & PFM_GEN_IA32_SEL_EDGE ? 1 : 0;
+			reg.sel_inv	   = cntrs[j].flags & PFM_GEN_IA32_SEL_INV ? 1 : 0;
 		}
 
+		/*
+		 * XXX: assumes perfmon2 mappings
+		 */
 		pc[j].reg_num     = assign[j];
-		pc[j].reg_addr    = GEN_IA32_SEL_BASE+assign[j];
+		pc[j].reg_pmd_num = assign[j];
+		pc[j].reg_evt_idx = j;
 		pc[j].reg_value   = reg.val;
 
-		pd[j].reg_num  = assign[j];
-		pd[j].reg_addr = GEN_IA32_CTR_BASE+assign[j];
-
-		__pfm_vbprintf("[PERFEVTSEL%u(pmc%u)=0x%llx event_sel=0x%x umask=0x%x os=%d usr=%d en=%d int=%d inv=%d edge=%d cnt_mask=%d] %s\n",
-			assign[j],
+		__pfm_vbprintf("[perfevtsel%u=0x%llx event_sel=0x%lx umask=0x%lx os=%d usr=%d en=%d int=%d inv=%d edge=%d cnt_mask=%d] %s\n",
 			assign[j],
 			reg.val,
 			reg.sel_event_select,
@@ -379,12 +302,9 @@ pfm_gen_ia32_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_gen_ia32_input_
 			reg.sel_edge,
 			reg.sel_cnt_mask,
 			gen_ia32_pe[e[j].event].pme_name);
-
-		__pfm_vbprintf("[PMC%u(pmd%u)]\n", pd[j].reg_num, pd[j].reg_num);
 	}
 	/* number of evtsel registers programmed */
 	outp->pfp_pmc_count = cnt;
-	outp->pfp_pmd_count = cnt;
 
 	return PFMLIB_SUCCESS;
 }
@@ -401,17 +321,46 @@ pfm_gen_ia32_dispatch_events(pfmlib_input_param_t *inp, void *model_in, pfmlib_o
 	return pfm_gen_ia32_dispatch_counters(inp, mod_in, outp);
 }
 
+/*
+ * Function used to print information about a specific events. More than
+ * one event can be printed in case an event code is given rather than
+ * a specific name. A callback function is used for printing.
+ */
 static int
-pfm_gen_ia32_get_event_code(unsigned int i, unsigned int cnt, int *code)
+pfm_gen_ia32_print_info(unsigned int v, int (*pf)(const char *fmt,...))
 {
-	if (cnt != PFMLIB_CNT_FIRST && cnt > gen_support->pmc_count)
-		return PFMLIB_ERR_INVAL;
+	pme_gen_ia32_entry_t *e;
 
-	*code = gen_ia32_pe[i].pme_code;
+	if (v >= gen_ia32_support.pme_count || pf == NULL) return PFMLIB_ERR_INVAL;
+	e = gen_ia32_pe+v;
+
+	if (e->pme_desc) (*pf)("Desc   : %s\n", e->pme_desc);
 
 	return PFMLIB_SUCCESS;
 }
 
+static int
+pfm_gen_ia32_get_event_code(unsigned int i, unsigned int cnt, int *code)
+{
+	if (cnt != PFMLIB_CNT_FIRST && cnt > gen_ia32_support.pmc_count)
+		return PFMLIB_ERR_INVAL;
+
+	*code = gen_ia32_pe[i].pme_entry_code.pme_code.pme_sel;
+
+	return PFMLIB_SUCCESS;
+}
+
+/*
+ * This function is accessible directly to the user
+ */
+int
+pfm_gen_ia32_get_event_umask(unsigned int i, unsigned long *umask)
+{
+	if (i >= gen_ia32_support.pme_count || umask == NULL) return PFMLIB_ERR_INVAL;
+	*umask = 0;
+	return PFMLIB_SUCCESS;
+}
+	
 static void
 pfm_gen_ia32_get_event_counters(unsigned int j, pfmlib_regmask_t *counters)
 {
@@ -419,17 +368,20 @@ pfm_gen_ia32_get_event_counters(unsigned int j, pfmlib_regmask_t *counters)
 
 	memset(counters, 0, sizeof(*counters));
 
-	for(i=0; i < gen_support->pmc_count; i++)
+	for(i=0; i < gen_ia32_support.pmc_count; i++)
 		pfm_regmask_set(counters, i);
 }
 
+
 static void
 pfm_gen_ia32_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 {
 	unsigned int i = 0;
 
+	memset(impl_pmcs, 0, sizeof(*impl_pmcs));
+
 	/* all pmcs are contiguous */
-	for(i=0; i < gen_support->pmc_count; i++)
+	for(i=0; i < gen_ia32_support.pmc_count; i++)
 		pfm_regmask_set(impl_pmcs, i);
 }
 
@@ -438,8 +390,10 @@ pfm_gen_ia32_get_impl_pmds(pfmlib_regmask_t *impl_pmds)
 {
 	unsigned int i = 0;
 
+	memset(impl_pmds, 0, sizeof(*impl_pmds));
+
 	/* all pmds are contiguous */
-	for(i=0; i < gen_support->pmc_count; i++)
+	for(i=0; i < gen_ia32_support.pmc_count; i++)
 		pfm_regmask_set(impl_pmds, i);
 }
 
@@ -448,6 +402,8 @@ pfm_gen_ia32_get_impl_counters(pfmlib_regmask_t *impl_counters)
 {
 	unsigned int i = 0;
 
+	memset(impl_counters, 0, sizeof(*impl_counters));
+
 	/* counting pmds are contiguous */
 	for(i=0; i < 4; i++)
 		pfm_regmask_set(impl_counters, i);
@@ -484,107 +440,25 @@ pfm_gen_ia32_get_event_description(unsigned int ev, char **str)
 	return PFMLIB_SUCCESS;
 }
 
-static char *
-pfm_gen_ia32_get_event_mask_name(unsigned int ev, unsigned int midx)
-{
-	return gen_ia32_pe[ev].pme_umasks[midx].pme_uname;
-}
-
-static int
-pfm_gen_ia32_get_event_mask_desc(unsigned int ev, unsigned int midx, char **str)
-{
-	char *s;
-
-	s = gen_ia32_pe[ev].pme_umasks[midx].pme_udesc;
-	if (s) {
-		*str = strdup(s);
-	} else {
-		*str = NULL;
-	}
-	return PFMLIB_SUCCESS;
-}
-
-static unsigned int
-pfm_gen_ia32_get_num_event_masks(unsigned int ev)
-{
-	return gen_ia32_pe[ev].pme_numasks;
-}
-
-static int
-pfm_gen_ia32_get_event_mask_code(unsigned int ev, unsigned int midx, unsigned int *code)
-{
-	*code =gen_ia32_pe[ev].pme_umasks[midx].pme_ucode;
-	return PFMLIB_SUCCESS;
-}
-
-static int
-pfm_gen_ia32_get_cycle_event(pfmlib_event_t *e)
-{
-	if (gen_ia32_cycle_event == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	e->event = gen_ia32_cycle_event;
-	return PFMLIB_SUCCESS;
-
-}
-
-static int
-pfm_gen_ia32_get_inst_retired(pfmlib_event_t *e)
-{
-	if (gen_ia32_inst_retired_event == -1)
-		return PFMLIB_ERR_NOTSUPP;
-
-	e->event = gen_ia32_inst_retired_event;
-	return PFMLIB_SUCCESS;
-}
-
-/* architected IA-32 PMU */
+/* Generic P6 processor support (not incl. Pentium M) */
 pfm_pmu_support_t gen_ia32_support={
-	.pmu_name		= "Intel architectural PMU",
+	.pmu_name		= "IA-32 Generic",
 	.pmu_type		= PFMLIB_GEN_IA32_PMU,
 	.pme_count		= 0,
 	.pmc_count		= 0,
 	.pmd_count		= 0,
 	.num_cnt		= 0,
+	.cycle_event		= PFMLIB_NO_EVT,
+	.inst_retired_event	= PFMLIB_NO_EVT,
 	.get_event_code		= pfm_gen_ia32_get_event_code,
 	.get_event_name		= pfm_gen_ia32_get_event_name,
 	.get_event_counters	= pfm_gen_ia32_get_event_counters,
+	.print_info		= pfm_gen_ia32_print_info,
 	.dispatch_events	= pfm_gen_ia32_dispatch_events,
 	.pmu_detect		= pfm_gen_ia32_detect,
 	.get_impl_pmcs		= pfm_gen_ia32_get_impl_pmcs,
 	.get_impl_pmds		= pfm_gen_ia32_get_impl_pmds,
 	.get_impl_counters	= pfm_gen_ia32_get_impl_counters,
 	.get_hw_counter_width	= pfm_gen_ia32_get_hw_counter_width,
-	.get_event_desc         = pfm_gen_ia32_get_event_description,
-	.get_cycle_event	= pfm_gen_ia32_get_cycle_event,
-	.get_inst_retired_event = pfm_gen_ia32_get_inst_retired,
-	.get_num_event_masks	= pfm_gen_ia32_get_num_event_masks,
-	.get_event_mask_name	= pfm_gen_ia32_get_event_mask_name,
-	.get_event_mask_code	= pfm_gen_ia32_get_event_mask_code,
-	.get_event_mask_desc	= pfm_gen_ia32_get_event_mask_desc
-};
-
-pfm_pmu_support_t coreduo_support={
-	.pmu_name		= "Intel Core Duo/Core Solo",
-	.pmu_type		= PFMLIB_COREDUO_PMU,
-	.pme_count		= PME_COREDUO_EVENT_COUNT,
-	.pmc_count		= 2,
-	.pmd_count		= 2,
-	.num_cnt		= 2,
-	.get_event_code		= pfm_gen_ia32_get_event_code,
-	.get_event_name		= pfm_gen_ia32_get_event_name,
-	.get_event_counters	= pfm_gen_ia32_get_event_counters,
-	.dispatch_events	= pfm_gen_ia32_dispatch_events,
-	.pmu_detect		= pfm_coreduo_detect,
-	.get_impl_pmcs		= pfm_gen_ia32_get_impl_pmcs,
-	.get_impl_pmds		= pfm_gen_ia32_get_impl_pmds,
-	.get_impl_counters	= pfm_gen_ia32_get_impl_counters,
-	.get_hw_counter_width	= pfm_gen_ia32_get_hw_counter_width,
-	.get_event_desc         = pfm_gen_ia32_get_event_description,
-	.get_num_event_masks	= pfm_gen_ia32_get_num_event_masks,
-	.get_event_mask_name	= pfm_gen_ia32_get_event_mask_name,
-	.get_event_mask_code	= pfm_gen_ia32_get_event_mask_code,
-	.get_event_mask_desc	= pfm_gen_ia32_get_event_mask_desc,
-	.get_cycle_event	= pfm_gen_ia32_get_cycle_event,
-	.get_inst_retired_event = pfm_gen_ia32_get_inst_retired
+	.get_event_desc         = pfm_gen_ia32_get_event_description
 };
