@@ -60,35 +60,17 @@
 #define pmc_thres	pmc_ita_count_reg.pmc_thres
 #define pmc_ism		pmc_ita_count_reg.pmc_ism
 
-/*
- * Description of the PMC register mappings use by
- * this module (as reported in pfmlib_reg_t.reg_num):
- *
- * 0 -> PMC0
- * 1 -> PMC1
- * n -> PMCn
- *
- * The following are in the model specific rr_br[]:
- * IBR0 -> 0
- * IBR1 -> 1
- * ...
- * IBR7 -> 7
- * DBR0 -> 0
- * DBR1 -> 1
- * ...
- * DBR7 -> 7
- *
- * We do not use a mapping table, instead we make up the
- * values on the fly given the base.
- */
-#define PFMLIB_ITA_PMC_BASE 0
-
-
 static int
 pfm_ita_detect(void)
 {
 	int ret = PFMLIB_ERR_NOTSUPP;
 
+	/*
+	 * check that the core library supports enough registers
+	 */
+	if (PFMLIB_MAX_PMCS < PMU_ITA_NUM_PMCS) return ret;
+	if (PFMLIB_MAX_PMDS < PMU_ITA_NUM_PMDS) return ret;
+
 	/*
 	 * we support all chips (there is only one!) in the Itanium family
 	 */
@@ -116,6 +98,7 @@ valid_assign(unsigned int *as, pfmlib_regmask_t *r_pmcs, unsigned int cnt)
 
 /*
  * Automatically dispatch events to corresponding counters following constraints.
+ * Upon return the pfmlib_pmc_t structure is ready to be submitted to kernel
  */
 static int
 pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pfmlib_output_param_t *outp)
@@ -124,7 +107,7 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_event_t *e;
-	pfmlib_reg_t *pc, *pd;
+	pfmlib_pmc_t *pc;
 	pfmlib_regmask_t *r_pmcs;
 	unsigned int i,j,k,l, m;
 	unsigned int max_l0, max_l1, max_l2, max_l3;
@@ -133,7 +116,6 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 
 	e      = inp->pfp_events;
 	pc     = outp->pfp_pmcs;
-	pd     = outp->pfp_pmds;
 	cnt    = inp->pfp_event_count;
 	r_pmcs = &inp->pfp_unavail_pmcs;
 
@@ -192,6 +174,7 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 	/* we cannot satisfy the constraints */
 	return PFMLIB_ERR_NOASSIGN;
 done:
+
 	for (j=0; j < cnt ; j++ ) {
 		reg.pmc_val = 0; /* clear all */
 		/* if plm is 0, then assume not specified per-event and use default */
@@ -204,27 +187,21 @@ done:
 		reg.pmc_es     = itanium_pe[e[j].event].pme_code;
 
 		pc[j].reg_num     = assign[j];
+		pc[j].reg_pmd_num = assign[j];
+		pc[j].reg_evt_idx = j;
 		pc[j].reg_value   = reg.pmc_val;
-		pc[j].reg_addr    = assign[j];
 
-		pd[j].reg_num = assign[j];
-		pd[j].reg_addr = assign[j];
-
-		__pfm_vbprintf("[PMC%u(pmc%u)=0x%06lx thres=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
-				assign[j],
-				assign[j],
-				reg.pmc_val,
-				reg.pmc_thres,
-				reg.pmc_es,reg.pmc_plm,
-				reg.pmc_umask, reg.pmc_pm,
-				reg.pmc_ism,
-				reg.pmc_oi,
-				itanium_pe[e[j].event].pme_name);
-		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[j].reg_num, pd[j].reg_num);
+		__pfm_vbprintf("[pmc%u=0x%06lx thres=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
+					assign[j], reg.pmc_val,
+					reg.pmc_thres,
+					reg.pmc_es,reg.pmc_plm,
+					reg.pmc_umask, reg.pmc_pm,
+					reg.pmc_ism,
+					reg.pmc_oi,
+					itanium_pe[e[j].event].pme_name);
 	}
 	/* number of PMC registers programmed */
 	outp->pfp_pmc_count = cnt;
-	outp->pfp_pmd_count = cnt;
 
 	return PFMLIB_SUCCESS;
 }
@@ -235,17 +212,12 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_ita_input_param_t fake_param;
-	pfmlib_reg_t *pc, *pd;
-	unsigned int pos1, pos2;
+	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	int pos = outp->pfp_pmc_count;
 	int iear_idx = -1;
 	unsigned int i, count;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
-
 	for (i=0; i < count; i++) {
 		if (is_iear(inp->pfp_events[i].event)) iear_idx = i;
 	}
@@ -287,27 +259,23 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 10))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num     = 10;  /* PMC10 is I-EAR config register */
-	pc[pos1].reg_value   = reg.pmc_val;
-	pc[pos1].reg_addr    = 10;
-	pos1++;
-	pd[pos2].reg_num     = 0; 
-	pd[pos2++].reg_addr  = 0;
-	pd[pos2].reg_num     = 1; 
-	pd[pos2++].reg_addr  = 1;
+	pc[pos].reg_num     = 10;  /* PMC10 is I-EAR config register */
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos].reg_value   = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC10(pmc10)=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
+	pos++;
+
+	__pfm_vbprintf("[pmc10=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc10_ita_reg.iear_tlb ? "Yes" : "No",
 			reg.pmc10_ita_reg.iear_plm,
 			reg.pmc10_ita_reg.iear_pm,
 			reg.pmc10_ita_reg.iear_ism,
 			reg.pmc10_ita_reg.iear_umask);
-	__pfm_vbprintf("[PMD0(pmd0)]\n[PMD1(pmd1)\n");
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
+	outp->pfp_pmc_count = pos;
 
 	return PFMLIB_SUCCESS;
 }
@@ -318,17 +286,12 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_ita_input_param_t fake_param;
-	pfmlib_reg_t *pc, *pd;
-	unsigned int pos1, pos2;
+	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	int pos = outp->pfp_pmc_count;
 	int dear_idx = -1;
 	unsigned int i, count;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
-
 	for (i=0; i < count; i++) {
 		if (is_dear(inp->pfp_events[i].event)) dear_idx = i;
 	}
@@ -372,18 +335,14 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 11))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num     = 11;  /* PMC11 is D-EAR config register */
-	pc[pos1].reg_value   = reg.pmc_val;
-	pc[pos1].reg_addr    = 11;
-	pos1++;
-	pd[pos2].reg_num     = 2; 
-	pd[pos2++].reg_addr  = 2;
-	pd[pos2].reg_num     = 3; 
-	pd[pos2++].reg_addr  = 3;
-	pd[pos2].reg_num     = 17; 
-	pd[pos2++].reg_addr  = 17;
+	pc[pos].reg_num     = 11;  /* PMC11 is D-EAR config register */
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos].reg_value   = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC11(pmc11)=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
+	pos++;
+
+	__pfm_vbprintf("[pmc11=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
 			reg.pmc_val,
 			reg.pmc11_ita_reg.dear_tlb ? "Yes" : "No",
 			reg.pmc11_ita_reg.dear_plm,	
@@ -391,11 +350,9 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 			reg.pmc11_ita_reg.dear_ism,
 			reg.pmc11_ita_reg.dear_umask,
 			reg.pmc11_ita_reg.dear_pt);
-	__pfm_vbprintf("[PMD2(pmd2)]\n[PMD3(pmd3)\nPMD17(pmd17)\n");
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
+	outp->pfp_pmc_count = pos;
 
 	return PFMLIB_SUCCESS;
 }
@@ -405,7 +362,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 {
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfm_ita_pmc_reg_t reg;
-	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	pfmlib_pmc_t *pc = outp->pfp_pmcs;
 	int pos = outp->pfp_pmc_count;
 
 	if (param == NULL) return PFMLIB_SUCCESS;
@@ -418,11 +375,12 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 			return PFMLIB_ERR_NOASSIGN;
 
 		pc[pos].reg_num     = 8;
-		pc[pos].reg_value   = reg.pmc_val;
-		pc[pos++].reg_addr  = 8;
+		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+		pc[pos++].reg_value = reg.pmc_val;
 
 
-		__pfm_vbprintf("[PMC8(pmc8)=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
+		__pfm_vbprintf("[pmc8=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
 				reg.pmc_val,
 				reg.pmc8_9_ita_reg.m,
 				reg.pmc8_9_ita_reg.i,
@@ -440,12 +398,12 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 			return PFMLIB_ERR_NOASSIGN;
 
 		pc[pos].reg_num     = 9;
-		pc[pos].reg_value   = reg.pmc_val;
-		pc[pos++].reg_addr  = 9;
+		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+		pc[pos++].reg_value = reg.pmc_val;
 
 
-		__pfm_vbprintf("[PMC9(pmc9)=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
-				reg.pmc_val,
+		__pfm_vbprintf("[pmc9 m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
 				reg.pmc8_9_ita_reg.m,
 				reg.pmc8_9_ita_reg.i,
 				reg.pmc8_9_ita_reg.f,
@@ -464,18 +422,13 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pf
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_ita_input_param_t fake_param;
-	pfmlib_reg_t *pc, *pd;
+	pfmlib_pmc_t *pc = outp->pfp_pmcs;
 	int found_btb=0;
 	unsigned int i, count;
-	unsigned int  pos1, pos2;
+	int  pos = outp->pfp_pmc_count;
 
 	reg.pmc_val = 0;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
-
 	count = inp->pfp_event_count;
 	for (i=0; i < count; i++) {
 		if (is_btb(inp->pfp_events[i].event)) found_btb = 1;
@@ -519,15 +472,18 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pf
 	reg.pmc12_ita_reg.btbc_bpt = param->pfp_ita_btb.btb_tac & 0x1;
 	reg.pmc12_ita_reg.btbc_bac = param->pfp_ita_btb.btb_bac & 0x1;
 
+	memset(pc+pos, 0, sizeof(pfmlib_pmc_t));
+
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 12))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num   = 12;
-	pc[pos1].reg_value = reg.pmc_val;
-	pc[pos1].reg_value = 12;
-	pos1++;
+	pc[pos].reg_num     = 12;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC12(pmc12)=0x%lx plm=%d pm=%d tar=%d tm=%d ptm=%d ppm=%d bpt=%d bac=%d]\n",
+
+	__pfm_vbprintf("[pmc12=0x%lx plm=%d pm=%d tar=%d tm=%d ptm=%d ppm=%d bpt=%d bac=%d]\n",
 			reg.pmc_val,
 			reg.pmc12_ita_reg.btbc_plm,
 			reg.pmc12_ita_reg.btbc_pm,
@@ -538,18 +494,8 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pf
 			reg.pmc12_ita_reg.btbc_bpt,
 			reg.pmc12_ita_reg.btbc_bac);
 
-	/*
-	 * PMD16 is included in list of used PMD
-	 */
-	for(i=8; i < 17; i++, pos2++) {
-		pd[pos2].reg_num = i;
-		pd[pos2].reg_addr = i;
-		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[pos2].reg_num, pd[pos2].reg_num);
-	}
-
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
+	outp->pfp_pmc_count = pos;
 
 	return PFMLIB_SUCCESS;
 }
@@ -580,7 +526,7 @@ check_intervals(pfmlib_ita_input_rr_t *irr, int mode, int *n_intervals)
 
 static void
 do_normal_rr(unsigned long start, unsigned long end,
-			     pfmlib_reg_t *br, int nbr, int dir, int *idx, int *reg_idx, int plm)
+			     pfmlib_pmc_t *br, int nbr, int dir, int *idx, int *reg_idx, int plm)
 {
 	unsigned long size, l_addr, c;
 	unsigned long l_offs = 0, r_offs = 0;
@@ -662,11 +608,9 @@ do_normal_rr(unsigned long start, unsigned long end,
 
 	br[*idx].reg_num     = *reg_idx;
 	br[*idx].reg_value   = l_addr;
-	br[*idx].reg_addr    = *reg_idx;
 
 	br[*idx+1].reg_num   = *reg_idx+1;
 	br[*idx+1].reg_value = db.val;
-	br[*idx+1].reg_addr  = *reg_idx+1;
 
 	*idx     += 2;
 	*reg_idx += 2;
@@ -697,7 +641,7 @@ do_normal_rr(unsigned long start, unsigned long end,
 
 
 static void
-print_one_range(pfmlib_ita_input_rr_desc_t *in_rr, pfmlib_ita_output_rr_desc_t *out_rr, pfmlib_reg_t *dbr, int base_idx, int n_pairs)
+print_one_range(pfmlib_ita_input_rr_desc_t *in_rr, pfmlib_ita_output_rr_desc_t *out_rr, pfmlib_pmc_t *dbr, int base_idx, int n_pairs)
 {
 	int j;
 	dbreg_t d;
@@ -728,7 +672,7 @@ compute_normal_rr(pfmlib_ita_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 	pfmlib_ita_input_rr_desc_t *in_rr;
 	pfmlib_ita_output_rr_desc_t *out_rr;
 	unsigned long r_end;
-	pfmlib_reg_t *br;
+	pfmlib_pmc_t *br;
 	dbreg_t d;
 	int i, j, br_index, reg_idx, prev_index;
 
@@ -752,7 +696,7 @@ compute_normal_rr(pfmlib_ita_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 				4 - (reg_idx>>1), /* how many pairs available */
 				0,
 				&br_index,
-				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : dfl_plm);
+				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : (unsigned long)dfl_plm);
 
 		DPRINT(("br_index=%d reg_idx=%d\n", br_index, reg_idx));
 		/*
@@ -791,7 +735,7 @@ pfm_dispatch_irange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 {
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	pfmlib_pmc_t *pc = outp->pfp_pmcs;
 	pfmlib_ita_input_rr_t *irr;
 	pfmlib_ita_output_rr_t *orr;
 	int pos = outp->pfp_pmc_count;
@@ -820,14 +764,17 @@ pfm_dispatch_irange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 
 	reg.pmc13_ita_reg.irange_ta = 0x0;
 
+	memset(pc+pos, 0, sizeof(pfmlib_pmc_t));
+
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 13))
 		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 13;
-	pc[pos].reg_value   = reg.pmc_val;
-	pc[pos++].reg_addr  = 13;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC13(pmc13)=0x%lx ta=%d]\n", reg.pmc_val, reg.pmc13_ita_reg.irange_ta);
+	__pfm_vbprintf("[pmc13=0x%lx ta=%d]\n", reg.pmc_val, reg.pmc13_ita_reg.irange_ta);
 	
 	outp->pfp_pmc_count = pos;
 
@@ -839,7 +786,7 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 {
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_event_t *e = inp->pfp_events;
-	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	pfmlib_pmc_t *pc = outp->pfp_pmcs;
 	pfmlib_ita_input_rr_t *irr;
 	pfmlib_ita_output_rr_t *orr;
 	pfm_ita_pmc_reg_t reg;
@@ -871,6 +818,14 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 		if (is_dear(e[i].event)) return PFMLIB_SUCCESS; /* will be done there */
 	}
 
+	/*
+	 * this will clear the pt field which is what we want for drange
+	 * (reg.pmc11_ita_reg.dear_pt = 0)
+	 *
+	 * XXX: could even get rid of reg altogether here
+	 */
+	memset(pc+pos, 0, sizeof(pfmlib_pmc_t));
+
 	reg.pmc_val = 0UL;
 	/*
 	 * here we have no other choice but to use the default priv level as there is no
@@ -882,10 +837,11 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 11;
-	pc[pos].reg_value   = reg.pmc_val;
-	pc[pos++].reg_addr  = 11;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC11(pmc11)=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
+	__pfm_vbprintf("[pmc11=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
 			reg.pmc_val,
 			reg.pmc11_ita_reg.dear_tlb ? "Yes" : "No",
 			reg.pmc11_ita_reg.dear_plm,	
@@ -1074,13 +1030,63 @@ pfm_ita_get_ear_mode(unsigned int i, pfmlib_ita_ear_mode_t *m)
 }
 
 
+/*
+ * Function used to print information about a specific events. More than
+ * one event can be printed in case an event code is given rather than
+ * a specific name. A callback function is used for printing.
+ */
+static int
+pfm_ita_print_info(unsigned int v, int (*pf)(const char *fmt,...))
+{
+	pme_ita_entry_t *e;
+        const char *quals[]={ "[Instruction Address Range]", "[OpCode Match]", "[Data Address Range]" };
+	long c;
+        int i;
+
+	if (v >= PME_ITA_EVENT_COUNT || pf == NULL) return PFMLIB_ERR_INVAL;
+	e = itanium_pe+v;
+
+	(*pf)("Umask  : ");
+	c = e->pme_umask;
+	for (i=3; i >=0; i--) {
+		(*pf)("%d", c & 1<<i ? 1 : 0);
+	}
+	(*pf)("\n");
+
+	(*pf)( "EAR    : %s (%s)\n",
+		e->pme_ear ? (e->pme_dear ? "Data" : "Inst") : "No",
+		e->pme_ear ? (e->pme_tlb ? "TLB Mode": "Cache Mode"): "N/A");
+	
+
+	(*pf)("BTB    : %s\n", e->pme_btb ? "Yes" : "No");
+
+	if (e->pme_maxincr > 1)
+		(*pf)("MaxIncr: %u  (Threshold [0-%u])\n", e->pme_maxincr,  e->pme_maxincr-1);
+ 	else
+		(*pf)("MaxIncr: %u  (Threshold 0)\n", e->pme_maxincr);
+
+	(*pf)("Qual   : ");
+
+	c = e->pme_qualifiers.qual;
+	if ((c & 0x7) == 0) {
+		(*pf)("None");
+	} else {
+		for (i=0; i < 3; i++ ) {
+                	if (c & 0x1) (*pf)("%s ", quals[i]);
+                	c >>= 1;
+        	}
+	}
+	(*pf)("\n");
+	return PFMLIB_SUCCESS;
+}
+
 static int
 pfm_ita_get_event_code(unsigned int i, unsigned int cnt, int *code)
 {
 	if (cnt != PFMLIB_CNT_FIRST && (cnt < 4 || cnt > 7))
 		return PFMLIB_ERR_INVAL;
 
-	*code = (int)itanium_pe[i].pme_code;
+	*code = itanium_pe[i].pme_code;
 
 	return PFMLIB_SUCCESS;
 }
@@ -1122,6 +1128,8 @@ pfm_ita_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 {
 	unsigned int i = 0;
 
+	memset(impl_pmcs, 0, sizeof(*impl_pmcs));
+
 	/* all pmcs are contiguous */
 	for(i=0; i < PMU_ITA_NUM_PMCS; i++)
 		pfm_regmask_set(impl_pmcs, i);
@@ -1132,6 +1140,8 @@ pfm_ita_get_impl_pmds(pfmlib_regmask_t *impl_pmds)
 {
 	unsigned int i = 0;
 
+	memset(impl_pmds, 0, sizeof(*impl_pmds));
+
 	/* all pmds are contiguous */
 	for(i=0; i < PMU_ITA_NUM_PMDS; i++)
 		pfm_regmask_set(impl_pmds, i);
@@ -1142,6 +1152,8 @@ pfm_ita_get_impl_counters(pfmlib_regmask_t *impl_counters)
 {
 	unsigned int i = 0;
 
+	memset(impl_counters, 0, sizeof(*impl_counters));
+
 	/* counting pmds are contiguous */
 	for(i=4; i < 8; i++)
 		pfm_regmask_set(impl_counters, i);
@@ -1153,21 +1165,6 @@ pfm_ita_get_hw_counter_width(unsigned int *width)
 	*width = PMU_ITA_COUNTER_WIDTH;
 }
 
-static int
-pfm_ita_get_cycle_event(pfmlib_event_t *e)
-{
-	e->event = PME_ITA_CPU_CYCLES;
-	return PFMLIB_SUCCESS;
-
-}
-
-static int
-pfm_ita_get_inst_retired(pfmlib_event_t *e)
-{
-	e->event = PME_ITA_IA64_INST_RETIRED;
-	return PFMLIB_SUCCESS;
-}
-
 pfm_pmu_support_t itanium_support={
 	.pmu_name		= "itanium",
 	.pmu_type		= PFMLIB_ITANIUM_PMU,
@@ -1175,16 +1172,17 @@ pfm_pmu_support_t itanium_support={
 	.pmc_count		= PMU_ITA_NUM_PMCS,
 	.pmd_count		= PMU_ITA_NUM_PMDS,
 	.num_cnt		= PMU_ITA_NUM_COUNTERS,
+	.cycle_event		= PME_ITA_CPU_CYCLES,
+	.inst_retired_event	= PME_ITA_IA64_INST_RETIRED,
 	.get_event_code		= pfm_ita_get_event_code,
 	.get_event_name		= pfm_ita_get_event_name,
 	.get_event_counters	= pfm_ita_get_event_counters,
+	.print_info		= pfm_ita_print_info,
 	.dispatch_events	= pfm_ita_dispatch_events,
 	.pmu_detect		= pfm_ita_detect,
 	.get_impl_pmcs		= pfm_ita_get_impl_pmcs,
 	.get_impl_pmds		= pfm_ita_get_impl_pmds,
 	.get_impl_counters	= pfm_ita_get_impl_counters,
-	.get_hw_counter_width	= pfm_ita_get_hw_counter_width,
-	.get_cycle_event	= pfm_ita_get_cycle_event,
-	.get_inst_retired_event = pfm_ita_get_inst_retired
+	.get_hw_counter_width	= pfm_ita_get_hw_counter_width
 	/* no event description available for Itanium */
 };
