@@ -53,11 +53,11 @@
 #define evt_use_irange(e)	((e)->pfp_mont_irange.rr_used)
 #define evt_use_drange(e)	((e)->pfp_mont_drange.rr_used)
 
-#define evt_grp(e)	(int)montecito_pe[e].pme_qualifiers.pme_qual.pme_group
-#define evt_set(e)	(int)montecito_pe[e].pme_qualifiers.pme_qual.pme_set
+#define evt_grp(e)	montecito_pe[e].pme_qualifiers.pme_qual.pme_group
+#define evt_set(e)	montecito_pe[e].pme_qualifiers.pme_qual.pme_set
 #define evt_umask(e)	montecito_pe[e].pme_umask
-#define evt_type(e)	(int)montecito_pe[e].pme_type
-#define evt_caf(e)	(int)montecito_pe[e].pme_caf
+#define evt_type(e)	montecito_pe[e].pme_type
+#define evt_caf(e)	montecito_pe[e].pme_caf
 
 #define FINE_MODE_BOUNDARY_BITS	16
 #define FINE_MODE_MASK		~((1U<<FINE_MODE_BOUNDARY_BITS)-1)
@@ -80,29 +80,6 @@
 #define UNEXISTING_SET	0xff
 
 static char * pfm_mont_get_event_name(unsigned int i);
-/*
- * Description of the PMC register mappings use by
- * this module (as reported in pfmlib_reg_t.reg_num):
- *
- * 0 -> PMC0
- * 1 -> PMC1
- * n -> PMCn
- *
- * The following are in the model specific rr_br[]:
- * IBR0 -> 0
- * IBR1 -> 1
- * ...
- * IBR7 -> 7
- * DBR0 -> 0
- * DBR1 -> 1
- * ...
- * DBR7 -> 7
- *
- * We do not use a mapping table, instead we make up the
- * values on the fly given the base.
- */
-
-static void pfm_mont_get_impl_counters(pfmlib_regmask_t *impl_counters);
 
 static int
 pfm_mont_detect(void)
@@ -110,6 +87,12 @@ pfm_mont_detect(void)
 	int tmp;
 	int ret = PFMLIB_ERR_NOTSUPP;
 
+	/*
+	 * check that the core library supports enough registers
+	 */
+	if (PFMLIB_MAX_PMCS < PMU_MONT_NUM_PMCS) return ret;
+	if (PFMLIB_MAX_PMDS < PMU_MONT_NUM_PMDS) return ret;
+
 	tmp = pfm_ia64_get_cpu_family();
 	if (tmp == 0x20) {
 		ret = PFMLIB_SUCCESS;
@@ -137,13 +120,12 @@ pfm_mont_detect(void)
  */
 static int
 check_cross_groups(pfmlib_input_param_t *inp, unsigned int *l1d_event, 
-		unsigned long *l2d_set1_mask, unsigned long *l2d_set2_mask)
+		unsigned long *l2d_set1_mask, unsigned long*l2d_set2_mask)
 {
 	int g, s, s1, s2;
 	unsigned int cnt = inp->pfp_event_count;
 	pfmlib_event_t *e = inp->pfp_events;
-	unsigned int i, j;
-	unsigned long l2d_mask1 = 0, l2d_mask2 = 0;
+	unsigned int i, j, l2d_mask1 = 0, l2d_mask2 = 0;
 	unsigned int l1d_event_idx = UNEXISTING_SET;
 
 	/*
@@ -451,6 +433,7 @@ check_cancel_events(pfmlib_input_param_t *inp)
 
 /*
  * Automatically dispatch events to corresponding counters following constraints.
+ * Upon return the pfarg_regt structure is ready to be submitted to kernel
  */
 static unsigned int l2d_set1_cnts[]={ 4, 5, 8 };
 static unsigned int l2d_set2_cnts[]={ 6, 7, 9 };
@@ -461,21 +444,21 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 	pfmlib_mont_input_param_t *param = mod_in;
 	pfm_mont_pmc_reg_t reg;
 	pfmlib_event_t *e;
-	pfmlib_reg_t *pc, *pd;
-	pfmlib_regmask_t avail_cntrs, impl_cntrs;
-	unsigned int i,j, k, max_cnt;
+	pfmlib_reg_t *pc;
+	pfmlib_regmask_t *r_pmcs;
+	unsigned int i,j;
 	unsigned int assign[PMU_MONT_NUM_COUNTERS];
 	unsigned int m, cnt;
 	unsigned int l1d_set;
-	unsigned long l2d_set1_mask, l2d_set2_mask, evt_mask, mesi;
-	unsigned long not_assigned_events, cnt_mask;
-	int l2d_set1_p, l2d_set2_p;
-	int ret;
+	unsigned long l2d_set1_mask, l2d_set2_mask, evt_mask;
+	unsigned long not_assigned_events, free_counter_mask;
+	int l2d_set1_p, l2d_set2_p, mesi;
+	int ret, not_found;
 
 	e      = inp->pfp_events;
 	pc     = outp->pfp_pmcs;
-	pd     = outp->pfp_pmds;
 	cnt    = inp->pfp_event_count;
+	r_pmcs = &inp->pfp_unavail_pmcs;
 
 	if (PFMLIB_DEBUG())
 		for (m=0; m < cnt; m++) {
@@ -506,11 +489,10 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 	 *
 	 * this is the strongest constraint
 	 */
-	pfm_mont_get_impl_counters(&impl_cntrs);
-	pfm_regmask_andnot(&avail_cntrs, &impl_cntrs, &inp->pfp_unavail_pmcs);
+	free_counter_mask   = 0xfff0 & ~r_pmcs->bits[0];
 	not_assigned_events = 0;
 
-	DPRINT(("avail_cntrs=0x%lx\n", avail_cntrs.bits[0]));
+	DPRINT(("free_counter_mask=0x%lx\n", free_counter_mask));
 
 	/*
 	 * we do not check ALL_THRD here because at least
@@ -518,12 +500,12 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 	 */
 	if (l1d_set != UNEXISTING_SET) {
 
-		if (!pfm_regmask_isset(&avail_cntrs, 5))
+		if (pfm_regmask_isset(r_pmcs, 5))
 			return PFMLIB_ERR_NOASSIGN;
 
 		assign[l1d_set] = 5;
 
-		pfm_regmask_clr(&avail_cntrs, 5);
+		free_counter_mask &= ~(1UL << 5);
 	}
 
 	l2d_set1_p = l2d_set2_p = 0;
@@ -541,10 +523,10 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 			if (l2d_set1_p < 3 && (l2d_set1_mask & evt_mask)) {
 				assign[i] = l2d_set1_cnts[l2d_set1_p];
 
-				if (!pfm_regmask_isset(&avail_cntrs, assign[i]))
+				if (pfm_regmask_isset(r_pmcs, assign[i]))
 					return PFMLIB_ERR_NOASSIGN;
 
-				pfm_regmask_clr(&avail_cntrs, assign[i]);
+				free_counter_mask &= ~(1UL << assign[i]);
 				l2d_set1_p++;
 				continue;
 			}
@@ -554,103 +536,52 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 			if (l2d_set2_p  < 3 && (l2d_set2_mask & evt_mask)) {
 				assign[i] = l2d_set2_cnts[l2d_set2_p];
 
-				if (!pfm_regmask_isset(&avail_cntrs, assign[i]))
+				if (pfm_regmask_isset(r_pmcs, assign[i]))
 					return PFMLIB_ERR_NOASSIGN;
 
-				pfm_regmask_clr(&avail_cntrs, assign[i]);
+				free_counter_mask &= ~(1UL << assign[i]);
 				l2d_set2_p++;
 				continue;
 			}
 			/*
 			 * if not l2d nor l1d, then defer placement until final pass
 			 */
-			if (i != l1d_set)
-				not_assigned_events |= evt_mask;
+			if (i != l1d_set) not_assigned_events |= evt_mask;
 
-			DPRINT(("phase 1: i=%u avail_cntrs=0x%lx l2d_set1_p=%d l2d_set2_p=%d not_assigned=0x%lx\n", 
+			DPRINT(("i=%u free_counter_mask=0x%lx l2d_set1_p=%d l2d_set2_p=%d not_assigned=0x%lx\n", 
 				i,
-				avail_cntrs.bits[0],
+				free_counter_mask,
 				l2d_set1_p,
 				l2d_set2_p,
 				not_assigned_events));
 	}
 	/*
-	 * assign BUS_* ER_* events (work only in PMC4-PMC9)
+	 * assign the rest of the events
 	 */
-	evt_mask = not_assigned_events;
-	for (i=0; evt_mask ; i++, evt_mask >>=1) {
+	for (i=0, j=0; not_assigned_events ; i++, not_assigned_events >>=1) {
 
-		if ((evt_mask & 0x1) == 0)
+		if ((not_assigned_events & 0x1) == 0)
 			continue;
 
-		cnt_mask = montecito_pe[e[i].event].pme_counters;
-		/*
-		 * only interested in events with restricted set of counters
-		 */
-		if (cnt_mask == 0xfff0)
-			continue;
+		not_found = 1;
 
-		for(j=0; cnt_mask; j++, cnt_mask >>=1) {
-			if ((cnt_mask & 0x1) == 0) 
-				continue;
-
-			DPRINT(("phase 2: i=%d j=%d cnt_mask=0x%lx avail_cntrs=0x%lx not_assigned_evnts=0x%lx\n",
-				i, j, cnt_mask, avail_cntrs.bits[0], not_assigned_events));
-
-			if (!pfm_regmask_isset(&avail_cntrs, j))
-				continue;
-
-			assign[i] = j;
-			not_assigned_events &= ~(1UL << i);
-			pfm_regmask_clr(&avail_cntrs, j);
-			break;
+		for(; not_found && free_counter_mask; j++, free_counter_mask >>=1) {
+			if (free_counter_mask & 0x1) {
+				assign[i] = j;
+				not_found = 0;
+			}
 		}
-		if (cnt_mask == 0)
+		if (not_found)
 			return PFMLIB_ERR_NOASSIGN;
 	}
-	/*
-	 * assign the rest of the events (no constraints)
-	 */
-	evt_mask = not_assigned_events;
-	max_cnt = PMU_MONT_FIRST_COUNTER + PMU_MONT_NUM_COUNTERS;
-	for (i=0, j=0; evt_mask ; i++, evt_mask >>=1) {
-
-		DPRINT(("phase 3a: i=%d j=%d evt_mask=0x%lx avail_cntrs=0x%lx not_assigned_evnts=0x%lx\n",
-			i, j, evt_mask, avail_cntrs.bits[0], not_assigned_events));
-		if ((evt_mask & 0x1) == 0)
-			continue;
-
-		while(j < max_cnt && !pfm_regmask_isset(&avail_cntrs, j)) {
-			DPRINT(("phase 3: i=%d j=%d evt_mask=0x%lx avail_cntrs=0x%lx not_assigned_evnts=0x%lx\n",
-				i, j, evt_mask, avail_cntrs.bits[0], not_assigned_events));
-			j++;
-		}
-
-		if (j == max_cnt)
-			return PFMLIB_ERR_NOASSIGN;
-
-		assign[i] = j;
-		j++;
-	}
 
 	for (j=0; j < cnt ; j++ ) {
 		mesi = 0;
-
-		/*
-		 * XXX: we do not support .all placement just yet
-		 */
-		if (param && param->pfp_mont_counters[j].flags & PFMLIB_MONT_FL_EVT_ALL_THRD) {
-			DPRINT((".all mode is not yet supported by libpfm\n"));
-			return PFMLIB_ERR_NOTSUPP;
-		}
-
-		if (has_mesi(e[j].event)) {
-			for(k=0;k< e[j].num_masks; k++) {
-				mesi |= 1UL << e[j].unit_masks[k];
-			}
-			/* by default we capture everything */
-			if (mesi == 0)
-				mesi = 0xf;
+		if (param && has_mesi(e[j].event)) {
+			mesi |= param->pfp_mont_counters[j].flags & PFMLIB_MONT_FL_MESI_M ? 1<<3 : 0;
+			mesi |= param->pfp_mont_counters[j].flags & PFMLIB_MONT_FL_MESI_E ? 1<<2 : 0;
+			mesi |= param->pfp_mont_counters[j].flags & PFMLIB_MONT_FL_MESI_S ? 1<<1 : 0;
+			mesi |= param->pfp_mont_counters[j].flags & PFMLIB_MONT_FL_MESI_I ? 1 : 0;
 		}
 		reg.pmc_val    = 0; /* clear all, bits 26-27 must be zero for proper operations */
 		/* if plm is 0, then assume not specified per-event and use default */
@@ -661,26 +592,21 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 		reg.pmc_ism    = 0x2; /* force IA-64 mode */
 		reg.pmc_umask  = is_ear(e[j].event) ? 0x0 : montecito_pe[e[j].event].pme_umask;
 		reg.pmc_es     = montecito_pe[e[j].event].pme_code;
-		reg.pmc_all    = 0; /* XXX force self for now */
-		reg.pmc_m      = (mesi>>3) & 0x1;
-		reg.pmc_e      = (mesi>>2) & 0x1;
-		reg.pmc_s      = (mesi>>1) & 0x1;
-		reg.pmc_i      =  mesi     & 0x1;
+		reg.pmc_m      = mesi & (1<<3);
+		reg.pmc_e      = mesi & (1<<2);
+		reg.pmc_s      = mesi & (1<<1);
+		reg.pmc_i      = mesi & 1;
 		/*
 		 * Note that we don't force PMC4.pmc_ena = 1 because the kernel takes care of this for us.
 		 * This way we don't have to program something in PMC4 even when we don't use it
 		 */
 		pc[j].reg_num     = assign[j];
+		pc[j].reg_pmd_num = assign[j];
+		pc[j].reg_evt_idx = j;
 		pc[j].reg_value   = reg.pmc_val;
-		pc[j].reg_addr    = assign[j];
 
-		pd[j].reg_num  = assign[j];
-		pd[j].reg_addr = assign[j];
-
-		__pfm_vbprintf("[PMC%u(pmc%u)=0x%06lx m=%d e=%d s=%d i=%d thres=%d all=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
-				assign[j],
-				assign[j],
-				reg.pmc_val,
+		__pfm_vbprintf("[pmc%u=0x%06lx m=%d e=%d s=%d i=%d thres=%d all=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
+				assign[j], reg.pmc_val,
 				reg.pmc_m,
 				reg.pmc_e,
 				reg.pmc_s,
@@ -692,11 +618,9 @@ pfm_mont_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t
 				reg.pmc_ism,
 				reg.pmc_oi,
 				montecito_pe[e[j].event].pme_name);
-		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[j].reg_num, pd[j].reg_num);
 	}
 	/* number of PMC registers programmed */
 	outp->pfp_pmc_count = cnt;
-	outp->pfp_pmd_count = cnt;
 
 	return PFMLIB_SUCCESS;
 }
@@ -706,17 +630,12 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 {
 	pfm_mont_pmc_reg_t reg;
 	pfmlib_mont_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc, *pd;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
 	pfmlib_mont_input_param_t fake_param;
-	unsigned int pos1, pos2;
+	int pos = outp->pfp_pmc_count;
 	unsigned int i, count;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
-
 	for (i=0; i < count; i++) {
 		if (is_iear(inp->pfp_events[i].event)) break;
 	}
@@ -766,34 +685,27 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 37))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num     = 37; /* PMC37 is I-EAR config register */
-	pc[pos1].reg_value   = reg.pmc_val;
-	pc[pos1].reg_addr  = 37;
-	pos1++;
-
-	pd[pos2].reg_num  = 34;
-	pd[pos2++].reg_addr = 34; 
-	pd[pos2].reg_num  = 35;
-	pd[pos2++].reg_addr = 35; 
+	pc[pos].reg_num     = 37; /* PMC37 is I-EAR config register */
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
 	if (param->pfp_mont_iear.ear_mode == PFMLIB_MONT_EAR_TLB_MODE) {
-		__pfm_vbprintf("[PMC37(pmc37)=0x%lx ctb=tlb plm=%d pm=%d umask=0x%x]\n",
+		__pfm_vbprintf("[pmc10=0x%lx ctb=tlb plm=%d pm=%d umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc37_mont_tlb_reg.iear_plm,
 			reg.pmc37_mont_tlb_reg.iear_pm,
 			reg.pmc37_mont_tlb_reg.iear_umask);
 	} else {
-		__pfm_vbprintf("[PMC37(pmc37)=0x%lx ctb=cache plm=%d pm=%d umask=0x%x]\n",
+		__pfm_vbprintf("[pmc10=0x%lx ctb=cache plm=%d pm=%d umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc37_mont_cache_reg.iear_plm,
 			reg.pmc37_mont_cache_reg.iear_pm,
 			reg.pmc37_mont_cache_reg.iear_umask);
 	}
-	__pfm_vbprintf("[PMD34(pmd34)]\n[PMD35(pmd35)\n");
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
+	outp->pfp_pmc_count = pos;
 
 	return PFMLIB_SUCCESS;
 }
@@ -803,16 +715,11 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 {
 	pfm_mont_pmc_reg_t reg;
 	pfmlib_mont_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc, *pd;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
 	pfmlib_mont_input_param_t fake_param;
-	unsigned int pos1, pos2;
+	int pos = outp->pfp_pmc_count;
 	unsigned int i, count;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
-
 	count = inp->pfp_event_count;
 	for (i=0; i < count; i++) {
 		if (is_dear(inp->pfp_events[i].event)) break;
@@ -860,19 +767,12 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 40))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num     = 40;  /* PMC11 is D-EAR config register */
-	pc[pos1].reg_value   = reg.pmc_val;
-	pc[pos1].reg_addr  = 40;
-	pos1++;
+	pc[pos].reg_num     = 40;  /* PMC11 is D-EAR config register */
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	pd[pos2].reg_num  = 32;
-	pd[pos2++].reg_addr = 32; 
-	pd[pos2].reg_num  = 33;
-	pd[pos2++].reg_addr = 33; 
-	pd[pos2].reg_num  = 36;
-	pd[pos2++].reg_addr = 36; 
-
-	__pfm_vbprintf("[PMC40(pmc40)=0x%lx mode=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
+	__pfm_vbprintf("[pmc40=0x%lx mode=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc40_mont_reg.dear_mode == 0 ? "L1D" :
 			(reg.pmc40_mont_reg.dear_mode == 1 ? "L1DTLB" : "ALAT"),
@@ -880,11 +780,10 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 			reg.pmc40_mont_reg.dear_pm,
 			reg.pmc40_mont_reg.dear_ism,
 			reg.pmc40_mont_reg.dear_umask);
-	__pfm_vbprintf("[PMD32(pmd32)]\n[PMD33(pmd33)\nPMD36(pmd36)\n");
+
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
+	outp->pfp_pmc_count = pos;
 
 	return PFMLIB_SUCCESS;
 }
@@ -903,7 +802,6 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 
 #define PMC36_DFL_VAL 0xfffffff0
 
-
 	/* 
 	 * mandatory default value for PMC36 as described in the documentation
 	 * all monitoring is opcode constrained. Better make sure the match/mask
@@ -921,8 +819,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 	 * check in any feature is used.
 	 * PMC36 must be setup when opcode matching is used OR when code range restriction is used
 	 */
-	if (used_pmc32 == 0 && used_pmc34 == 0 && param->pfp_mont_irange.rr_used == 0)
-		return 0;
+	if (used_pmc32 == 0 && used_pmc34 == 0 && param->pfp_mont_irange.rr_used == 0) return 0;
 
 	/*
 	 * check for rr_nbr_used to make sure that the range request produced something on output
@@ -950,12 +847,15 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 			reg1.pmc32_34_mont_reg.opcm_inv   = 0;
 		}
 
+		memset(pc+pos, 0, sizeof(pfmlib_reg_t));
+
 		if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 32))
 			return PFMLIB_ERR_NOASSIGN;
 
 		pc[pos].reg_num     = 32;
-		pc[pos].reg_value   = reg1.pmc_val;
-		pc[pos++].reg_addr  = 32;
+		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+		pc[pos++].reg_value = reg1.pmc_val;
 
 		/*
 		 * will be constrained by PMC32
@@ -967,8 +867,9 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 			 * used pmc33 only when we have active opcode matching
 			 */
 			pc[pos].reg_num     = 33;
-			pc[pos].reg_value   = reg2.pmc_val;
-			pc[pos++].reg_addr  = 33;
+			pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+			pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+			pc[pos++].reg_value = reg2.pmc_val;
 
 			has_1st_pair = has_2nd_pair = 0;
 			count        = inp->pfp_event_count;
@@ -981,7 +882,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 			if (has_2nd_pair || has_1st_pair == 0) pmc36.pmc36_mont_reg.opcm_ch2_ig_opcm = 0;
 		}
 
-		__pfm_vbprintf("[PMC32(pmc32)=0x%lx m=%d i=%d f=%d b=%d mask=0x%lx inv=%d ig_ad=%d]\n",
+		__pfm_vbprintf("[pmc32=0x%lx m=%d i=%d f=%d b=%d mask=0x%lx inv=%d ig_ad=%d]\n",
 				reg1.pmc_val,
 				reg1.pmc32_34_mont_reg.opcm_m,
 				reg1.pmc32_34_mont_reg.opcm_i,
@@ -990,8 +891,8 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 				reg1.pmc32_34_mont_reg.opcm_mask,
 				reg1.pmc32_34_mont_reg.opcm_inv,
 				reg1.pmc32_34_mont_reg.opcm_ig_ad);
-		if (used_pmc32)
-			__pfm_vbprintf("[PMC33(pmc33)=0x%lx match=0x%lx]\n",
+		if (used_pmc32) 
+			__pfm_vbprintf("[pmc33=0x%lx match=0x%lx]\n",
 					reg2.pmc_val,
 					reg2.pmc33_35_mont_reg.opcm_match);
 	}
@@ -1000,6 +901,8 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 	 * will be constrained by PMC34
 	 */
 	if (used_pmc34) {
+		memset(pc+pos, 0, sizeof(pfmlib_reg_t));
+
 		reg1.pmc_val = 0x01ffffffffff; /* pmc34 default value */
 		reg2.pmc_val = 0;
 
@@ -1018,11 +921,13 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 			return PFMLIB_ERR_NOASSIGN;
 
 		pc[pos].reg_num     = 34;
-		pc[pos].reg_value   = reg1.pmc_val;
-		pc[pos++].reg_addr  = 34;
+		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+		pc[pos++].reg_value = reg1.pmc_val;
 		pc[pos].reg_num     = 35;
-		pc[pos].reg_value   = reg2.pmc_val;
-		pc[pos++].reg_addr  = 35;
+		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+		pc[pos++].reg_value = reg2.pmc_val;
 
 		has_1st_pair = has_2nd_pair = 0;
 		count        = inp->pfp_event_count;
@@ -1033,7 +938,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 		if (has_1st_pair || has_2nd_pair == 0) pmc36.pmc36_mont_reg.opcm_ch1_ig_opcm = 0;
 		if (has_2nd_pair || has_1st_pair == 0) pmc36.pmc36_mont_reg.opcm_ch3_ig_opcm = 0;
 
-		__pfm_vbprintf("[PMC34(pmc34)=0x%lx m=%d i=%d f=%d b=%d mask=0x%lx]\n",
+		__pfm_vbprintf("[pmc34=0x%lx m=%d i=%d f=%d b=%d mask=0x%lx]\n",
 				reg1.pmc_val,
 				reg1.pmc32_34_mont_reg.opcm_m,
 				reg1.pmc32_34_mont_reg.opcm_i,
@@ -1041,21 +946,24 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 				reg1.pmc32_34_mont_reg.opcm_b,
 				reg1.pmc32_34_mont_reg.opcm_mask);
 
-		__pfm_vbprintf("[PMC35(pmc35)=0x%lx match=0x%lx]\n",
+		__pfm_vbprintf("[pmc35=0x%lx match=0x%lx]\n",
 				reg2.pmc_val,
 				reg2.pmc33_35_mont_reg.opcm_match);
 
 	}
+	memset(pc+pos, 0, sizeof(pfmlib_reg_t));
+
 	if (pmc36.pmc_val != PMC36_DFL_VAL) {
 
 		if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 36))
 			return PFMLIB_ERR_NOASSIGN;
 
 		pc[pos].reg_num     = 36;
-		pc[pos].reg_value   = pmc36.pmc_val;
-		pc[pos++].reg_addr  = 36;
+		pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+		pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+		pc[pos++].reg_value = pmc36.pmc_val;
 
-		__pfm_vbprintf("[PMC36(pmc36)=0x%lx ch0_ig_op=%d ch1_ig_op=%d ch2_ig_op=%d ch3_ig_op=%d]\n",
+		__pfm_vbprintf("[pmc36=0x%lx ch0_ig_op=%d ch1_ig_op=%d ch2_ig_op=%d ch3_ig_op=%d]\n",
 				pmc36.pmc_val,
 				pmc36.pmc36_mont_reg.opcm_ch0_ig_opcm,
 				pmc36.pmc36_mont_reg.opcm_ch1_ig_opcm,
@@ -1074,17 +982,13 @@ pfm_dispatch_etb(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in, p
 	pfmlib_event_t *e= inp->pfp_events;
 	pfm_mont_pmc_reg_t reg;
 	pfmlib_mont_input_param_t *param = mod_in;
-	pfmlib_reg_t *pc, *pd;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
 	pfmlib_mont_input_param_t fake_param;
 	int found_etb = 0, found_bad_dear = 0;
 	int has_etb_param;
-	unsigned int i, pos1, pos2;
+	unsigned int i, pos = outp->pfp_pmc_count;
 	unsigned int count;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
 	/*
 	 * explicit ETB settings
 	 */
@@ -1141,6 +1045,7 @@ pfm_dispatch_etb(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in, p
 		memset(&fake_param, 0, sizeof(fake_param));
 		param = &fake_param;
 
+		param->pfp_mont_etb.etb_ds  = 0; 	/* capture branch targets */
 		param->pfp_mont_etb.etb_tm  = 0x3; 	/* all branches */
 		param->pfp_mont_etb.etb_ptm = 0x3; 	/* all branches */
 		param->pfp_mont_etb.etb_ppm = 0x3; 	/* all branches */
@@ -1155,7 +1060,7 @@ pfm_dispatch_etb(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in, p
 	 */
 	reg.pmc39_mont_reg.etbc_plm = param->pfp_mont_etb.etb_plm ? param->pfp_mont_etb.etb_plm : inp->pfp_dfl_plm;
 	reg.pmc39_mont_reg.etbc_pm  = inp->pfp_flags & PFMLIB_PFP_SYSTEMWIDE ? 1 : 0;
-	reg.pmc39_mont_reg.etbc_ds  = 0; /* 1 is reserved */
+	reg.pmc39_mont_reg.etbc_ds  = param->pfp_mont_etb.etb_ds & 0x1;
 	reg.pmc39_mont_reg.etbc_tm  = param->pfp_mont_etb.etb_tm & 0x3;
 	reg.pmc39_mont_reg.etbc_ptm = param->pfp_mont_etb.etb_ptm & 0x3;
 	reg.pmc39_mont_reg.etbc_ppm = param->pfp_mont_etb.etb_ppm & 0x3;
@@ -1176,12 +1081,12 @@ assign_zero:
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 39))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num     = 39;
-	pc[pos1].reg_value   = reg.pmc_val;
-	pc[pos1].reg_addr  = 39;
-	pos1++;
+	pc[pos].reg_num     = 39;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC39(pmc39)=0x%lx plm=%d pm=%d ds=%d tm=%d ptm=%d ppm=%d brt=%d]\n",
+	__pfm_vbprintf("[pmc39=0x%lx plm=%d pm=%d ds=%d tm=%d ptm=%d ppm=%d brt=%d]\n",
 				reg.pmc_val,
 				reg.pmc39_mont_reg.etbc_plm,
 				reg.pmc39_mont_reg.etbc_pm,
@@ -1191,29 +1096,8 @@ assign_zero:
 				reg.pmc39_mont_reg.etbc_ppm,
 				reg.pmc39_mont_reg.etbc_brt);
 
-	/*
-	 * only add ETB PMDs when actually using BTB.
-	 * Not needed when dealing with D-EAR TLB and DEAR-ALAT
-	 * PMC39 restriction
-	 */
-	if (found_etb || has_etb_param) {
-		pd[pos2].reg_num = 38;
-		pd[pos2++].reg_addr = 38;
-		pd[pos2].reg_num = 39;
-		pd[pos2++].reg_addr = 39;
-		__pfm_vbprintf("[PMD38(pmd38)]\n[PMD39(pmd39)\n");
-
-
-		for(i=48; i < 64; i++, pos2++) {
-			pd[pos2].reg_num = i;
-			pd[pos2].reg_addr = i;
-			__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[pos2].reg_num, pd[pos2].reg_num);
-		}
-	}
-
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
+	outp->pfp_pmc_count = pos;
 
 	return PFMLIB_SUCCESS;
 }
@@ -1304,13 +1188,11 @@ do_normal_rr(unsigned long start, unsigned long end,
 	db.db.db_plm  = plm;
 
 
-	br[*idx].reg_num    = *reg_idx;
-	br[*idx].reg_value  = l_addr;
-	br[*idx].reg_addr   = *reg_idx;
+	br[*idx].reg_num     = *reg_idx;
+	br[*idx].reg_value   = l_addr;
 
 	br[*idx+1].reg_num   = *reg_idx+1;
 	br[*idx+1].reg_value = db.val;
-	br[*idx+1].reg_addr  = *reg_idx+1;
 
 	*idx     += 2;
 	*reg_idx += 2;
@@ -1418,10 +1300,8 @@ compute_fine_rr(pfmlib_mont_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 
 		br[0].reg_num   = reg_idx;
 		br[0].reg_value = addr;
-		br[0].reg_addr  = 1+reg_idx;
 		br[1].reg_num   = reg_idx+1;
 		br[1].reg_value = db.val;
-		br[1].reg_addr  = 1+reg_idx+1;
 
 		/*
 		 * setup upper limit pair
@@ -1444,11 +1324,9 @@ compute_fine_rr(pfmlib_mont_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 
 		br[2].reg_num   = reg_idx+4;
 		br[2].reg_value = addr;
-		br[2].reg_addr  = 1+reg_idx+4;
 
 		br[3].reg_num   = reg_idx+5;
 		br[3].reg_value = db.val;
-		br[3].reg_addr  = 1+reg_idx+5;
 
 		if (PFMLIB_VERBOSE()) print_one_range(in_rr, out_rr, br, 0, 2, 1, irr->rr_flags);
 	}
@@ -1512,7 +1390,6 @@ found:
 	/* when the event is not IA64_INST_RETIRED, then we MUST use ibrp0 */
 	br[0].reg_num   = reg_idx;
 	br[0].reg_value = p_start;
-	br[0].reg_addr  = 1+reg_idx;
 
 	db.val        = 0;
 	db.db.db_mask = ~((1UL << m)-1);
@@ -1521,7 +1398,6 @@ found:
 
 	br[1].reg_num   = reg_idx + 1;
 	br[1].reg_value = db.val;
-	br[1].reg_addr  = 1+reg_idx+1;
 
 	out_rr->rr_soff = start - p_start;
 	out_rr->rr_eoff = p_end - end;
@@ -1567,7 +1443,7 @@ compute_normal_rr(pfmlib_mont_input_rr_t *irr, int dfl_plm, int n, int *base_idx
 				4 - (reg_idx>>1), /* how many pairs available */
 				0,
 				&br_index,
-				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : dfl_plm);
+				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : (unsigned long)dfl_plm);
 
 		DPRINT(("br_index=%d reg_idx=%d\n", br_index, reg_idx));
 
@@ -1746,14 +1622,18 @@ pfm_dispatch_irange(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in
 		if ((retired_mask & 0x8) == 0 &&  reg.pmc38_mont_reg.iarc_ig_ibrp3 == 0) return PFMLIB_ERR_IRRINVAL;
 	}
 
+	/* initialize pmc request slot */
+	memset(pc+pos, 0, sizeof(pfmlib_reg_t));
+
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 38))
 		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 38;
-	pc[pos].reg_value   = reg.pmc_val;
-	pc[pos++].reg_addr  = 38;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC38(pmc38)=0x%lx ig_ibrp0=%d ig_ibrp1=%d ig_ibrp2=%d ig_ibrp3=%d fine=%d]\n",
+	__pfm_vbprintf("[pmc38=0x%lx ig_ibrp0=%d ig_ibrp1=%d ig_ibrp2=%d ig_ibrp3=%d fine=%d]\n",
 			reg.pmc_val,
 			reg.pmc38_mont_reg.iarc_ig_ibrp0,
 			reg.pmc38_mont_reg.iarc_ig_ibrp1,
@@ -1917,10 +1797,11 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in
 		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 41;
-	pc[pos].reg_value   = reg.pmc_val;
-	pc[pos++].reg_addr  = 41;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC41(pmc41)=0x%lx cfg_dtag0=%d cfg_dtag1=%d cfg_dtag2=%d cfg_dtag3=%d ena_dbrp0=%d ena_dbrp1=%d ena_dbrp2=%d ena_dbrp3=%d]\n",
+	__pfm_vbprintf("[pmc41=0x%lx cfg_dtag0=%d cfg_dtag1=%d cfg_dtag2=%d cfg_dtag3=%d ena_dbrp0=%d ena_dbrp1=%d ena_dbrp2=%d ena_dbrp3=%d]\n",
 			reg.pmc_val,
 			reg.pmc41_mont_reg.darc_cfg_dtag0,
 			reg.pmc41_mont_reg.darc_cfg_dtag1,
@@ -1988,14 +1869,10 @@ pfm_dispatch_ipear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 	pfm_mont_pmc_reg_t reg;
 	pfmlib_mont_input_param_t *param = mod_in;
 	pfmlib_event_t *e = inp->pfp_events;
-	pfmlib_reg_t *pc, *pd;
-	unsigned int pos1, pos2;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
+	int pos = outp->pfp_pmc_count;
 	unsigned int i, count;
 
-	pc = outp->pfp_pmcs;
-	pd = outp->pfp_pmds;
-	pos1 = outp->pfp_pmc_count;
-	pos2 = outp->pfp_pmd_count;
 	/*
 	 * check if there is something to do
 	 */
@@ -2024,33 +1901,20 @@ pfm_dispatch_ipear(pfmlib_input_param_t *inp, pfmlib_mont_input_param_t *mod_in,
 	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 42))
 		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos1].reg_num     = 42;
-	pc[pos1].reg_value   = reg.pmc_val;
-	pc[pos1].reg_addr  = 42;
-	pos1++;
+	pc[pos].reg_num     = 42;
+	pc[pos].reg_pmd_num = PFMLIB_REG_PMD_NONE;
+	pc[pos].reg_evt_idx = PFMLIB_EVENT_NONE;
+	pc[pos++].reg_value = reg.pmc_val;
 
-	__pfm_vbprintf("[PMC42(pmc42)=0x%lx plm=%d pm=%d mode=%d delay=%d]\n",
+	outp->pfp_pmc_count = pos;
+
+	__pfm_vbprintf("[pmc42=0x%lx plm=%d pm=%d mode=%d delay=%d]\n",
 			reg.pmc_val,
 			reg.pmc42_mont_reg.ipear_plm,
 			reg.pmc42_mont_reg.ipear_pm,
 			reg.pmc42_mont_reg.ipear_mode,
 			reg.pmc42_mont_reg.ipear_delay);
 
-	pd[pos2].reg_num = 38;
-	pd[pos2++].reg_addr = 38;
-	pd[pos2].reg_num = 39;
-	pd[pos2++].reg_addr = 39;
-	__pfm_vbprintf("[PMD38(pmd38)]\n[PMD39(pmd39)\n");
-
-	for(i=48; i < 64; i++, pos2++) {
-		pd[pos2].reg_num = i;
-		pd[pos2].reg_addr = i;
-		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[pos2].reg_num, pd[pos2].reg_num);
-	}
-
-	outp->pfp_pmc_count = pos1;
-	outp->pfp_pmd_count = pos2;
-
 	return PFMLIB_SUCCESS;
 }
 
@@ -2217,13 +2081,102 @@ done:
 	return PFMLIB_SUCCESS;
 }
 
+/*
+ * Function used to print information about a specific events. More than
+ * one event can be printed in case an event code is given rather than
+ * a specific name. A callback function is used for printing.
+ */
+static int
+pfm_mont_print_info(unsigned int v, int (*pf)(const char *fmt,...))
+{
+	pme_mont_entry_t *e;
+        static const char *quals[]={ "Instruction Address Range", "OpCode Match", "Data Address Range", "All", "MESI" };
+	static const char *groups[]= {"???", "L1D Cache", "L2D Cache"};
+	long c;
+        int i;
+	char *str;
+
+	if (v >= PME_MONT_EVENT_COUNT || pf == NULL) return PFMLIB_ERR_INVAL;
+	e = montecito_pe+v;
+
+	if (is_iear(e-montecito_pe)) {
+		i= is_ear_tlb(e-montecito_pe) ? 6 : 7;
+	} else {
+		i=3;
+	}
+	(*pf)("Umask  : ");
+	c = e->pme_umask;
+	for (; i >=0; i--) {
+		(*pf)("%c", c & 1<<i ? '1' : '0');
+	}
+	(*pf)("\n");
+
+	(*pf)( "EAR    : %s (%s)\n",
+		event_is_ear(e) ? (event_is_dear(e) ? "Data" : "Inst") : "No",
+		event_is_ear(e) ? (event_is_ear_tlb(e) ? "TLB Mode": (event_is_ear_alat(e) ? "ALAT Mode": "Cache Mode")): "N/A");
+	
+
+	(*pf)("ETB    : %s\n", event_is_etb(e) ? "Yes" : "No");
+	if (e->pme_maxincr > 1)
+		(*pf)("MaxIncr: %u  (Threshold [0-%u])\n", e->pme_maxincr,  e->pme_maxincr-1);
+ 	else
+		(*pf)("MaxIncr: %u  (Threshold 0)\n", e->pme_maxincr);
+
+	(*pf)("Qual   : ");
+
+	c = e->pme_qualifiers.qual;
+	if ((c&0x1f) == 0) {
+		(*pf)("None");
+	} else {
+		for (i=0; i < 5; i++ ) {
+                	if (c & 0x1) (*pf)("[%s] ", quals[i]);
+                	c >>= 1;
+        	}
+	}
+	(*pf)("\n");
+
+	if (e->pme_qualifiers.pme_qual.pme_group == PFMLIB_MONT_EVT_NO_GRP)
+		(*pf)("Group  : None\n");
+	else {
+		unsigned long g = e->pme_qualifiers.pme_qual.pme_group;
+
+		(*pf)("Group  : %s\n", g < 3 ? groups[g]: "none");
+	}
+	if (e->pme_qualifiers.pme_qual.pme_set == PFMLIB_MONT_EVT_NO_SET)
+		(*pf)("Set    : None\n");
+	else
+		(*pf)("Set    : %ld\n", e->pme_qualifiers.pme_qual.pme_set);
+
+	switch(e->pme_caf) {
+		case PFMLIB_MONT_EVT_ACTIVE:
+			str = "Active";
+			break;
+		case PFMLIB_MONT_EVT_FLOATING:
+			str = "Floating";
+			break;
+		case PFMLIB_MONT_EVT_CAUSAL:
+			str = "Causal";
+			break;
+		case PFMLIB_MONT_EVT_SELF_FLOATING:
+			str = "Self-Floating";
+			break;
+		default:
+			str = "??";
+	}
+	(*pf)("Type   : %s\n", str);
+		      
+	if (e->pme_desc) (*pf)("Desc   : %s\n", e->pme_desc);
+
+	return PFMLIB_SUCCESS;
+}
+
 static int
 pfm_mont_get_event_code(unsigned int i, unsigned int cnt, int *code)
 {
-	if (cnt != PFMLIB_CNT_FIRST && (cnt < 4 || cnt > 15))
+	if (cnt != PFMLIB_CNT_FIRST && (cnt < 4 || cnt > 7))
 		return PFMLIB_ERR_INVAL;
 
-	*code = (int)montecito_pe[i].pme_code;
+	*code = montecito_pe[i].pme_code;
 
 	return PFMLIB_SUCCESS;
 }
@@ -2251,12 +2204,20 @@ int
 pfm_mont_get_event_set(unsigned int i, int *set)
 {
 	if (i >= PME_MONT_EVENT_COUNT || set == NULL) return PFMLIB_ERR_INVAL;
-	*set = evt_set(i) == 0xf ? PFMLIB_MONT_EVT_NO_SET : evt_set(i);
+	*set = evt_set(i) == 0xf ? -1 : evt_set(i);
 	return PFMLIB_SUCCESS;
 }
 
 int
 pfm_mont_get_event_type(unsigned int i, int *type)
+{
+	if (i >= PME_MONT_EVENT_COUNT || type == NULL) return PFMLIB_ERR_INVAL;
+	*type = evt_type(i);
+	return PFMLIB_SUCCESS;
+}
+
+int
+pfm_mont_get_event_caf(unsigned int i, int *type)
 {
 	if (i >= PME_MONT_EVENT_COUNT || type == NULL) return PFMLIB_ERR_INVAL;
 	*type = evt_caf(i);
@@ -2316,6 +2277,8 @@ pfm_mont_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 {
 	unsigned int i = 0;
 
+	memset(impl_pmcs, 0, sizeof(*impl_pmcs));
+
 	for(i=0; i < 16; i++)
 		pfm_regmask_set(impl_pmcs, i);
 
@@ -2328,6 +2291,8 @@ pfm_mont_get_impl_pmds(pfmlib_regmask_t *impl_pmds)
 {
 	unsigned int i = 0;
 
+	memset(impl_pmds, 0, sizeof(*impl_pmds));
+
 	for(i=4; i < 16; i++)
 		pfm_regmask_set(impl_pmds, i);
 	for(i=32; i < 40; i++)
@@ -2341,6 +2306,8 @@ pfm_mont_get_impl_counters(pfmlib_regmask_t *impl_counters)
 {
 	unsigned int i = 0;
 
+	memset(impl_counters, 0, sizeof(*impl_counters));
+
 	/* counter pmds are contiguous */
 	for(i=4; i < 16; i++)
 		pfm_regmask_set(impl_counters, i);
@@ -2365,65 +2332,6 @@ pfm_mont_get_event_description(unsigned int ev, char **str)
 	return PFMLIB_SUCCESS;
 }
 
-static int
-pfm_mont_get_cycle_event(pfmlib_event_t *e)
-{
-	e->event = PME_MONT_CPU_OP_CYCLES_ALL;
-	return PFMLIB_SUCCESS;
-
-}
-
-static int
-pfm_mont_get_inst_retired(pfmlib_event_t *e)
-{
-	e->event = PME_MONT_IA64_INST_RETIRED;
-	return PFMLIB_SUCCESS;
-}
-
-static unsigned int
-pfm_mont_get_num_event_masks(unsigned int event)
-{
-	return has_mesi(event) ? 4 : 0;
-}
-
-static char *
-pfm_mont_get_event_mask_name(unsigned int event, unsigned int mask)
-{
-	switch(mask) {
-		case 0: return "I";
-		case 1: return "S";
-		case 2: return "E";
-		case 3: return "M";	
-	}
-	return NULL;
-}
-
-static int
-pfm_mont_get_event_mask_desc(unsigned int event, unsigned int mask, char **desc)
-{
-	switch(mask) {
-		case 0: *desc = strdup("invalid");
-			break;
-		case 1: *desc = strdup("shared");
-			break;
-		case 2: *desc = strdup("exclusive");
-			break;
-		case 3: *desc = strdup("modified");	
-			break;
-		default:
-			return PFMLIB_ERR_INVAL;
-	}
-	return PFMLIB_SUCCESS;
-}
-
-static int
-pfm_mont_get_event_mask_code(unsigned int event,
-		 	     unsigned int mask, unsigned int *code)
-{
-	*code = mask;
-	return PFMLIB_SUCCESS;
-}
-
 pfm_pmu_support_t montecito_support={
 	.pmu_name		= "dual-core Itanium 2",
 	.pmu_type		= PFMLIB_MONTECITO_PMU,
@@ -2431,20 +2339,17 @@ pfm_pmu_support_t montecito_support={
 	.pmc_count		= PMU_MONT_NUM_PMCS,
 	.pmd_count		= PMU_MONT_NUM_PMDS,
 	.num_cnt		= PMU_MONT_NUM_COUNTERS,
+	.cycle_event		= PME_MONT_CPU_OP_CYCLES_ALL,
+	.inst_retired_event	= PME_MONT_IA64_INST_RETIRED,
 	.get_event_code		= pfm_mont_get_event_code,
 	.get_event_name		= pfm_mont_get_event_name,
 	.get_event_counters	= pfm_mont_get_event_counters,
+	.print_info		= pfm_mont_print_info,
 	.dispatch_events	= pfm_mont_dispatch_events,
 	.pmu_detect		= pfm_mont_detect,
 	.get_impl_pmcs		= pfm_mont_get_impl_pmcs,
 	.get_impl_pmds		= pfm_mont_get_impl_pmds,
 	.get_impl_counters	= pfm_mont_get_impl_counters,
 	.get_hw_counter_width	= pfm_mont_get_hw_counter_width,
-	.get_event_desc         = pfm_mont_get_event_description,
-	.get_cycle_event	= pfm_mont_get_cycle_event,
-	.get_inst_retired_event = pfm_mont_get_inst_retired,
-	.get_num_event_masks	= pfm_mont_get_num_event_masks,
-	.get_event_mask_name	= pfm_mont_get_event_mask_name,
-	.get_event_mask_desc	= pfm_mont_get_event_mask_desc,
-	.get_event_mask_code	= pfm_mont_get_event_mask_code
+	.get_event_desc         = pfm_mont_get_event_description
 };
