@@ -206,7 +206,10 @@ pfms_thread_mainloop(void *arg)
 				ctx = td->data;
 				myctx = *ctx;
 
-				ret = fd = pfm_create_context(&myctx, NULL, NULL, 0);
+				ret = pfm_create_context(&myctx, NULL, 0);
+				if (ret == 0)
+					fd = myctx.ctx_fd;
+
 				dprint("CPU%u CMD_CTX ret=%d errno=%d fd=%d\n", mycpu, ret, errno, fd);
 				break;
 
@@ -906,7 +909,7 @@ main(int argc, char **argv)
 	 */
 	for(j=0, k= 0; j < ncpus; j++) {
 		for (i=0; i < inp.pfp_event_count; i++, k++) {
-			pfm_get_full_event_name(&inp.pfp_events[i], name, len);
+			pfm_get_event_name(inp.pfp_events[i].event, name, len);
 			printf("CPU%-3d PMD%u %20"PRIu64" %s\n",
 			j,
 			pd[k].reg_num,
